@@ -8,7 +8,7 @@
 //
 // Copyright (C) 2002 Tim Coleman
 // Portions (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-// Portions (C) 2003 Daniel Morgan
+// Portions (C) 2003,2005 Daniel Morgan
 //
 
 //
@@ -293,6 +293,12 @@ namespace Mono.Data.Tds.Protocol {
 
 					break;
 				case TdsPacketSubType.TableName:
+				//	done = true;
+					peek = Comm.Peek ();
+					done = (peek != (byte) TdsPacketSubType.ColumnDetail);
+
+					break;
+				case TdsPacketSubType.ColumnDetail:
 					done = true;
 					break;
 				default:
@@ -486,12 +492,26 @@ namespace Mono.Data.Tds.Protocol {
 				element = (columnSize != 0);
 				break;
 			case TdsColumnType.UniqueIdentifier :
-				if (comm.Peek () != 16) // If it's null, then what to do?
+				if (comm.Peek () != 16) { // If it's null, then what to do?
+					/*byte swallowed =*/ comm.GetByte();	
+					element = DBNull.Value;
 					break;
-
+				}
 				len = comm.GetByte () & 0xff;
 				if (len > 0) {
 					byte[] guidBytes = comm.GetBytes (len, true);
+					if (!BitConverter.IsLittleEndian) {
+						byte[] swappedguidBytes = new byte[len];
+						for (int i = 0; i < 4; i++)
+							swappedguidBytes[i] = guidBytes[4-i-1];
+						for (int i = 4; i < 6; i++)
+							swappedguidBytes[i] = guidBytes[6-(i-4)-1];
+						for (int i = 6; i < 8; i++)
+							swappedguidBytes[i] = guidBytes[8-(i-6)-1];
+						for (int i = 8; i < 16; i++)
+							swappedguidBytes[i] = guidBytes[i];
+						Array.Copy(swappedguidBytes, 0, guidBytes, 0, len);
+					}
 					element = new Guid (guidBytes);
 				}
 				break;
@@ -552,7 +572,9 @@ namespace Mono.Data.Tds.Protocol {
 				}
 				break;
 			case 4 :
-				result = epoch.AddDays ((int) comm.GetTdsShort ());
+                                // MSDN says small datetime is stored in 2 bytes as no of days
+                                // *after* 1/1/1900. so, cast to unsigned short
+				result = epoch.AddDays ((ushort) comm.GetTdsShort ());
 				short minutes = comm.GetTdsShort ();
 				if (minutes != 0) 
 					result = ((DateTime) result).AddMinutes ((int) minutes);
@@ -565,8 +587,14 @@ namespace Mono.Data.Tds.Protocol {
 			return result;
 		}
 
-		private object GetDecimalValue (byte precision, byte scale)
-		{
+		private object GetDecimalValue (byte precision, byte scale) {
+			if (tdsVersion < TdsVersion.tds70)
+				return GetDecimalValueTds50 (precision, scale);
+			else
+				return GetDecimalValueTds70 (precision, scale);
+		}
+		
+		private object GetDecimalValueTds70 (byte precision, byte scale) {
 			int[] bits = new int[4] {0,0,0,0};
 
 			int len = (comm.GetByte() & 0xff) - 1;
@@ -587,6 +615,43 @@ namespace Mono.Data.Tds.Protocol {
 				return new Decimal (bits[0], bits[1], bits[2], !positive, scale);
 		}
 
+		private object GetDecimalValueTds50 (byte precision, byte scale) {
+			int[] bits = new int[4] {0,0,0,0};
+
+			int len = (comm.GetByte() & 0xff);
+			if (len == 0)
+				return DBNull.Value;
+
+			byte[] dec_bytes=comm.GetBytes(len,false);	
+		
+			byte[] easy=new byte[4];
+
+			bool positive = dec_bytes[0]==1;
+
+			if (len > 17)
+				throw new OverflowException ();
+
+			for (int i = 1, index = 0; i < len && i < 16; i += 
+				4, index += 1) {
+				for(int j=0; j<4; j++)
+					if(i+j<len)
+						easy[j]=dec_bytes[len-
+							(i+j)];
+					else
+						easy[j]=0;
+				if(!BitConverter.IsLittleEndian)
+					easy=comm.Swap(easy);
+				bits[index] = BitConverter.ToInt32(easy,0);
+			}
+			if (bits [3] != 0) 
+				return new TdsBigDecimal (precision, 
+					scale, positive, bits);
+			else
+				return new Decimal(bits[0], bits[1], bits
+					[2], positive, scale);
+			
+		}
+
 		private object GetFloatValue (TdsColumnType columnType)
 		{
 			int columnSize = 0;
@@ -749,8 +814,11 @@ namespace Mono.Data.Tds.Protocol {
 
 			int len = comm.GetTdsInt ();
 
+			//if the len is 0 , then the string can be a '' string 
+			// this method is called only for Text and NText. Hence will
+			// return a empty string
 			if (len == 0)
-				return DBNull.Value;
+				return "";
 
 			if (wideChars)
 				result = comm.GetString (len / 2);
@@ -964,8 +1032,14 @@ namespace Mono.Data.Tds.Protocol {
 
 			if (op == (byte) 0xc1) 
 				rowCount = 0;
-			if (type == TdsPacketSubType.DoneInProc) 
-				rowCount = -1;
+
+                        bool validRowCount = ( (status & 0x10) != 0);
+
+			if (type == TdsPacketSubType.DoneInProc) {
+                                if (validRowCount && rowCount > 0)
+                                        recordsAffected += rowCount;
+                        }
+                        
 
 			moreResults = ((status & 0x01) != 0);
 			bool cancelled = ((status & 0x20) != 0);
@@ -976,8 +1050,9 @@ namespace Mono.Data.Tds.Protocol {
 					goto case TdsPacketSubType.Done;
 
 				case TdsPacketSubType.Done:
-					if (rowCount > 0)
+					if (validRowCount && rowCount > 0)
 						recordsAffected += rowCount;
+
 					break;
 			}
 
@@ -1011,8 +1086,7 @@ namespace Mono.Data.Tds.Protocol {
 			case TdsEnvPacketSubType.CharSet :
 				cLen = comm.GetByte () & 0xff;
 				if (tdsVersion == TdsVersion.tds70) {
-					//this.language = comm.GetString (cLen); // FIXME
-					comm.GetString (cLen);
+					SetCharset (comm.GetString (cLen));
 					comm.Skip (len - 2 - cLen * 2);
 				}
 				else {
@@ -1025,7 +1099,7 @@ namespace Mono.Data.Tds.Protocol {
 				cLen = comm.GetByte () & 0xff;
 				string newDB = comm.GetString (cLen);
 				cLen = comm.GetByte () & 0xff;
-				string oldDB = comm.GetString (cLen);
+				/*string oldDB = */ comm.GetString (cLen);
 				database = newDB;
 				break;
 			default:
@@ -1126,9 +1200,9 @@ namespace Mono.Data.Tds.Protocol {
 		protected void ProcessDynamic ()
 		{
 			Comm.Skip (2);
-			byte type = Comm.GetByte ();
-			byte status = Comm.GetByte ();
-			string id = Comm.GetString (Comm.GetByte ());
+			/*byte type =*/ Comm.GetByte ();
+			/*byte status =*/ Comm.GetByte ();
+			/*string id =*/ Comm.GetString (Comm.GetByte ());
 		}
 
 		protected virtual TdsPacketSubType ProcessSubPacket ()
@@ -1253,5 +1327,118 @@ namespace Mono.Data.Tds.Protocol {
 		}
 
 		#endregion // Private Methods
+
+#if NET_2_0
+                #region asynchronous methods
+                protected IAsyncResult BeginExecuteQueryInternal (string sql, bool wantResults, 
+                                                          AsyncCallback callback, object state)
+                {
+                        moreResults = true;
+			doneProc = false;
+			messages.Clear ();
+			outputParameters.Clear ();
+
+                        TdsAsyncResult ar = new TdsAsyncResult (callback, state);
+                        ar.TdsAsyncState.WantResults = wantResults;
+
+			Comm.StartPacket (TdsPacketType.Query);
+			Comm.Append (sql);
+			Comm.SendPacket ();
+
+                        Comm.BeginReadPacket (new AsyncCallback(OnBeginExecuteQueryCallback), 
+                                                                    ar);
+                        return ar;
+                }
+                
+                protected void EndExecuteQueryInternal (IAsyncResult ar)
+                {
+                        if (!ar.IsCompleted)
+                                ar.AsyncWaitHandle.WaitOne ();
+                        TdsAsyncResult result = (TdsAsyncResult) ar;
+                        if (result.IsCompletedWithException)
+                                throw result.Exception;
+                }
+
+                protected void OnBeginExecuteQueryCallback (IAsyncResult ar)
+                {
+                        TdsAsyncResult result = (TdsAsyncResult) ar.AsyncState;
+                        TdsAsyncState tdsState = (TdsAsyncState) result.TdsAsyncState;
+
+                        try {
+                                Comm.EndReadPacket (ar);
+                                if (!tdsState.WantResults)
+                                        SkipToEnd ();
+                        } catch (Exception e) {
+                                result.MarkComplete (e);
+                                return;
+                        }
+                        result.MarkComplete ();
+                }
+                
+
+                public virtual IAsyncResult BeginExecuteNonQuery (string sql,
+                                                                  TdsMetaParameterCollection parameters,
+                                                                  AsyncCallback callback,
+                                                                  object state)
+                {
+                        // abstract, kept to be backward compatiable.
+                        throw new NotImplementedException ("should not be called!");
+                }
+                
+                public virtual void EndExecuteNonQuery (IAsyncResult ar)
+                {
+                        // abstract method
+                        throw new NotImplementedException ("should not be called!");
+                }
+                
+                public virtual IAsyncResult BeginExecuteQuery (string sql,
+                                                                  TdsMetaParameterCollection parameters,
+                                                                  AsyncCallback callback,
+                                                                  object state)
+                {
+                        // abstract, kept to be backward compatiable.
+                        throw new NotImplementedException ("should not be called!");
+                }
+                
+                public virtual void EndExecuteQuery (IAsyncResult ar)
+                {
+                        // abstract method
+                        throw new NotImplementedException ("should not be called!");
+                }
+
+                public virtual IAsyncResult BeginExecuteProcedure (string prolog,
+                                                                    string epilog,
+                                                                    string cmdText,
+                                                                    bool IsNonQuery,
+                                                                    TdsMetaParameterCollection parameters,
+                                                                    AsyncCallback callback,
+                                                                    object state)
+                {
+                        throw new NotImplementedException ("should not be called!");
+                }
+                
+                public virtual void EndExecuteProcedure (IAsyncResult ar)
+                {
+                        // abstract method
+                        throw new NotImplementedException ("should not be called!");
+                }
+                
+                public void WaitFor (IAsyncResult ar)
+                {
+                        if (! ar.IsCompleted)
+                                ar.AsyncWaitHandle.WaitOne ();
+                }
+
+                public void CheckAndThrowException (IAsyncResult ar)
+                {
+                        TdsAsyncResult result = (TdsAsyncResult) ar;
+                        if (result.IsCompleted && result.IsCompletedWithException)
+                                throw result.Exception;
+                }
+
+                #endregion // asynchronous methods
+#endif // NET_2_0
+
+
 	}
 }
