@@ -144,10 +144,10 @@ namespace Mono.Data.Tds.Protocol {
 					}
 				}
 			}
-			if (count > 0 || exec.Length > 0)
-				exec = "exec " + exec;
-
-			return String.Format ("{0}{1}{2}{3} {4}\n{5}", declare.ToString (), set.ToString (), exec, procedure, BuildParameters (), select.ToString ());	
+                        exec = "exec " + exec;
+                        
+                        string sql = String.Format ("{0}{1}{2}{3} {4}\n{5}", declare.ToString (), set.ToString (), exec, procedure, BuildParameters (), select.ToString ());
+			return sql;
 		}
 
 		public override bool Connect (TdsConnectionParameters connectionParameters)
@@ -185,7 +185,7 @@ namespace Mono.Data.Tds.Protocol {
 			domain = connectionParameters.DefaultDomain = Environment.UserDomainName;
 
 			int idx = 0;
-			if ((idx = username.IndexOf (@"\")) > -1) {
+			if ((idx = username.IndexOf ("\\")) > -1) {
 				domain = username.Substring (0, idx);
 				username = username.Substring (idx + 1);
 
@@ -353,16 +353,19 @@ namespace Mono.Data.Tds.Protocol {
 
 		public override bool Reset ()
 		{
-			try
-			{
+			try {
 				ExecProc ("exec sp_reset_connection");
-				return true;
-			}
-			catch
-			{
-				Console.WriteLine ("Error reseting");
-				return false;
+			} catch (Exception e) {
+				System.Reflection.PropertyInfo pinfo = e.GetType ().GetProperty ("Class");
+				if (pinfo != null && pinfo.PropertyType == typeof (byte)) {
+					byte klass = (byte) pinfo.GetValue (e, null);
+					// 11 to 16 indicates error that can be fixed by the user such as 'Invalid object name'
+					if (klass < 11 || klass > 16)
+						return false;
+				}
 			}
+
+			return true;
 		}
 
 		public override void ExecPrepared (string commandText, TdsMetaParameterCollection parameters, int timeout, bool wantResults)
@@ -406,6 +409,12 @@ namespace Mono.Data.Tds.Protocol {
 
 			string value = null;
 			switch (parameter.TypeName) {
+                        case "smalldatetime":
+			case "datetime":
+				DateTime d = (DateTime)parameter.Value;
+				value = String.Format(System.Globalization.CultureInfo.InvariantCulture, 
+                                                      "'{0:MMM dd yyyy hh:mm:ss tt}'", d );
+                                break;
 			case "bigint":
 			case "decimal":
 			case "float":
@@ -415,7 +424,12 @@ namespace Mono.Data.Tds.Protocol {
 			case "smallint":
 			case "smallmoney":
 			case "tinyint":
-				value = parameter.Value.ToString ();
+				object paramValue = parameter.Value;
+				Type paramType = paramValue.GetType ();
+				if (paramType.IsEnum)
+					paramValue = Convert.ChangeType (paramValue,
+						Type.GetTypeCode (paramType));
+				value = paramValue.ToString ();
 				break;
 			case "nvarchar":
 			case "nchar":
@@ -546,5 +560,75 @@ namespace Mono.Data.Tds.Protocol {
 		}
 
 		#endregion // Methods
+
+#if NET_2_0
+                #region Asynchronous Methods
+                public override IAsyncResult BeginExecuteNonQuery (string cmdText,
+                                                          TdsMetaParameterCollection parameters,
+                                                          AsyncCallback callback,
+                                                          object state)
+                {
+                        Parameters = parameters;
+                        string sql = cmdText;
+			if (Parameters != null && Parameters.Count > 0)
+				sql = BuildExec (cmdText);
+
+                        IAsyncResult ar = BeginExecuteQueryInternal (sql, false, callback, state);
+                        return ar;
+                }
+
+                public override void EndExecuteNonQuery (IAsyncResult ar)
+                {
+                        EndExecuteQueryInternal (ar);
+                }
+
+                public override IAsyncResult BeginExecuteQuery (string cmdText,
+                                                                TdsMetaParameterCollection parameters,
+                                                                AsyncCallback callback,
+                                                                object state)
+                {
+                        Parameters = parameters;
+                        string sql = cmdText;
+			if (Parameters != null && Parameters.Count > 0)
+				sql = BuildExec (cmdText);
+
+                        IAsyncResult ar = BeginExecuteQueryInternal (sql, true, callback, state);
+                        return ar;
+                }
+
+                public override void EndExecuteQuery (IAsyncResult ar)
+                {
+                        EndExecuteQueryInternal (ar);
+                }
+
+
+                public override IAsyncResult BeginExecuteProcedure (string prolog,
+                                                                    string epilog,
+                                                                    string cmdText,
+                                                                    bool IsNonQuery,
+                                                                    TdsMetaParameterCollection parameters,
+                                                                    AsyncCallback callback,
+                                                                    object state)
+                {
+
+                        
+                        Parameters = parameters;
+			string pcall = BuildProcedureCall (cmdText);
+                        string sql = String.Format ("{0};{1};{2};", prolog, pcall, epilog);
+
+                        IAsyncResult ar = BeginExecuteQueryInternal (sql, !IsNonQuery, callback, state);
+                        return ar;
+                }
+
+                public override void EndExecuteProcedure (IAsyncResult ar)
+                {
+                        EndExecuteQueryInternal (ar);
+                }
+
+
+
+                #endregion // Asynchronous Methods
+#endif // NET_2_0
+
 	}
 }
