@@ -133,6 +133,13 @@ namespace Mono.Data.Tds.Protocol {
 		
 		#region Methods
 
+		public byte[] Swap(byte[] toswap) {
+			byte[] ret = new byte[toswap.Length];
+			for(int i = 0; i < toswap.Length; i++)
+				ret [toswap.Length - i - 1] = toswap[i];
+
+			return ret;
+		}
 		public void Append (object o)
 		{
 			switch (o.GetType ().ToString ()) {
@@ -187,12 +194,18 @@ namespace Mono.Data.Tds.Protocol {
 
 		public void Append (short s)
 		{
-			Append (BitConverter.GetBytes (s));
+			if(!BitConverter.IsLittleEndian)
+				Append (Swap (BitConverter.GetBytes(s)));
+			else 
+				Append (BitConverter.GetBytes (s));
 		}
 
 		public void Append (int i)
 		{
-			Append (BitConverter.GetBytes (i));
+			if(!BitConverter.IsLittleEndian)
+				Append (Swap (BitConverter.GetBytes(i)));
+			else
+				Append (BitConverter.GetBytes (i));
 		}
 
 		public void Append (string s)
@@ -201,7 +214,10 @@ namespace Mono.Data.Tds.Protocol {
 				Append (encoder.GetBytes (s));
 			else 
 				foreach (char c in s)
-					Append (BitConverter.GetBytes (c));
+					if(!BitConverter.IsLittleEndian)
+						Append (Swap (BitConverter.GetBytes (c)));
+					else
+						Append (BitConverter.GetBytes (c));
 		}	
 
 		// Appends with padding
@@ -233,7 +249,10 @@ namespace Mono.Data.Tds.Protocol {
 				Append ((byte) (((byte) (l >> 0)) & 0xff));
 			}
 			else 
-				Append (BitConverter.GetBytes (l));
+				if (!BitConverter.IsLittleEndian)
+					Append (Swap (BitConverter.GetBytes (l)));
+				else
+					Append (BitConverter.GetBytes (l));
 		}
 
 		public void Close ()
@@ -335,8 +354,10 @@ namespace Mono.Data.Tds.Protocol {
 
 			for (int i = 0; i < 2; i += 1)
 				input[i] = GetByte ();
-
-			return (BitConverter.ToInt16 (input, 0));
+			if(!BitConverter.IsLittleEndian)
+				return (BitConverter.ToInt16 (Swap (input), 0));
+			else
+				return (BitConverter.ToInt16 (input, 0));
 		}
 
 
@@ -345,7 +366,10 @@ namespace Mono.Data.Tds.Protocol {
 			byte[] input = new byte[4];
 			for (int i = 0; i < 4; i += 1)
 				input[i] = GetByte ();
-			return (BitConverter.ToInt32 (input, 0));
+			if(!BitConverter.IsLittleEndian)
+				return (BitConverter.ToInt32 (Swap (input), 0));
+			else
+				return (BitConverter.ToInt32 (input, 0));
 		}
 
 		public long GetTdsInt64 ()
@@ -353,13 +377,22 @@ namespace Mono.Data.Tds.Protocol {
 			byte[] input = new byte[8];
 			for (int i = 0; i < 8; i += 1)
 				input[i] = GetByte ();
-			return (BitConverter.ToInt64 (input, 0));
+			if(!BitConverter.IsLittleEndian)
+				return (BitConverter.ToInt64 (Swap (input), 0));
+			else
+				return (BitConverter.ToInt64 (input, 0));
 		}
 
 		private void GetPhysicalPacket ()
 		{
-			int nread = 0;
+                        int dataLength = GetPhysicalPacketHeader ();
+                        GetPhysicalPacketData (dataLength);
+		}
 
+                private int GetPhysicalPacketHeader ()
+                {
+                        int nread = 0;
+                                                
 			// read the header
 			while (nread < 8)
 				nread += stream.Read (tmpBuf, nread, 8 - nread);
@@ -379,19 +412,26 @@ namespace Mono.Data.Tds.Protocol {
 			if (len < 0) {
 				throw new Exception (String.Format ("Confused by a length of {0}", len));
 			}
+                        
+                        return len;
 
-			// now get the data
-			nread = 0;
-			while (nread < len) {
-				nread += stream.Read (inBuffer, nread, len - nread);
+                }
+                
+                private void GetPhysicalPacketData (int length)
+                {
+                        // now get the data
+			int nread = 0;
+			while (nread < length) {
+				nread += stream.Read (inBuffer, nread, length - nread);
 			}
 
 			packetsReceived++;
 
 			// adjust the bookkeeping info about the incoming buffer
-			inBufferLength = len;
+			inBufferLength = length;
 			inBufferIndex = 0;
-		}
+                }
+                
 
 		private static int Ntohs (byte[] buf, int offset)
 		{
@@ -492,6 +532,59 @@ namespace Mono.Data.Tds.Protocol {
 		}
 
 		#endregion // Methods
+#if NET_2_0
+                #region Async Methods
+
+                public IAsyncResult BeginReadPacket (AsyncCallback callback, object stateObject)
+		{
+                        TdsAsyncResult ar = new TdsAsyncResult (callback, stateObject);
+
+                        stream.BeginRead (tmpBuf, 0, 8, new AsyncCallback(OnReadPacketCallback), ar);
+                        return ar;
+		}
+                
+                /// <returns>Packet size in bytes</returns>
+                public int EndReadPacket (IAsyncResult ar)
+                {
+                        if (!ar.IsCompleted)
+                                ar.AsyncWaitHandle.WaitOne ();
+                        return (int) ((TdsAsyncResult) ar).ReturnValue;
+                }
+                
+
+                public void OnReadPacketCallback (IAsyncResult socketAsyncResult)
+                {
+                        TdsAsyncResult ar = (TdsAsyncResult) socketAsyncResult.AsyncState;
+                        int nread = stream.EndRead (socketAsyncResult);
+                        
+                        while (nread < 8)
+                                nread += stream.Read (tmpBuf, nread, 8 - nread);
+
+			TdsPacketType packetType = (TdsPacketType) tmpBuf[0];
+			if (packetType != TdsPacketType.Logon && packetType != TdsPacketType.Query && packetType != TdsPacketType.Reply) 
+			{
+				throw new Exception (String.Format ("Unknown packet type {0}", tmpBuf[0]));
+			}
+
+			// figure out how many bytes are remaining in this packet.
+			int len = Ntohs (tmpBuf, 2) - 8;
+
+			if (len >= inBuffer.Length) 
+				inBuffer = new byte[len];
+
+			if (len < 0) {
+				throw new Exception (String.Format ("Confused by a length of {0}", len));
+			}
+
+                        GetPhysicalPacketData (len);
+                        int value = len + 8;
+                        ar.ReturnValue = ((object)value); // packet size
+                        ar.MarkComplete ();
+                }
+                
+                #endregion // Async Methods
+#endif // NET_2_0
+
 	}
 
 }
