@@ -21,7 +21,7 @@
 //    use a set of C helper routines to map from the constants we define
 //    to the values of the native OS.
 //
-//    Bitfields are flagged with the [Map] attribute, and a helper program
+//    Bitfields were flagged with the [Map] attribute, and a helper program
 //    generates a set of map_XXXX routines that we can call to convert
 //    from our value definitions to the value definitions expected by the
 //    OS.
@@ -29,6 +29,17 @@
 //    Methods that require tuning are bound as `internal syscal_NAME' methods
 //    and then a `NAME' method is exposed.
 //
+// Deprecated Warning:
+//
+//    This class is deprecated, and exists only for backward compatibility.
+//    Please use and maintain Mono.Unix.Syscall.
+//
+//    The [Map] attributes have been removed.  The naming and methodology of
+//    the mapping routines has changed.  The old map functions still exist in
+//    MonoPosixHelper, but they will not be updated any further.
+//    Consequently, there is little point in maintaining the [Map] attributes
+//    in this file, as they would only bloat MonoPosixHelper.
+//
 
 //
 // Permission is hereby granted, free of charge, to any person obtaining
@@ -55,11 +66,9 @@ using System;
 using System.Text;
 using System.Runtime.InteropServices;
 
-//[assembly:Mono.Posix.IncludeAttribute (new string [] {"sys/types.h", "sys/stat.h", "sys/wait.h", "unistd.h", "fcntl.h", "signal.h"},
-				       //new string [] {"_GNU_SOURCE"})]
 namespace Mono.Posix {
 
-	[Map][Flags]
+	[Flags]
 	public enum OpenFlags {
 		//
 		// One of these
@@ -91,7 +100,7 @@ namespace Mono.Posix {
 		// O_LARGEFILE = 8192
 	}
 	
-	[Flags][Map]
+	[Flags]
 	public enum FileMode {
 		S_ISUID   = 2048,
 		S_ISGID   = 1024,
@@ -107,13 +116,13 @@ namespace Mono.Posix {
 		S_IXOTH   = 1
 	}
 
-	[Flags][Map]
+	[Flags]
 	public enum WaitOptions {
 		WNOHANG,
 		WUNTRACED
 	}
 
-	[Flags][Map]
+	[Flags]
 	public enum AccessMode {
 		R_OK = 1,
 		W_OK = 2,
@@ -121,7 +130,7 @@ namespace Mono.Posix {
 		F_OK = 8
 	}
 
-	[Map]
+	
 	public enum Signals {
 		SIGHUP, SIGINT, SIGQUIT, SIGILL, SIGTRAP, SIGABRT, SIGBUS,
 		SIGFPE, SIGKILL, SIGUSR1, SIGSEGV, SIGUSR2, SIGPIPE,
@@ -134,6 +143,7 @@ namespace Mono.Posix {
 		// SIGRTMIN
 	}
 	
+	[Obsolete ("Syscall is unmaintained.  Please use Mono.Unix.Syscall.")]
 	public class Syscall {
 		[DllImport ("libc", SetLastError=true)]
 		public static extern int exit (int status);
@@ -212,7 +222,7 @@ namespace Mono.Posix {
 		[DllImport ("MonoPosixHelper", EntryPoint="wstopsig")]
 		public static extern int WSTOPSIG (int status);
 
-		[DllImport ("libc", SetLastError=true)]
+		[DllImport ("libc", EntryPoint="creat", SetLastError=true)]
 		internal static extern int syscall_creat (string pathname, int flags);
 
 		public static int creat (string pathname, FileMode flags)
@@ -277,7 +287,7 @@ namespace Mono.Posix {
 
 		// TODO: utime
 
-		[DllImport ("libc")]
+		[DllImport ("libc", EntryPoint="access", SetLastError=true)]
 		internal extern static int syscall_access (string pathname, int mode);
 
 		[DllImport ("MonoPosixHelper")]
@@ -302,7 +312,7 @@ namespace Mono.Posix {
 		[DllImport ("libc", SetLastError=true)]
 		public static extern int rename (string oldPath, string newPath);
 		
-		[DllImport ("libc", SetLastError=true)]
+		[DllImport ("libc", EntryPoint="mkdir", SetLastError=true)]
 		internal extern static int syscall_mkdir (string pathname, int mode);
 
 		public static int mkdir (string pathname, FileMode mode)
@@ -538,10 +548,10 @@ namespace Mono.Posix {
 		public readonly DateTime MTime;
 		public readonly DateTime CTime;
 		
-		public static readonly DateTime UnixEpoch = new DateTime(1970, 1, 1).ToLocalTime();
+		public static readonly DateTime UnixEpoch = new DateTime(1970, 1, 1);
 		
 		public static DateTime UnixToDateTime(long unix) {
-			return UnixEpoch.Add(TimeSpan.FromSeconds(unix));
+			return UnixEpoch.Add(TimeSpan.FromSeconds(unix)).ToLocalTime();
 		}
 
 		internal Stat(
