@@ -1,3 +1,5 @@
+// Transport Security Layer (TLS)
+// Copyright (c) 2003-2004 Carlos Guzman Alvarez
 
 //
 // Permission is hereby granted, free of charge, to any person obtaining
@@ -19,29 +21,6 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-/* Transport Security Layer (TLS)
- * Copyright (c) 2003-2004 Carlos Guzman Alvarez
- * 
- * Permission is hereby granted, free of charge, to any person 
- * obtaining a copy of this software and associated documentation 
- * files (the "Software"), to deal in the Software without restriction, 
- * including without limitation the rights to use, copy, modify, merge, 
- * publish, distribute, sublicense, and/or sell copies of the Software, 
- * and to permit persons to whom the Software is furnished to do so, 
- * subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included 
- * in all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
- * DEALINGS IN THE SOFTWARE.
- */
 
 using System;
 using System.IO;
@@ -200,9 +179,9 @@ namespace Mono.Security.Protocol.Tls
 			get { return this.expandedKeyMaterialSize; }
 		}
 
-		public byte	EffectiveKeyBits
+		public short	EffectiveKeyBits
 		{
-			get { return this.EffectiveKeyBits; }
+			get { return this.effectiveKeyBits; }
 		}
 		
 		public byte IvSize
@@ -234,7 +213,7 @@ namespace Mono.Security.Protocol.Tls
 			short code, string name, CipherAlgorithmType cipherAlgorithmType, 
 			HashAlgorithmType hashAlgorithmType, ExchangeAlgorithmType exchangeAlgorithmType,
 			bool exportable, bool blockMode, byte keyMaterialSize, 
-			byte expandedKeyMaterialSize, short effectiveKeyBytes, 
+			byte expandedKeyMaterialSize, short effectiveKeyBits, 
 			byte ivSize, byte blockSize)
 		{
 			this.code					= code;
@@ -391,9 +370,6 @@ namespace Mono.Security.Protocol.Tls
 
 		public byte[] PRF(byte[] secret, string label, byte[] data, int length)
 		{
-			HashAlgorithm md5	= MD5.Create();
-			HashAlgorithm sha1	= SHA1.Create();
-
 			/* Secret Length calc exmplain from the RFC2246. Section 5
 			 * 
 			 * S1 and S2 are the two halves of the secret and each is the same
@@ -614,4 +590,4 @@ namespace Mono.Security.Protocol.Tls
 
 		#endregion
 	}
-}
\ No newline at end of file
+}
