@@ -1,3 +1,5 @@
+// Transport Security Layer (TLS)
+// Copyright (c) 2003-2004 Carlos Guzman Alvarez
 
 //
 // Permission is hereby granted, free of charge, to any person obtaining
@@ -19,29 +21,6 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-/* Transport Security Layer (TLS)
- * Copyright (c) 2003-2004 Carlos Guzman Alvarez
- * 
- * Permission is hereby granted, free of charge, to any person 
- * obtaining a copy of this software and associated documentation 
- * files (the "Software"), to deal in the Software without restriction, 
- * including without limitation the rights to use, copy, modify, merge, 
- * publish, distribute, sublicense, and/or sell copies of the Software, 
- * and to permit persons to whom the Software is furnished to do so, 
- * subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included 
- * in all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
- * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
- * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
- * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
- * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
- * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
- * DEALINGS IN THE SOFTWARE.
- */
 
 using System;
 using System.Collections;
@@ -423,7 +402,7 @@ namespace Mono.Security.Protocol.Tls
 						{
 							// Read next record and write it into the inputBuffer
 							long	position	= this.inputBuffer.Position;					
-							byte[]	record		= this.protocol.ReceiveRecord();
+							byte[]	record		= this.protocol.ReceiveRecord(this.innerStream);
 					
 							if (record != null && record.Length > 0)
 							{
@@ -671,7 +650,7 @@ namespace Mono.Security.Protocol.Tls
 				this.context.HandshakeState = HandshakeState.Started;
 
 				// Receive Client Hello message
-				this.protocol.ReceiveRecord();
+				this.protocol.ReceiveRecord (this.innerStream);
 
 				// If received message is not an ClientHello send a
 				// Fatal Alert
@@ -707,7 +686,7 @@ namespace Mono.Security.Protocol.Tls
 				// is received
 				while (this.context.LastHandshakeMsg != HandshakeType.Finished)
 				{
-					this.protocol.ReceiveRecord();
+					this.protocol.ReceiveRecord (this.innerStream);
 				}
 				
 				// Send ChangeCipherSpec and ServerFinished messages
