@@ -26,15 +26,15 @@
 using System;
 using System.Data;
 using System.Collections;
-using NpgsqlTypes;
 
+using NpgsqlTypes;
 
 namespace Npgsql
 {
     /// <summary>
     /// Provides a means of reading a forward-only stream of rows from a PostgreSQL backend.  This class cannot be inherited.
     /// </summary>
-    public class NpgsqlDataReader : IDataReader, IEnumerable
+    public sealed class NpgsqlDataReader : IDataReader, IEnumerable
     {
         private NpgsqlConnection 	_connection;
         private ArrayList 			_resultsets;
@@ -77,7 +77,8 @@ namespace Npgsql
 
         private void CheckHaveResultSet()
         {
-            if (! HaveResultSet()) {
+            if (! HaveResultSet())
+            {
                 throw new InvalidOperationException("Cannot read data. No result set.");
             }
         }
@@ -86,9 +87,12 @@ namespace Npgsql
         {
             CheckHaveResultSet();
 
-            if (_rowIndex < 0) {
+            if (_rowIndex < 0)
+            {
                 throw new InvalidOperationException("DataReader positioned before beginning of result set. Did you call Read()?");
-            } else if (_rowIndex >= _currentResultset.Count) {
+            }
+            else if (_rowIndex >= _currentResultset.Count)
+            {
                 throw new InvalidOperationException("DataReader positioned beyond end of result set.");
             }
         }
@@ -147,21 +151,37 @@ namespace Npgsql
             {
                 NpgsqlEventLog.LogPropertyGet(LogLevel.Debug, CLASSNAME, "RecordsAffected");
 
-                if (HaveResultSet()) {
+                if (HaveResultSet())
+                {
                     return -1;
                 }
 
                 String[] _returnStringTokens = ((String)_responses[_resultsetIndex]).Split(null);	// whitespace separator.
 
-                try {
+                try
+                {
                     return Int32.Parse(_returnStringTokens[_returnStringTokens.Length - 1]);
                 }
-                catch (FormatException) {
+                catch (FormatException)
+                {
                     return -1;
                 }
             }
         }
 
+        /// <summary>
+        /// Indicates if NpgsqlDatareader has rows to be read.
+        /// </summary>
+
+        public Boolean HasRows
+        {
+            get
+            {
+                return _currentResultset.Count > 0;
+            }
+
+        }
+
         /// <summary>
         /// Closes the data reader object.
         /// </summary>
@@ -170,8 +190,10 @@ namespace Npgsql
             if ((_behavior & CommandBehavior.CloseConnection) == CommandBehavior.CloseConnection)
             {
                 _connection.Close();
-                _isClosed = true;
+
             }
+
+            _isClosed = true;
         }
 
         /// <summary>
@@ -202,12 +224,16 @@ namespace Npgsql
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "Read");
 
-            CheckHaveResultSet();
+            if (!HaveResultSet())
+	    	return false;
 
-            if (_rowIndex < _currentResultset.Count) {
+            if (_rowIndex < _currentResultset.Count)
+            {
                 _rowIndex++;
                 return (_rowIndex < _currentResultset.Count);
-            } else {
+            }
+            else
+            {
                 return false;
             }
         }
@@ -252,10 +278,23 @@ namespace Npgsql
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetName");
 
-            if (! HaveResultSet())
-                return String.Empty;
-            else
-                return _currentResultset.RowDescription[Index].name;
+            CheckHaveResultSet();
+
+            return _currentResultset.RowDescription[Index].name;
+        }
+
+        /// <summary>
+        /// Return the data type OID of the column at index <param name="Index"></param>.
+        /// </summary>
+        public String GetDataTypeOID(Int32 Index)
+        {
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetDataTypeName");
+
+            CheckHaveResultSet();
+
+            NpgsqlBackendTypeInfo  TI = GetTypeInfo(Index);
+
+            return _currentResultset.RowDescription[Index].type_oid.ToString();
         }
 
         /// <summary>
@@ -263,11 +302,20 @@ namespace Npgsql
         /// </summary>
         public String GetDataTypeName(Int32 Index)
         {
-            // FIXME: have a type name instead of the oid
-            if (! HaveResultSet())
-                return String.Empty;
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetDataTypeName");
+
+            CheckHaveResultSet();
+
+            NpgsqlBackendTypeInfo  TI = GetTypeInfo(Index);
+
+            if (TI == null)
+            {
+                return _currentResultset.RowDescription[Index].type_oid.ToString();
+            }
             else
-                return (_currentResultset.RowDescription[Index].type_oid).ToString();
+            {
+                return TI.Name;
+            }
         }
 
         /// <summary>
@@ -277,13 +325,65 @@ namespace Npgsql
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetFieldType");
 
+            CheckHaveResultSet();
 
-            if (! HaveResultSet())
-                return null;
+            NpgsqlBackendTypeInfo  TI = GetTypeInfo(Index);
+
+            if (TI == null)
+            {
+                return typeof(String);  //Default type is string.
+            }
             else
-                return NpgsqlTypesHelper.GetSystemTypeFromTypeOid(_connection.OidToNameMapping, _currentResultset.RowDescription[Index].type_oid);
+            {
+                return TI.Type;
+            }
         }
 
+        /// <summary>
+        /// Return the data DbType of the column at index <param name="Index"></param>.
+        /// </summary>
+        public DbType GetFieldDbType(Int32 Index)
+        {
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetFieldType");
+
+            CheckHaveResultSet();
+
+            NpgsqlBackendTypeInfo  TI = GetTypeInfo(Index);
+
+            if (TI == null)
+            {
+                return DbType.String;
+            }
+            else
+            {
+                //return TI.DBType;
+                return DbType.String;
+            }
+        }
+
+        /// <summary>
+        /// Return the data NpgsqlDbType of the column at index <param name="Index"></param>.
+        /// </summary>
+        public NpgsqlDbType GetFieldNpgsqlDbType(Int32 Index)
+        {
+            NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetFieldType");
+
+            CheckHaveResultSet();
+
+            NpgsqlBackendTypeInfo  TI = GetTypeInfo(Index);
+
+            if (TI == null)
+            {
+                return NpgsqlDbType.Text;
+            }
+            else
+            {
+                return TI.NpgsqlDbType;
+
+            }
+        }
+
+
         /// <summary>
         /// Return the value of the column at index <param name="Index"></param>.
         /// </summary>
@@ -291,7 +391,8 @@ namespace Npgsql
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetValue");
 
-            if (Index < 0 || Index >= _currentResultset.RowDescription.NumFields) {
+            if (Index < 0 || Index >= _currentResultset.RowDescription.NumFields)
+            {
                 throw new IndexOutOfRangeException("Column index out of range");
             }
 
@@ -314,7 +415,8 @@ namespace Npgsql
             // It's also possible to pass an array with more that FieldCount elements.
             Int32 maxColumnIndex = (Values.Length < FieldCount) ? Values.Length : FieldCount;
 
-            for (Int32 i = 0; i < maxColumnIndex; i++) {
+            for (Int32 i = 0; i < maxColumnIndex; i++)
+            {
                 Values[i] = GetValue(i);
             }
 
@@ -356,7 +458,7 @@ namespace Npgsql
         }
 
         /// <summary>
-        /// Gets the value of a column converted to a Boolean.
+        /// Gets the value of a column as Boolean.
         /// </summary>
         public Boolean GetBoolean(Int32 i)
         {
@@ -364,11 +466,11 @@ namespace Npgsql
             // and parsing from there?
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetBoolean");
 
-            return Convert.ToBoolean(GetValue(i));
+            return (Boolean) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to a Byte.  Not implemented.
+        /// Gets the value of a column as Byte.  Not implemented.
         /// </summary>
         public Byte GetByte(Int32 i)
         {
@@ -385,18 +487,27 @@ namespace Npgsql
 
             result = (Byte[]) GetValue(i);
 
+            if (buffer == null)
+                return result.Length;
+
+
+            // We just support read all the field for while. So, any fieldOffset value other than 0 will not read
+            // anything and return 0.
+
+            if (fieldOffset != 0)
+                return 0;
+
             // [TODO] Implement blob support.
-            if (buffer != null)
-            {
-                result.CopyTo(buffer, 0);
-            }
+
+            result.CopyTo(buffer, 0);
+
 
             return result.Length;
 
         }
 
         /// <summary>
-        /// Gets the value of a column converted to a Char.  Not implemented.
+        /// Gets the value of a column as Char.  Not implemented.
         /// </summary>
         public Char GetChar(Int32 i)
         {
@@ -427,83 +538,83 @@ namespace Npgsql
         }
 
         /// <summary>
-        /// Gets the value of a column converted to Int16.
+        /// Gets the value of a column as Int16.
         /// </summary>
         public Int16 GetInt16(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetInt16");
 
-            return Convert.ToInt16(GetValue(i));
+            return (Int16) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to Int32.
+        /// Gets the value of a column as Int32.
         /// </summary>
         public Int32 GetInt32(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetInt32");
 
-            return Convert.ToInt32(GetValue(i));
+            return (Int32) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to Int64.
+        /// Gets the value of a column as Int64.
         /// </summary>
         public Int64 GetInt64(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetInt64");
 
-            return Convert.ToInt64(GetValue(i));
+            return (Int64) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to Single.
+        /// Gets the value of a column as Single.
         /// </summary>
         public Single GetFloat(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetFloat");
 
-            return Convert.ToSingle(GetValue(i));
+            return (Single) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to Double.
+        /// Gets the value of a column as Double.
         /// </summary>
         public Double GetDouble(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetDouble");
 
-            return Convert.ToDouble(GetValue(i));
+            return (Double) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to a String.
+        /// Gets the value of a column as String.
         /// </summary>
         public String GetString(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetString");
 
-            return Convert.ToString(GetValue(i));
+            return (String) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to Decimal.
+        /// Gets the value of a column as Decimal.
         /// </summary>
         public Decimal GetDecimal(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetDecimal");
 
-            return Convert.ToDecimal(GetValue(i));
+            return (Decimal) GetValue(i);
         }
 
         /// <summary>
-        /// Gets the value of a column converted to a DateTime.
+        /// Gets the value of a column as DateTime.
         /// </summary>
         public DateTime GetDateTime(Int32 i)
         {
             NpgsqlEventLog.LogMethodEnter(LogLevel.Debug, CLASSNAME, "GetDateTime");
 
-            return Convert.ToDateTime(GetValue(i));
+            return (DateTime) GetValue(i);
         }
 
         /// <summary>
@@ -524,6 +635,11 @@ namespace Npgsql
             return (GetValue(i) == DBNull.Value);
         }
 
+        internal NpgsqlBackendTypeInfo GetTypeInfo(Int32 FieldIndex)
+        {
+            return _currentResultset.RowDescription[FieldIndex].type_info;
+        }
+
         private DataTable GetResultsetSchema()
         {
 
