@@ -9,10 +9,12 @@
 // Assembly: System.Data.OracleClient.dll
 // Namespace: System.Data.OracleClient.Oci
 // 
-// Author: 
+// Authors: 
 //     Tim Coleman <tim@timcoleman.com>
+//     Daniel Morgan <danielmorgan@verizon.net>
 //         
 // Copyright (C) Tim Coleman, 2003
+// Copyright (C) Daniel Morgan, 2004
 // 
 
 using System;
@@ -20,7 +22,8 @@ using System.Data.OracleClient;
 using System.Runtime.InteropServices;
 using System.Text;
 
-namespace System.Data.OracleClient.Oci {
+namespace System.Data.OracleClient.Oci 
+{
 	internal sealed class OciDefineHandle : OciHandle, IDisposable
 	{
 		#region Fields
@@ -34,8 +37,16 @@ namespace System.Data.OracleClient.Oci {
 		OciDataType ociType;
 		OciDataType definedType;
 		int definedSize;
-		int rlenp;
+		short rlenp = 0;
+		short precision;
 		short scale;
+		Type fieldType;
+		string name;
+
+		// Oracle defines the LONG VARCHAR have a size of 2 to the 31 power - 5
+		// maybe this should settable via a config file for System.Data.OracleClient.dll
+		// see DefineLong
+		internal static int LongVarCharMaxValue = (int) Int16.MaxValue - 5;
 
 		OciErrorHandle errorHandle;
 
@@ -54,10 +65,15 @@ namespace System.Data.OracleClient.Oci {
 		public void DefineByPosition (int position)
 		{
 			OciParameterDescriptor parameter = ((OciStatementHandle) Parent).GetParameter (position);
-			definedSize = parameter.GetDataSize ();
-			scale = parameter.GetScale ();
+			
+			name = parameter.GetName ();
 			definedType = parameter.GetDataType ();
+			definedSize = parameter.GetDataSize ();
+			precision = parameter.GetPrecision ();
+			scale = parameter.GetScale ();
+
 			Define (position);
+
 			parameter.Dispose ();
 		}
 
@@ -69,6 +85,10 @@ namespace System.Data.OracleClient.Oci {
 			get { return definedType; }
 		}
 
+		public Type FieldType {
+			get { return fieldType; }
+		}
+
 		public int DefinedSize {
 			get { return definedSize; }
 		}
@@ -86,7 +106,7 @@ namespace System.Data.OracleClient.Oci {
 			get { return scale; }
 		}
 
-		public int Size {
+		public short Size {
 			get { return rlenp; }
 		}
 
@@ -102,17 +122,28 @@ namespace System.Data.OracleClient.Oci {
 		{
 			switch (definedType) {
 			case OciDataType.Date:
-				definedSize = 7;
 				DefineDate (position); 
 				return;
 			case OciDataType.Clob:
 			case OciDataType.Blob:
-				definedSize = -1;
 				DefineLob (position, definedType);
 				return;
 			case OciDataType.Raw:
 				DefineRaw( position);
 				return;
+			case OciDataType.RowIdDescriptor:
+				definedSize = 10;
+				DefineChar (position);
+				return;
+			case OciDataType.Integer:
+			case OciDataType.Number:
+			case OciDataType.Float:
+				DefineNumber (position);
+				return;
+			case OciDataType.Long:
+			case OciDataType.LongVarChar:
+				DefineLong (position);
+				return;
 			default:
 				DefineChar (position); // HANDLE ALL OTHERS AS CHAR FOR NOW
 				return;
@@ -121,7 +152,10 @@ namespace System.Data.OracleClient.Oci {
 
 		void DefineDate (int position)
 		{
+			definedSize = 7;
 			ociType = OciDataType.Date;
+			fieldType = typeof(System.DateTime);
+
 			value = Marshal.AllocHGlobal (definedSize);
 
 			int status = 0;
@@ -144,16 +178,56 @@ namespace System.Data.OracleClient.Oci {
 			}
 		}
 
+		void DefineLong (int position) 
+		{
+			fieldType = typeof (System.String);
+
+			// LONG VARCHAR max length is 2 to the 31 power - 5
+			// the first 4 bytes of a LONG VARCHAR value contains the length
+			// Int32.MaxValue - 5 causes out of memory in mono on win32
+			// because I do not have 2GB of memory available
+			// so Int16.MaxValue - 5 is used instead.
+			// LAMESPEC for Oracle OCI - you can not get the length of the LONG VARCHAR value
+			// until after you get the value.  This could be why Oracle deprecated LONG VARCHAR.
+			// If you specify a definedSize less then the length of the column value,
+			// then you will get an OCI_ERROR ORA-01406: fetched column value was truncated
+			definedSize = LongVarCharMaxValue;
+			
+			value = Marshal.AllocHGlobal (definedSize);
+			ociType = OciDataType.LongVarChar;
+
+			int status = 0;
+			status = OciCalls.OCIDefineByPos (Parent,
+				out handle,
+				ErrorHandle,
+				position + 1,
+				value,
+				definedSize,
+				ociType,
+				ref indicator,
+				ref rlenp,
+				IntPtr.Zero, 0);
+
+			rlenp = (short) definedSize;
+
+			if (status != 0) {
+				OciErrorInfo info = ErrorHandle.HandleError ();
+				throw new OracleException (info.ErrorCode, info.ErrorMessage);
+			}
+		}
+
 		void DefineChar (int position)
 		{
-			ociType = OciDataType.Char;
-			
+			fieldType = typeof (System.String);
+
 			// The buffer is able to contain twice the defined size
 			// to allow usage of multibyte characters
 			value = Marshal.AllocHGlobal (definedSize * 2);
 
-			int status = 0;
+			ociType = OciDataType.Char;
 
+			int status = 0;
+			
 			status = OciCalls.OCIDefineByPos (Parent,
 						out handle,
 						ErrorHandle,
@@ -172,9 +246,42 @@ namespace System.Data.OracleClient.Oci {
 			}
 		}
 
+		void DefineNumber (int position) 
+		{
+			fieldType = typeof (System.Decimal);
+			value = Marshal.AllocHGlobal (definedSize);
+
+			ociType = OciDataType.Char;
+
+			int status = 0;
+			
+			status = OciCalls.OCIDefineByPos (Parent,
+				out handle,
+				ErrorHandle,
+				position + 1,
+				value,
+				definedSize * 2,
+				ociType,
+				ref indicator,
+				ref rlenp,
+				IntPtr.Zero,
+				0);
+
+			if (status != 0) {
+				OciErrorInfo info = ErrorHandle.HandleError ();
+				throw new OracleException (info.ErrorCode, info.ErrorMessage);
+			}
+		}
+
 		void DefineLob (int position, OciDataType type)
 		{
 			ociType = type;
+
+			if (ociType == OciDataType.Clob)
+				fieldType = typeof(System.String);
+			else if (ociType == OciDataType.Blob)
+				fieldType = Type.GetType("System.Byte[]");
+
 			int status = 0;
 
 			definedSize = -1;
@@ -202,6 +309,8 @@ namespace System.Data.OracleClient.Oci {
 							IntPtr.Zero,
 							0);
 
+			definedSize = Int32.MaxValue;
+
 			if (status != 0) {
 				OciErrorInfo info = ErrorHandle.HandleError ();
 				throw new OracleException (info.ErrorCode, info.ErrorMessage);
@@ -211,6 +320,7 @@ namespace System.Data.OracleClient.Oci {
 		void DefineRaw (int position)
 		{
 			ociType = OciDataType.Raw;
+			fieldType = Type.GetType("System.Byte[]");
 
 			value = Marshal.AllocHGlobal (definedSize);
 
@@ -237,10 +347,18 @@ namespace System.Data.OracleClient.Oci {
 		{
 			if (!disposed) {
 				try {
-					Marshal.FreeHGlobal (value);
+					switch (definedType) {
+					case OciDataType.Clob:
+					case OciDataType.Blob:
+						break;
+					default:
+						Marshal.FreeHGlobal (value);
+						break;
+					}
 					disposed = true;
 				} finally {
 					base.Dispose (disposing);
+					value = IntPtr.Zero;
 				}
 			}
 		}
@@ -254,6 +372,8 @@ namespace System.Data.OracleClient.Oci {
 		{
 			object tmp;
 
+			byte [] buffer = null;
+
 			switch (DataType) {
 			case OciDataType.VarChar2:
 			case OciDataType.String:
@@ -261,7 +381,8 @@ namespace System.Data.OracleClient.Oci {
 			case OciDataType.Char:
 			case OciDataType.CharZ:
 			case OciDataType.OciString:
-				byte [] buffer = new byte [Size];
+			case OciDataType.RowIdDescriptor:
+				buffer = new byte [Size];
 				Marshal.Copy (Value, buffer, 0, Size);
 				
 				// Get length of returned string
@@ -274,35 +395,26 @@ namespace System.Data.OracleClient.Oci {
 				OciCalls.OCICharSetToUnicode (env, ret, buffer, out rsize);
 	
 				return ret.ToString ();
+			case OciDataType.LongVarChar:
+			case OciDataType.Long:
+				buffer = new byte [LongVarCharMaxValue];
+				Marshal.Copy (Value, buffer, 0, buffer.Length);
+				
+				int longSize = 0;
+				if (BitConverter.IsLittleEndian)
+					longSize = BitConverter.ToInt32 (new byte[]{buffer[0], buffer[1], buffer[2], buffer[3]}, 0);
+				else
+					longSize = BitConverter.ToInt32 (new byte[]{buffer[3], buffer[2], buffer[1], buffer[0]}, 0);
 
+				ASCIIEncoding encoding = new ASCIIEncoding ();
+				string e = encoding.GetString (buffer, 4, longSize);
+				return e;
 			case OciDataType.Integer:
-				tmp = Marshal.PtrToStringAnsi (Value, Size);
-				if (tmp != null)
-					return Int32.Parse (String.Copy ((string) tmp));
-				break;
 			case OciDataType.Number:
-				tmp = Marshal.PtrToStringAnsi (Value, Size);
-				if (tmp != null) {
-					if (Scale == 0) {
-						try {
-							if (Size < 10)
-								return Int32.Parse (String.Copy ((string) tmp));
-							else
-								return Int64.Parse (String.Copy ((string) tmp));
-						} catch {
-							// Fallback: Scale is reported as 0 when column
-							// was created as NUMBER without parameters.
-							return Decimal.Parse (String.Copy ((string) tmp));
-						}
-					}
-					else
-						return Decimal.Parse (String.Copy ((string) tmp));
-				}
-				break;
 			case OciDataType.Float:
 				tmp = Marshal.PtrToStringAnsi (Value, Size);
 				if (tmp != null)
-					return Double.Parse (String.Copy ((string) tmp));
+					return Decimal.Parse (String.Copy ((string) tmp));
 				break;
 			case OciDataType.Date:
 				return UnpackDate ();
@@ -310,11 +422,47 @@ namespace System.Data.OracleClient.Oci {
 				byte [] raw_buffer = new byte [Size];
 				Marshal.Copy (Value, raw_buffer, 0, Size);
 				return raw_buffer;
+			case OciDataType.Blob:
+			case OciDataType.Clob:
+				return GetOracleLob ();
 			}
 
 			return DBNull.Value;
 		}
 
+		internal object GetOracleValue () 
+		{
+			object ovalue = GetValue ();
+
+			switch (DataType) {
+			case OciDataType.Raw:
+				return new OracleBinary ((byte[]) ovalue);
+			case OciDataType.Date:
+				return new OracleDateTime ((DateTime) ovalue);
+			case OciDataType.Blob:
+			case OciDataType.Clob:
+				OracleLob lob = (OracleLob) ovalue;
+				return lob;
+			case OciDataType.Integer:
+			case OciDataType.Number:
+			case OciDataType.Float:
+				return new OracleNumber ((decimal) ovalue);
+			case OciDataType.VarChar2:
+			case OciDataType.String:
+			case OciDataType.VarChar:
+			case OciDataType.Char:
+			case OciDataType.CharZ:
+			case OciDataType.OciString:
+			case OciDataType.LongVarChar:
+			case OciDataType.Long:
+			case OciDataType.RowIdDescriptor:
+				return new OracleString ((string) ovalue);
+			default:
+				// TODO: do other types
+				throw new NotImplementedException ();
+			}
+		}
+
 		[MonoTODO ("Be able to handle negative dates... i.e. BCE.")]
 		public DateTime UnpackDate ()
 		{
