@@ -30,6 +30,7 @@ namespace System.Data.OracleClient.Oci {
 		bool moreResults;
 		OciServiceHandle serviceHandle;
 		ArrayList values;
+		OracleCommand command;
 	
 		#endregion // Fields
 
@@ -63,6 +64,11 @@ namespace System.Data.OracleClient.Oci {
 			get { return values; }
 		}
 
+		public OracleCommand Command {
+			get { return command; }
+			set { command = value; }
+		}
+
 		#endregion // Properties
 
 		#region Methods
@@ -156,7 +162,7 @@ namespace System.Data.OracleClient.Oci {
 				IntPtr.Zero,
 				IntPtr.Zero,
 				(OciExecuteMode)executeMode);
-		
+
 			switch (status) {
 			case OciGlue.OCI_DEFAULT:
 				if (!nonQuery) {
@@ -208,6 +214,11 @@ namespace System.Data.OracleClient.Oci {
 			case OciGlue.OCI_DEFAULT:
 				moreResults = true;
 				break;
+			case OciGlue.OCI_SUCCESS_WITH_INFO:
+				//OciErrorInfo ei = ErrorHandle.HandleError ();
+				//command.Connection.CreateInfoMessage (ei);
+				moreResults = true;
+				break;
 			default:
 				OciErrorInfo info = ErrorHandle.HandleError ();
 				throw new OracleException (info.ErrorCode, info.ErrorMessage);
