@@ -8,10 +8,10 @@
 // Namespace: System.Data.OracleClient
 //
 // Authors: 
-//    Daniel Morgan <danmorg@sc.rr.com>
+//    Daniel Morgan <danielmorgan@verizon.net>
 //    Tim Coleman <tim@timcoleman.com>
 //
-// Copyright (C) Daniel Morgan, 2002
+// Copyright (C) Daniel Morgan, 2002, 2004-2005
 // Copyright (C) Tim Coleman , 2003
 //
 // Licensed under the MIT/X11 License.
@@ -21,9 +21,13 @@ using System;
 using System.ComponentModel;
 using System.Data;
 using System.Data.OracleClient.Oci;
+using System.Drawing.Design;
+using System.Text;
 
 namespace System.Data.OracleClient {
-	public class OracleCommand : Component, ICloneable, IDbCommand
+	[Designer ("Microsoft.VSDesigner.Data.VS.OracleCommandDesigner, " + Consts.AssemblyMicrosoft_VSDesigner)]
+	[ToolboxItem (true)]
+	public sealed class OracleCommand : Component, ICloneable, IDbCommand
 	{
 		#region Fields
 
@@ -36,7 +40,7 @@ namespace System.Data.OracleClient {
 		OracleTransaction transaction;
 		UpdateRowSource updatedRowSource;
 
-		OciStatementHandle preparedStatement;
+		private OciStatementHandle preparedStatement;
 		OciStatementType statementType;
 
 		#endregion // Fields
@@ -68,28 +72,42 @@ namespace System.Data.OracleClient {
 			UpdatedRowSource = UpdateRowSource.Both;
 			DesignTimeVisible = false;
 
-                        parameters = new OracleParameterCollection (this);
+			parameters = new OracleParameterCollection (this);
 		}
 
 		#endregion // Constructors
 
 		#region Properties
 
+		[DefaultValue ("")]
+		[RefreshProperties (RefreshProperties.All)]
+		[Editor ("Microsoft.VSDesigner.Data.Oracle.Design.OracleCommandTextEditor, " + Consts.AssemblyMicrosoft_VSDesigner, typeof(UITypeEditor))]
 		public string CommandText {
 			get { return commandText; }
 			set { commandText = value; }
 		}
 
+		[RefreshProperties (RefreshProperties.All)]
+		[DefaultValue (CommandType.Text)]
 		public CommandType CommandType {
 			get { return commandType; }
-			set { commandType = value; }
+			set { 
+				if (value == CommandType.TableDirect)
+					throw new ArgumentException ("OracleClient provider does not support TableDirect CommandType.");
+				commandType = value; 
+			}
 		}
 
+		[DefaultValue (null)]
+		[Editor ("Microsoft.VSDesigner.Data.Design.DbConnectionEditor, " + Consts.AssemblyMicrosoft_VSDesigner, typeof(UITypeEditor))]
 		public OracleConnection Connection {
 			get { return connection; }
 			set { connection = value; }
 		}
 
+		[DefaultValue (true)]
+		[Browsable (false)]
+		[DesignOnly (true)]
 		public bool DesignTimeVisible {
 			get { return designTimeVisible; }
 			set { designTimeVisible = value; }
@@ -108,6 +126,8 @@ namespace System.Data.OracleClient {
 			set { }
 		}
 
+		[Editor ("Microsoft.VSDesigner.Data.Design.DbConnectionEditor, " + Consts.AssemblyMicrosoft_VSDesigner, typeof(UITypeEditor))]
+		[DefaultValue (null)]
 		IDbConnection IDbCommand.Connection {
 			get { return Connection; }
 			set { 
@@ -130,15 +150,19 @@ namespace System.Data.OracleClient {
 			}
 		}
 
+		[DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
 		public OracleParameterCollection Parameters {
 			get { return parameters; }
 		}
 
+		[Browsable (false)]
+		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		public OracleTransaction Transaction {
 			get { return transaction; }
 			set { transaction = value; }
 		}
 
+		[DefaultValue (UpdateRowSource.Both)]
 		public UpdateRowSource UpdatedRowSource {
 			get { return updatedRowSource; }
 			set { updatedRowSource = value; }
@@ -187,7 +211,44 @@ namespace System.Data.OracleClient {
 		[MonoTODO]
 		public object Clone ()
 		{
-			throw new NotImplementedException ();
+			// create a new OracleCommand object with the same properties
+			
+			OracleCommand cmd = new OracleCommand ();
+			
+			cmd.CommandText = this.CommandText;
+			cmd.CommandType = this.CommandType;
+
+			// FIXME: not sure if I should set the same object here
+			// or get a clone of these too
+			cmd.Connection = this.Connection;
+			cmd.Transaction = this.Transaction;
+			
+			foreach (OracleParameter parm in this.Parameters) {
+
+				OracleParameter newParm = cmd.CreateParameter ();
+
+				newParm.DbType = parm.DbType;
+				newParm.Direction = parm.Direction;
+				newParm.IsNullable = parm.IsNullable;
+				newParm.Offset = parm.Offset;
+				newParm.OracleType = parm.OracleType;
+				newParm.ParameterName = parm.ParameterName;
+				newParm.Precision = parm.Precision;
+				newParm.Scale = parm.Scale;
+				newParm.SourceColumn = parm.SourceColumn;
+				newParm.SourceVersion = parm.SourceVersion;
+				newParm.Value = parm.Value;
+
+				cmd.Parameters.Add (newParm);
+			}
+
+			//cmd.Container = this.Container;
+			cmd.DesignTimeVisible = this.DesignTimeVisible;
+			//cmd.DesignMode = this.DesignMode;
+			cmd.Site = this.Site;
+			//cmd.UpdateRowSource = this.UpdateRowSource;
+
+			return cmd;
 		}
 
 		internal void CloseDataReader ()
@@ -205,10 +266,15 @@ namespace System.Data.OracleClient {
 		private int ExecuteNonQueryInternal (OciStatementHandle statement, bool useAutoCommit)
 		{
 			if (preparedStatement == null)
-				statement.Prepare (CommandText);
+				PrepareStatement (statement);
+
+			bool isNonQuery = IsNonQuery (statement);
 
 			BindParameters (statement);
-			statement.ExecuteNonQuery (useAutoCommit);
+			if (isNonQuery == true)
+				statement.ExecuteNonQuery (useAutoCommit);
+			else
+				statement.ExecuteQuery ();
 
 			int rowsAffected = statement.GetAttributeInt32 (OciAttributeType.RowCount, ErrorHandle);
 		
@@ -266,9 +332,60 @@ namespace System.Data.OracleClient {
 		}
 
 		[MonoTODO]
-		public object ExecuteOracleScalar (out OracleString rowid)
+		public object ExecuteOracleScalar ()
 		{
-			throw new NotImplementedException ();
+			object output = DBNull.Value;
+
+			AssertConnectionIsOpen ();
+			AssertTransactionMatch ();
+			AssertCommandTextIsSet ();
+
+			if (Transaction != null)
+				Transaction.AttachToServiceContext ();
+
+			OciStatementHandle statement = GetStatementHandle ();
+			try {
+				if (preparedStatement == null)
+					PrepareStatement (statement);
+
+				bool isNonQuery = IsNonQuery (statement);
+
+				BindParameters (statement);
+
+				if (isNonQuery == true)
+					ExecuteNonQueryInternal (statement, false);
+				else {
+					statement.ExecuteQuery ();
+
+					if (statement.Fetch ()) {
+						OciDefineHandle defineHandle = (OciDefineHandle) statement.Values [0];
+						if (!defineHandle.IsNull)
+							output = defineHandle.GetOracleValue ();
+						switch (defineHandle.DataType) {
+						case OciDataType.Blob:
+						case OciDataType.Clob:
+							((OracleLob) output).connection = Connection;
+							break;
+						}
+					}
+				}
+
+				return output;
+			}
+			finally {
+				SafeDisposeHandle (statement);
+			}
+		}
+
+		private bool IsNonQuery (OciStatementHandle statementHandle) 
+		{
+			// assumes Prepare() has been called prior to calling this function
+
+			OciStatementType statementType = statementHandle.GetStatementType ();
+			if (statementType.Equals (OciStatementType.Select))
+				return false;
+
+			return true;
 		}
 
 		public OracleDataReader ExecuteReader ()
@@ -284,19 +401,28 @@ namespace System.Data.OracleClient {
 			AssertNoDataReader ();
 			bool hasRows = false;
 
+                        this.behavior = behavior;
+				
 			if (Transaction != null) 
 				Transaction.AttachToServiceContext ();
 			
 			OciStatementHandle statement = GetStatementHandle ();
 			OracleDataReader rd = null;
+
 			try	{
 				if (preparedStatement == null)
-					statement.Prepare (CommandText);
+					PrepareStatement (statement);
 				else
 					preparedStatement = null;	// OracleDataReader releases the statement handle
 
+				bool isNonQuery = IsNonQuery (statement);
+
 				BindParameters (statement);
-				hasRows = statement.ExecuteQuery ();
+
+				if (isNonQuery)
+					ExecuteNonQueryInternal (statement, false);
+				else
+					hasRows = statement.ExecuteQuery ();
 
 				rd = new OracleDataReader (this, statement, hasRows);
 			}
@@ -310,7 +436,7 @@ namespace System.Data.OracleClient {
 
 		public object ExecuteScalar ()
 		{
-			object output;
+			object output = DBNull.Value;
 
 			AssertConnectionIsOpen ();
 			AssertTransactionMatch ();
@@ -322,15 +448,39 @@ namespace System.Data.OracleClient {
 			OciStatementHandle statement = GetStatementHandle ();
 			try {
 				if (preparedStatement == null)
-					statement.Prepare (CommandText);
+					PrepareStatement (statement);
+
+				bool isNonQuery = IsNonQuery (statement);
+
 				BindParameters (statement);
 
-				statement.ExecuteQuery ();
+				if (isNonQuery == true)
+					ExecuteNonQueryInternal (statement, false);
+				else {
+					statement.ExecuteQuery ();
 
-				if (statement.Fetch ()) 
-					output = ((OciDefineHandle) statement.Values [0]).GetValue ();
-				else
-					output = DBNull.Value;
+					if (statement.Fetch ()) {
+						OciDefineHandle defineHandle = (OciDefineHandle) statement.Values [0];
+						if (defineHandle.IsNull)
+							output = DBNull.Value;
+						else {
+							switch (defineHandle.DataType) {
+							case OciDataType.Blob:
+							case OciDataType.Clob:
+								OracleLob lob = (OracleLob) defineHandle.GetValue ();
+								lob.connection = Connection;
+								output = lob.Value;
+								lob.Close ();
+								break;
+							default:
+								output = defineHandle.GetValue ();
+								break;
+							}
+						}
+					}
+					else
+						output = DBNull.Value;
+				}
 			}
 			finally {
 				SafeDisposeHandle (statement);
@@ -348,6 +498,7 @@ namespace System.Data.OracleClient {
 			OciStatementHandle h = (OciStatementHandle) Connection.Environment.Allocate (OciHandleType.Statement);
 			h.ErrorHandle = Connection.ErrorHandle;
 			h.Service = Connection.ServiceContext;
+			h.Command = this;
 			return h;
 		}
 
@@ -372,11 +523,29 @@ namespace System.Data.OracleClient {
 			return ExecuteReader (behavior);
 		}
 
+		void PrepareStatement (OciStatementHandle statement) 
+		{
+			if (commandType == CommandType.StoredProcedure) {
+				StringBuilder sb = new StringBuilder ();
+				if (Parameters.Count > 0)
+					foreach (OracleParameter parm in Parameters) {
+						if (sb.Length > 0)
+							sb.Append (",");
+						sb.Append (":" + parm.ParameterName);
+					}
+
+				string sql = "call " + commandText + "(" + sb.ToString() + ")";
+				statement.Prepare (sql);
+			}
+			else	// Text
+				statement.Prepare (commandText);
+		}
+
 		public void Prepare ()
 		{
 			AssertConnectionIsOpen ();
 			OciStatementHandle statement = GetStatementHandle ();
-			statement.Prepare (CommandText);
+			PrepareStatement (statement);
 			preparedStatement = statement;
 		}
 
