@@ -17,8 +17,10 @@
 //
 
 using System;
+using System.Runtime.Serialization;
 
 namespace System.Data.OracleClient {
+	[Serializable]
 	public sealed class OracleException : SystemException
 	{
 		#region Fields
@@ -36,6 +38,12 @@ namespace System.Data.OracleClient {
 			this.message = message;
 		}
 
+		private OracleException (SerializationInfo si, StreamingContext sc) : base(si, sc)
+		{
+			message = si.GetString ("message");
+			code = si.GetInt32 ("source");
+		}
+
 		#endregion // Constructors
 
 		#region Properties
