@@ -1,8 +1,9 @@
 //
 // System.Data.Sql.ISqlConnection
 //
-// Author:
+// Authors:
 //   Tim Coleman (tim@timcoleman.com)
+//   Umadevi S (sumadevi@novell.com)
 //
 // Copyright (C) Tim Coleman, 2003
 //
@@ -44,10 +45,13 @@ namespace System.Data.Sql {
 
 		#region Methods
 
-		ISqlTransaction BeginTransaction ();
-		ISqlTransaction BeginTransaction (IsolationLevel iso);
+		new ISqlTransaction BeginTransaction ();
+		new ISqlTransaction BeginTransaction (IsolationLevel iso);
 		ISqlTransaction BeginTransaction (string transactionName);
 		ISqlTransaction BeginTransaction (IsolationLevel iso, string transactionName);
+		new ISqlCommand CreateCommand ();
+		ISqlCommand CreateCommand (string commandText);
+		ISqlExecutionContext CreateExecutionContext (SqlDefinition sqlDefinition);
 
 		#endregion // Methods
 	}
