@@ -1,8 +1,9 @@
 //
 // System.Data.Sql.InvalidUdtException
 //
-// Author:
+// Authors:
 //   Tim Coleman (tim@timcoleman.com)
+//   Umadevi S (sumadevi@novell.com)	
 //
 // Copyright (C) Tim Coleman, 2003
 //
@@ -32,11 +33,38 @@
 
 #if NET_2_0
 
-using System;
+using System.Runtime.Serialization;
 
 namespace System.Data.Sql {
-	public sealed class InvalidUdtException : Exception
+	public sealed class InvalidUdtException : SystemException
 	{
+		#region constructors 
+		
+		[MonoTODO]
+		public InvalidUdtException ()
+		{}
+		
+		[MonoTODO]
+		public InvalidUdtException (string message)	
+		{}
+
+		[MonoTODO]
+		public InvalidUdtException (string message, Exception innerException)
+		{}
+
+		[MonoTODO]
+		public InvalidUdtException (Type t, string reason)
+		{}
+
+		#endregion
+
+
+		#region methods
+
+		[MonoTODO]
+		public override void GetObjectData (SerializationInfo si, StreamingContext context)
+		{}
+		#endregion
 	}
 }
 
