@@ -37,16 +37,23 @@ using Mono.Data.Tds.Protocol;
 using System;
 using System.ComponentModel;
 using System.Data;
+using System.Data.Common;
 using System.Runtime.Serialization;
 using System.Text;
 
 namespace System.Data.SqlClient {
-	[Serializable]
-	public sealed class SqlException : SystemException
+	[SerializableAttribute()]
+
+	public sealed class SqlException
+#if NET_2_0
+	 : DbException
+#else
+	 : SystemException
+#endif //NET_1_1
 	{
 		#region Fields
 
-		SqlErrorCollection errors; 
+		private SqlErrorCollection errors; 
 
 		#endregion // Fields
 
@@ -63,6 +70,11 @@ namespace System.Data.SqlClient {
 		{
 			errors = new SqlErrorCollection (theClass, lineNumber, message, number, procedure, server, source, state);
 		}
+		
+		private SqlException(SerializationInfo si, StreamingContext sc)
+		{
+			errors = (SqlErrorCollection)si.GetValue("Errors", typeof(SqlErrorCollection));
+		}
 
 		#endregion // Constructors
 
@@ -127,7 +139,7 @@ namespace System.Data.SqlClient {
 			if (si == null)
 				throw new ArgumentNullException ("si");
 
-			si.AddValue ("errors", errors);
+			si.AddValue ("Errors", errors, typeof(SqlErrorCollection));
 			base.GetObjectData (si, context);
 		}
 
