@@ -6,23 +6,55 @@
 //   Daniel Morgan (danmorg@sc.rr.com)
 //   Tim Coleman (tim@timcoleman.com)
 //   Diego Caravana (diego@toth.it)
+//   Umadevi S (sumadevi@novell.com)
 //
 // (C) Ximian, Inc. 2002
 // Copyright (C) Tim Coleman, 2002
 //
 
+//
+// Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using Mono.Data.Tds;
 using Mono.Data.Tds.Protocol;
 using System;
 using System.ComponentModel;
 using System.Data;
 using System.Data.Common;
+#if NET_2_0
+using System.Data.ProviderBase;
+#endif // NET_2_0
+using System.Data.SqlTypes;
 using System.Runtime.InteropServices;
 using System.Text;
 
 namespace System.Data.SqlClient {
 	[TypeConverterAttribute (typeof (SqlParameterConverter))]
+#if NET_2_0
+	public sealed class SqlParameter : DbParameterBase, IDbDataParameter, IDataParameter, ICloneable
+#else
 	public sealed class SqlParameter : MarshalByRefObject, IDbDataParameter, IDataParameter, ICloneable
+#endif // NET_2_0
 	{
 		#region Fields
 
@@ -50,7 +82,7 @@ namespace System.Data.SqlClient {
 
 		public SqlParameter (string parameterName, object value) 
 		{
-			metaParameter = new TdsMetaParameter (parameterName, value);
+			metaParameter = new TdsMetaParameter (parameterName, SqlTypeToFrameworkType (value));
 			this.sourceVersion = DataRowVersion.Current;
 			InferSqlType (value);
 		}
@@ -73,8 +105,10 @@ namespace System.Data.SqlClient {
 		[EditorBrowsable (EditorBrowsableState.Advanced)]	 
 		public SqlParameter (string parameterName, SqlDbType dbType, int size, ParameterDirection direction, bool isNullable, byte precision, byte scale, string sourceColumn, DataRowVersion sourceVersion, object value) 
 		{
-			metaParameter = new TdsMetaParameter (parameterName, size, isNullable, precision, scale, value);
-
+			metaParameter = new TdsMetaParameter (parameterName, size, 
+							      isNullable, precision, 
+							      scale, 
+							      SqlTypeToFrameworkType (value));
 			SqlDbType = dbType;
 			Direction = direction;
 			SourceColumn = sourceColumn;
@@ -84,9 +118,10 @@ namespace System.Data.SqlClient {
 		// This constructor is used internally to construct a
 		// SqlParameter.  The value array comes from sp_procedure_params_rowset.
 		// This is in SqlCommand.DeriveParameters.
-		internal SqlParameter (object[] dbValues)
+		internal SqlParameter (object[] dbValues) 
+                        : this (dbValues [3].ToString (), String.Empty)
 		{
-			Precision = 0;
+                        Precision = 0;
 			Scale = 0;
 			Direction = ParameterDirection.Input;
 
@@ -105,16 +140,20 @@ namespace System.Data.SqlClient {
 			case 4:
 				Direction = ParameterDirection.ReturnValue;
 				break;
+                        default:
+                                Direction = ParameterDirection.Input;
+                                break;
 			}
+			IsNullable = (dbValues [8] != null && 
+                                      dbValues [8] != DBNull.Value) ? (bool) dbValues [8] : false;
 
-			IsNullable = (bool) dbValues[8];
+			if (dbValues [12] != null && dbValues [12] != DBNull.Value)
+				Precision = (byte) ((short) dbValues [12]);
 
-			if (dbValues[12] != null)
-				Precision = (byte) ((short) dbValues[12]);
-			if (dbValues[13] != null)
-				Scale = (byte) ((short) dbValues[13]);
+			if (dbValues [13] != null && dbValues [13] != DBNull.Value)
+				Scale = (byte) ( (short) dbValues [13]);
 
-			SetDbTypeName ((string) dbValues[16]);
+			SetDbTypeName ((string) dbValues [16]);
 		}
 
 		#endregion // Constructors
@@ -128,12 +167,18 @@ namespace System.Data.SqlClient {
 			set { container = value; }
 		}
 
+#if ONLY_1_0 || ONLY_1_1	
 		[Browsable (false)]
-		[DataCategory ("Data")]
 		[DataSysDescription ("The parameter generic type.")]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		[RefreshProperties (RefreshProperties.All)]
-		public DbType DbType {
+#endif		                                         
+		[DataCategory ("Data")]
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 DbType DbType {
 			get { return dbType; }
 			set { 
 				SetDbType (value); 
@@ -142,9 +187,18 @@ namespace System.Data.SqlClient {
 		}
 
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("Input, output, or bidirectional parameter.")]
 		[DefaultValue (ParameterDirection.Input)]
-		public ParameterDirection Direction {
+#endif
+#if NET_2_0
+		[RefreshProperties (RefreshProperties.All)]
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 ParameterDirection Direction {
 			get { return direction; }
 			set { 
 				direction = value; 
@@ -171,76 +225,140 @@ namespace System.Data.SqlClient {
 			set { metaParameter.ParameterName = value; }
 		}
 
+#if ONLY_1_0 || ONLY_1_1
 		[Browsable (false)]
 		[DataSysDescription ("a design-time property used for strongly typed code-generation.")]
 		[DefaultValue (false)]
 		[DesignOnly (true)]
 		[EditorBrowsable (EditorBrowsableState.Advanced)]	 
-		public bool IsNullable	{
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 bool IsNullable	{
 			get { return metaParameter.IsNullable; }
 			set { metaParameter.IsNullable = value; }
 		}
 
 		[Browsable (false)]
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("Offset in variable length data types.")]
 		[DefaultValue (0)]
-		public int Offset {
+#endif
+#if NET_2_0
+		[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 int Offset {
 			get { return offset; }
 			set { offset = value; }
 		}
-		
+	
+#if ONLY_1_0 || ONLY_1_1	
 		[DataSysDescription ("Name of the parameter, like '@p1'")]
 		[DefaultValue ("")]
-		public string ParameterName {
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 string ParameterName {
 			get { return metaParameter.ParameterName; }
 			set { metaParameter.ParameterName = value; }
 		}
 
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("For decimal, numeric, varnumeric DBTypes.")]
 		[DefaultValue (0)]
-		public byte Precision {
+#endif
+#if NET_2_0
+		[Browsable (false)]
+		[EditorBrowsable (EditorBrowsableState.Never)]
+#endif		
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 byte Precision {
 			get { return metaParameter.Precision; }
 			set { metaParameter.Precision = value; }
 		}
 
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("For decimal, numeric, varnumeric DBTypes.")]
 		[DefaultValue (0)]
-                public byte Scale {
+#endif
+#if NET_2_0
+		[Browsable (false)]
+                [EditorBrowsable (EditorBrowsableState.Never)]
+#endif
+                public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 byte Scale {
 			get { return metaParameter.Scale; }
 			set { metaParameter.Scale = value; }
 		}
 
 		[DataCategory ("Data")]
-		[DataSysDescription ("Size of variable length datatypes (strings & arrays).")]
+#if ONLY_1_0 || ONLY_1_1
+		[DataSysDescription ("Size of variable length data types (strings & arrays).")]
 		[DefaultValue (0)]
-                public int Size {
+#endif
+                public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 int Size {
 			get { return metaParameter.Size; }
 			set { metaParameter.Size = value; }
 		}
 
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("When used by a DataAdapter.Update, the source column name that is used to find the DataSetColumn name in the ColumnMappings. This is to copy a value between the parameter and a datarow.")]
 		[DefaultValue ("")]
-		public string SourceColumn {
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 string SourceColumn {
 			get { return sourceColumn; }
 			set { sourceColumn = value; }
 		}
 
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("When used by a DataAdapter.Update (UpdateCommand only), the version of the DataRow value that is used to update the data source.")]
 		[DefaultValue (DataRowVersion.Current)]
-		public DataRowVersion SourceVersion {
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 DataRowVersion SourceVersion {
 			get { return sourceVersion; }
 			set { sourceVersion = value; }
 		}
 		
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("The parameter native type.")]
 		[DefaultValue (SqlDbType.NVarChar)]
+#endif
 		[RefreshProperties (RefreshProperties.All)]
+#if NET_2_0
+		[DbProviderSpecificTypeProperty(true)]
+#endif
 		public SqlDbType SqlDbType {
 			get { return sqlDbType; }
 			set { 
@@ -250,18 +368,32 @@ namespace System.Data.SqlClient {
 		}
 
 		[DataCategory ("Data")]
+#if ONLY_1_0 || ONLY_1_1
 		[DataSysDescription ("Value of the parameter.")]
 		[DefaultValue (null)]
+#endif
 	        [TypeConverterAttribute (typeof (StringConverter))]
-		public object Value {
+#if NET_2_0
+		[RefreshProperties (RefreshProperties.All)]		
+#endif
+		public 
+#if NET_2_0
+		override
+#endif // NET_2_0
+	 object Value {
 			get { return metaParameter.Value; }
 			set { 
 				if (!isTypeSet)
 					InferSqlType (value);
-				metaParameter.Value = value; 
+				metaParameter.Value = SqlTypeToFrameworkType (value);
 			}
 		}
 
+//#if NET_2_0
+//		public SqlCompareOptions CompareInfo{
+
+//#endif
+
 		#endregion // Properties
 
 		#region Methods
@@ -281,44 +413,63 @@ namespace System.Data.SqlClient {
 
 			switch (type.FullName) {
 			case "System.Int64":
+			case "System.Data.SqlTypes.SqlInt64":
 				SetSqlDbType (SqlDbType.BigInt);
 				break;
 			case "System.Boolean":
+			case "System.Data.SqlTypes.SqlBoolean":
 				SetSqlDbType (SqlDbType.Bit);
 				break;
 			case "System.String":
+			case "System.Data.SqlTypes.SqlString":
 				SetSqlDbType (SqlDbType.NVarChar);
 				break;
 			case "System.DateTime":
+			case "System.Data.SqlTypes.SqlDateTime":
 				SetSqlDbType (SqlDbType.DateTime);
 				break;
 			case "System.Decimal":
+			case "System.Data.SqlTypes.SqlDecimal":
 				SetSqlDbType (SqlDbType.Decimal);
 				break;
 			case "System.Double":
+			case "System.Data.SqlTypes.SqlDouble":
 				SetSqlDbType (SqlDbType.Float);
 				break;
 			case "System.Byte[]":
+			case "System.Data.SqlTypes.SqlBinary":
 				SetSqlDbType (SqlDbType.VarBinary);
 				break;
 			case "System.Byte":
+			case "System.Data.SqlTypes.SqlByte":
 				SetSqlDbType (SqlDbType.TinyInt);
 				break;
 			case "System.Int32":
+			case "System.Data.SqlTypes.SqlInt32":
 				SetSqlDbType (SqlDbType.Int);
 				break;
 			case "System.Single":
+			case "System.Data.SqlTypes.Single":
 				SetSqlDbType (SqlDbType.Real);
 				break;
 			case "System.Int16":
+			case "System.Data.SqlTypes.SqlInt16":
 				SetSqlDbType (SqlDbType.SmallInt);
 				break;
 			case "System.Guid":
+			case "System.Data.SqlTypes.SqlGuid":
 				SetSqlDbType (SqlDbType.UniqueIdentifier);
 				break;
-			case "System.Object":
-				SetSqlDbType (SqlDbType.Variant);
+			case "System.Money":
+			case "System.SmallMoney":
+			case "System.Data.SqlTypes.SqlMoney":
+				SetSqlDbType (SqlDbType.Money);
 				break;
+			case "System.Object":
+			case "System.DBNull":
+				SetSqlDbType (SqlDbType.Variant); // variant can contain numeric,
+								//string,binary or data and also nul								    //values, so we can later resolve 									// it to correct type.	
+				break;	
 			default:
 				throw new ArgumentException (exception);				
 			}
@@ -605,6 +756,57 @@ namespace System.Data.SqlClient {
 			return ParameterName;
 		}
 
+		private object SqlTypeToFrameworkType (object value)
+		{
+			if (! (value is INullable)) // if the value is not SqlType
+				return value;
+
+			// Map to .net type, as Mono TDS respects only types from .net
+			switch (value.GetType ().FullName) {
+			case "System.Data.SqlTypes.SqlBinary":
+				return ( (SqlBinary) value).Value;
+			case "System.Data.SqlTypes.SqlBoolean":
+				return ( (SqlBoolean) value).Value;
+			case "System.Data.SqlTypes.SqlByte":
+				return ( (SqlByte) value).Value;
+			case "System.Data.SqlTypes.SqlDateTime":
+				return ( (SqlDateTime) value).Value;
+			case "System.Data.SqlTypes.SqlDecimal":
+				return ( (SqlDecimal) value).Value;
+			case "System.Data.SqlTypes.SqlDouble":
+				return ( (SqlDouble) value).Value;
+			case "System.Data.SqlTypes.SqlGuid":
+				return ( (SqlGuid) value).Value;
+			case "System.Data.SqlTypes.SqlInt16":
+				return ( (SqlInt16) value).Value;
+			case "System.Data.SqlTypes.SqlInt32 ":
+				return ( (SqlInt32 ) value).Value;
+			case "System.Data.SqlTypes.SqlInt64":
+				return ( (SqlInt64) value).Value;
+			case "System.Data.SqlTypes.SqlMoney":
+				return ( (SqlMoney) value).Value;
+			case "System.Data.SqlTypes.SqlSingle":
+				return ( (SqlSingle) value).Value;
+			case "System.Data.SqlTypes.SqlString":
+				return ( (SqlString) value).Value;
+			}
+			return value;
+		}
+
+#if NET_2_0
+                [MonoTODO]
+                public override void CopyTo (DbParameter param)
+                {
+                        throw new NotImplementedException ();
+                }
+                
+                [MonoTODO]
+                public override void ResetDbType ()
+                {
+                        throw new NotImplementedException ();
+                }
+#endif // NET_2_0
+
 		#endregion // Methods
 	}
 }
