@@ -9,6 +9,7 @@
 //   Tim Coleman (tim@timcoleman.com)
 //   Ville Palo <vi64pa@koti.soon.fi>
 //   Atsushi Enomoto <atsushi@ximian.com>
+//   Konstantin Triger <kostat@mainsoft.com>
 //
 // (C) Ximian, Inc. 2002
 // Copyright (C) Tim Coleman, 2002, 2003
@@ -108,10 +109,16 @@ namespace System.Data {
 				caseSensitive = value; 
 				if (!caseSensitive) {
 					foreach (DataTable table in Tables) {
+						table.ResetCaseSensitiveIndexes();
 						foreach (Constraint c in table.Constraints)
 							c.AssertConstraint ();
 					}
 				}
+				else {
+					foreach (DataTable table in Tables) {
+						table.ResetCaseSensitiveIndexes();
+					}
+				}
 			}
 		}
 
@@ -138,19 +145,7 @@ namespace System.Data {
 		public bool EnforceConstraints {
 			get { return enforceConstraints; } 
 			set { 
-				if (value != enforceConstraints) {
-					enforceConstraints = value; 
-					if (value) {
-						foreach (DataTable table in Tables) {
-							// first assert all unique constraints
-							foreach (UniqueConstraint uc in table.Constraints.UniqueConstraints)
-								uc.AssertConstraint ();
-							// then assert all foreign keys
-							foreach (ForeignKeyConstraint fk in table.Constraints.ForeignKeyConstraints)
-								fk.AssertConstraint ();
-						}
-					}
-				}
+				InternalEnforceConstraints(value,true);
 			}
 		}
 
@@ -189,6 +184,28 @@ namespace System.Data {
 			}
 		}
 
+		internal void InternalEnforceConstraints(bool value,bool resetIndexes)
+		{
+				if (value != enforceConstraints) {
+					if (value) {
+						foreach (DataTable table in Tables) {
+							// FIXME : is that correct?
+							// By design  the indexes should be updated at this point.
+							// In Fill from BeginLoadData till EndLoadData indexes are not updated (reset in EndLoadData)
+							// In DataRow.EndEdit indexes are always updated.
+							if (resetIndexes) {
+								table.ResetIndexes();
+							}
+							// assert all constraints
+							foreach (Constraint constraint in table.Constraints)
+								constraint.AssertConstraint();
+						}
+					}
+
+					enforceConstraints = value;
+				}
+		}
+
 		public void Merge (DataRow[] rows)
 		{
 			Merge (rows, false, MissingSchemaAction.Add);
@@ -330,22 +347,34 @@ namespace System.Data {
 				tempTable.AcceptChanges ();
 		}
 
+                /// <summary>
+                /// Clears all the tables
+                /// </summary>
 		public void Clear ()
 		{
 			if (_xmlDataDocument != null)
 				throw new NotSupportedException ("Clear function on dataset and datatable is not supported when XmlDataDocument is bound to the DataSet.");
-			for (int t = 0; t < tableCollection.Count; t++) {
+                        bool enforceConstraints = this.EnforceConstraints;
+                        this.EnforceConstraints = false;
+                        for (int t = 0; t < tableCollection.Count; t++) {
 				tableCollection[t].Clear ();
 			}
+                        this.EnforceConstraints = enforceConstraints;
 		}
 
 		public virtual DataSet Clone ()
 		{
-			DataSet Copy = new DataSet ();
+			// need to return the same type as this...
+			DataSet Copy = (DataSet) Activator.CreateInstance(GetType(), true);
+	
 			CopyProperties (Copy);
 
 			foreach (DataTable Table in Tables) {
-				Copy.Tables.Add (Table.Clone ());
+		        // tables are often added in no-args constructor, don't add them
+		        // twice.
+		        	if (!Copy.Tables.Contains(Table.TableName)) {
+			    		Copy.Tables.Add (Table.Clone ());
+        			}
 			}
 
 			//Copy Relationships between tables after existance of tables
@@ -448,46 +477,45 @@ namespace System.Data {
 				return null;
 			
 			DataSet copySet = Clone ();
+			bool prev = copySet.EnforceConstraints;
+			copySet.EnforceConstraints = false;
+
 			Hashtable addedRows = new Hashtable ();
 
-			IEnumerator tableEnumerator = Tables.GetEnumerator ();
-			DataTable origTable;
-			DataTable copyTable;
-			while (tableEnumerator.MoveNext ()) {
-				origTable = (DataTable)tableEnumerator.Current;
-				copyTable = copySet.Tables[origTable.TableName];
-				
-				// Look for relations that have this table as child
-				IEnumerator relations = origTable.ParentRelations.GetEnumerator ();
-
-				IEnumerator rowEnumerator = origTable.Rows.GetEnumerator ();
-				while (rowEnumerator.MoveNext ()) {
-					DataRow row = (DataRow)rowEnumerator.Current;
-					
-					if (row.IsRowChanged (rowStates))
-						AddChangedRow (addedRows, copySet, copyTable, relations, row);
+			for (int i = 0; i < Tables.Count; i++) {
+				DataTable origTable = Tables [i];
+				DataTable copyTable = copySet.Tables[origTable.TableName];
+				for (int j = 0; j < origTable.Rows.Count; j++) {
+					DataRow row = origTable.Rows [j];
+					if (!row.IsRowChanged (rowStates)
+					    || addedRows.Contains (row))
+						continue;
+					AddChangedRow (addedRows, copyTable, row);
 				}
 			}
+			copySet.EnforceConstraints = prev;
 			return copySet;
 		}
 		
-		void AddChangedRow (Hashtable addedRows, DataSet copySet, DataTable copyTable, IEnumerator relations, DataRow row)
+		private void AddChangedRow (Hashtable addedRows, DataTable copyTable, DataRow row)
 		{
 			if (addedRows.ContainsKey (row)) return;
-			
-			relations.Reset ();
-			while (relations.MoveNext ()) {
-				DataRow parentRow = row.GetParentRow ((DataRelation) relations.Current);
-				if (parentRow == null || addedRows.ContainsKey (parentRow)) continue;
-				DataTable parentCopyTable = copySet.Tables [parentRow.Table.TableName];
-				AddChangedRow (addedRows, copySet, parentCopyTable, parentRow.Table.ParentRelations.GetEnumerator (), parentRow);
+
+			foreach (DataRelation relation in row.Table.ParentRelations) {
+				DataRow parent = row.GetParentRow (relation);
+				if (parent == null)
+					continue;
+				// add the parent row
+				DataTable parentCopyTable = copyTable.DataSet.Tables [parent.Table.TableName];
+				AddChangedRow (addedRows, parentCopyTable, parent);
 			}
-		
-			DataRow newRow = copyTable.NewRow ();
-			copyTable.Rows.Add (newRow);
+
+			// add the current row
+			DataRow newRow = copyTable.NewNotInitializedRow();
+			copyTable.Rows.AddInternal(newRow);
 			row.CopyValuesToRow (newRow);
 			newRow.XmlRowID = row.XmlRowID;
-			addedRows.Add (row,row);
+			addedRows.Add (row, row);
 		}
 
 #if NET_2_0
@@ -756,10 +784,8 @@ namespace System.Data {
 			XmlTextWriter xwriter = new XmlTextWriter (writer);
 			try {
 				xwriter.Formatting = Formatting.Indented;
-//				xwriter.WriteStartDocument ();
 				WriteXmlSchema (xwriter);
 			} finally {
-//				xwriter.WriteEndDocument ();
 				xwriter.Close ();
 			}
 		}
@@ -851,117 +877,221 @@ namespace System.Data {
 			return ReadXml (new XmlTextReader (reader), mode);
 		}
 
-		public XmlReadMode ReadXml (XmlReader reader, XmlReadMode mode)
+		// LAMESPEC: XmlReadMode.Fragment is far from presisely
+		// documented. MS.NET infers schema against this mode.
+		public XmlReadMode ReadXml (XmlReader input, XmlReadMode mode)
 		{
-			switch (reader.ReadState) {
+			switch (input.ReadState) {
 			case ReadState.EndOfFile:
 			case ReadState.Error:
 			case ReadState.Closed:
 				return mode;
 			}
 			// Skip XML declaration and prolog
-			reader.MoveToContent();
-			if (reader.EOF)
+			input.MoveToContent ();
+			if (input.EOF)
 				return mode;
 
-			XmlReadMode Result = mode;
+			// FIXME: We need more decent code here, but for now
+			// I don't know the precise MS.NET behavior, I just
+			// delegate to specific read process.
+			switch (mode) {
+			case XmlReadMode.IgnoreSchema:
+				return ReadXmlIgnoreSchema (input, mode, true);
+			case XmlReadMode.ReadSchema:
+				return ReadXmlReadSchema (input, mode, true);
+			}
+			// remaining modes are: Auto, InferSchema, Fragment, Diffgram
+
+			XmlReader reader = input;
+
+			int depth = reader.Depth;
+			XmlReadMode result = mode;
+			bool skippedTopLevelElement = false;
+			string potentialDataSetName = null;
+			XmlDocument doc = null;
+			bool shouldReadData = mode != XmlReadMode.DiffGram;
+			bool shouldNotInfer = Tables.Count > 0;
+
+			switch (mode) {
+			case XmlReadMode.Auto:
+			case XmlReadMode.InferSchema:
+				doc = new XmlDocument ();
+				do {
+					doc.AppendChild (doc.ReadNode (reader));
+				} while (!reader.EOF &&
+					doc.DocumentElement == null);
+				reader = new XmlNodeReader (doc);
+				reader.MoveToContent ();
+				break;
+			case XmlReadMode.DiffGram:
+				if (!(reader.LocalName == "diffgram" &&
+					reader.NamespaceURI == XmlConstants.DiffgrNamespace))
+					goto case XmlReadMode.Auto;
+				break;
+			}
+
+			switch (mode) {
+			case XmlReadMode.Auto:
+			case XmlReadMode.InferSchema:
+			case XmlReadMode.ReadSchema:
+				if (!(reader.LocalName == "diffgram" &&
+					reader.NamespaceURI == XmlConstants.DiffgrNamespace) &&
+					!(reader.LocalName == "schema" &&
+					reader.NamespaceURI == XmlSchema.Namespace))
+					potentialDataSetName = reader.LocalName;
+				goto default;
+			case XmlReadMode.Fragment:
+				break;
+			default:
+				if (!(reader.LocalName == "diffgram" &&
+					reader.NamespaceURI == XmlConstants.DiffgrNamespace) &&
+					!(reader.LocalName == "schema" &&
+					reader.NamespaceURI == XmlSchema.Namespace)) {
+					if (!reader.IsEmptyElement) {
+						reader.Read ();
+						reader.MoveToContent ();
+						skippedTopLevelElement = true;
+					}
+					else {
+						switch (mode) {
+						case XmlReadMode.Auto:
+						case XmlReadMode.InferSchema:
+							DataSetName = reader.LocalName;
+							break;
+						}
+						reader.Read ();
+					}
+				}
+				break;
+			}
+
+			// If schema, then read the first element as schema 
+			if (reader.LocalName == "schema" && reader.NamespaceURI == XmlSchema.Namespace) {
+				shouldNotInfer = true;
+				switch (mode) {
+				case XmlReadMode.IgnoreSchema:
+				case XmlReadMode.InferSchema:
+					reader.Skip ();
+					break;
+				case XmlReadMode.Fragment:
+					ReadXmlSchema (reader);
+					break;
+				case XmlReadMode.DiffGram:
+				case XmlReadMode.Auto:
+					if (Tables.Count == 0) {
+						ReadXmlSchema (reader);
+						if (mode == XmlReadMode.Auto)
+							result = XmlReadMode.ReadSchema;
+					} else {
+					// otherwise just ignore and return IgnoreSchema
+						reader.Skip ();
+						result = XmlReadMode.IgnoreSchema;
+					}
+					break;
+				case XmlReadMode.ReadSchema:
+					ReadXmlSchema (reader);
+					break;
+				}
+			}
 
 			// If diffgram, then read the first element as diffgram 
 			if (reader.LocalName == "diffgram" && reader.NamespaceURI == XmlConstants.DiffgrNamespace) {
 				switch (mode) {
 				case XmlReadMode.Auto:
+				case XmlReadMode.IgnoreSchema:
 				case XmlReadMode.DiffGram:
 					XmlDiffLoader DiffLoader = new XmlDiffLoader (this);
 					DiffLoader.Load (reader);
-					// (and leave rest of the reader as is)
-					return  XmlReadMode.DiffGram;
+					if (mode == XmlReadMode.Auto)
+						result = XmlReadMode.DiffGram;
+					shouldReadData = false;
+					break;
 				case XmlReadMode.Fragment:
 					reader.Skip ();
-					// (and continue to read)
 					break;
 				default:
 					reader.Skip ();
-					// (and leave rest of the reader as is)
-					return mode;
-				}
-			}
-			// If schema, then read the first element as schema 
-			if (reader.LocalName == "schema" && reader.NamespaceURI == XmlSchema.Namespace) {
-				switch (mode) {
-				case XmlReadMode.IgnoreSchema:
-				case XmlReadMode.InferSchema:
-					reader.Skip ();
-					// (and break up read)
-					return mode;
-				case XmlReadMode.Fragment:
-					ReadXmlSchema (reader);
-					// (and continue to read)
 					break;
-				case XmlReadMode.Auto:
-					if (Tables.Count == 0) {
-						ReadXmlSchema (reader);
-						return XmlReadMode.ReadSchema;
-					} else {
-					// otherwise just ignore and return IgnoreSchema
-						reader.Skip ();
-						return XmlReadMode.IgnoreSchema;
-					}
-				default:
-					ReadXmlSchema (reader);
-					// (and leave rest of the reader as is)
-					return mode; // When DiffGram, return DiffGram
 				}
 			}
-			// Otherwise, read as dataset... but only when required.
-			XmlReadMode explicitReturnMode = XmlReadMode.Auto;
-			XmlDocument doc;
-			switch (mode) {
-			case XmlReadMode.Auto:
-				if (Tables.Count > 0)
-					goto case XmlReadMode.IgnoreSchema;
-				else
-					goto case XmlReadMode.InferSchema;
-			case XmlReadMode.InferSchema:
-				doc = new XmlDocument ();
-				do {
-					doc.AppendChild (doc.ReadNode (reader));
-					reader.MoveToContent ();
-					if (doc.DocumentElement != null)
-						break;
-				} while (!reader.EOF);
-				InferXmlSchema (doc, null);
-				reader = new XmlNodeReader (doc);
-				explicitReturnMode = XmlReadMode.InferSchema;
-				break;
-			case XmlReadMode.ReadSchema:
-				doc = new XmlDocument ();
-				do {
-					doc.AppendChild (doc.ReadNode (reader));
-					reader.MoveToContent ();
-					if (doc.DocumentElement != null)
-						break;
-				} while (!reader.EOF);
-				if (doc.DocumentElement != null) {
-					XmlElement schema = doc.DocumentElement ["schema", XmlSchema.Namespace] as XmlElement;
-					if (schema != null) {
-						ReadXmlSchema (new XmlNodeReader (schema));
-						explicitReturnMode = XmlReadMode.ReadSchema;
-					}
-				}
-				reader = new XmlNodeReader (doc);
-				break;
-			case XmlReadMode.IgnoreSchema:
-			case XmlReadMode.Fragment:
-				break;
-			default:
-				reader.Skip ();
-				return mode;
-			}
 
-			XmlDataReader.ReadXml (this, reader, mode);
-			if (explicitReturnMode != XmlReadMode.Auto)
-				return explicitReturnMode;
-			return mode == XmlReadMode.Auto ? XmlReadMode.IgnoreSchema : mode;
+			// if schema after diffgram, just skip it.
+			if (!shouldReadData && reader.LocalName == "schema" && reader.NamespaceURI == XmlSchema.Namespace) {
+				shouldNotInfer = true;
+				switch (mode) {
+				default:
+					reader.Skip ();
+					break;
+				case XmlReadMode.ReadSchema:
+				case XmlReadMode.DiffGram:
+					if (Tables.Count == 0)
+						ReadXmlSchema (reader);
+					break;
+				}
+			}
+
+			if (reader.EOF)
+				return result == XmlReadMode.Auto ?
+					potentialDataSetName != null && !shouldNotInfer ?
+					XmlReadMode.InferSchema :
+					XmlReadMode.IgnoreSchema : result;
+
+			// Otherwise, read as dataset... but only when required.
+			if (shouldReadData && !shouldNotInfer) {
+				switch (mode) {
+				case XmlReadMode.Auto:
+					if (Tables.Count > 0)
+						goto case XmlReadMode.IgnoreSchema;
+					else
+						goto case XmlReadMode.InferSchema;
+				case XmlReadMode.InferSchema:
+					InferXmlSchema (doc, null);
+					if (mode == XmlReadMode.Auto)
+						result = XmlReadMode.InferSchema;
+					break;
+				case XmlReadMode.IgnoreSchema:
+				case XmlReadMode.Fragment:
+				case XmlReadMode.DiffGram:
+					break;
+				default:
+					shouldReadData = false;
+					break;
+				}
+			}
+
+			if (shouldReadData) {
+				XmlReader dataReader = reader;
+				if (doc != null) {
+					dataReader = new XmlNodeReader (doc);
+					dataReader.MoveToContent ();
+				}
+				if (reader.NodeType == XmlNodeType.Element)
+					XmlDataReader.ReadXml (this, dataReader,
+						mode);
+			}
+
+			if (skippedTopLevelElement) {
+				switch (result) {
+				case XmlReadMode.Auto:
+				case XmlReadMode.InferSchema:
+//					DataSetName = potentialDataSetName;
+//					result = XmlReadMode.InferSchema;
+					break;
+				}
+				if (reader.NodeType == XmlNodeType.EndElement)
+					reader.ReadEndElement ();
+			}
+//*
+			while (input.Depth > depth)
+				input.Read ();
+			if (input.NodeType == XmlNodeType.EndElement)
+				input.Read ();
+//*/
+			input.MoveToContent ();
+
+			return result == XmlReadMode.Auto ?
+				XmlReadMode.IgnoreSchema : result;
 		}
 		#endregion // Public Methods
 
@@ -1035,14 +1165,7 @@ namespace System.Data {
 		
 		protected virtual void ReadXmlSerializable (XmlReader reader)
 		{
-			reader.MoveToContent ();
-			reader.ReadStartElement ();
-			reader.MoveToContent ();
-			ReadXmlSchema (reader);
-			reader.MoveToContent ();
-			ReadXml (reader, XmlReadMode.DiffGram);
-			reader.MoveToContent ();
-			reader.ReadEndElement ();
+			ReadXml (reader);
 		}
 
 		void IXmlSerializable.ReadXml (XmlReader reader)
@@ -1058,7 +1181,7 @@ namespace System.Data {
 
 		XmlSchema IXmlSerializable.GetSchema ()
 		{
-			return BuildSchema ();
+			return GetSchemaSerializable ();
 		}
 
 		protected virtual bool ShouldSerializeRelations ()
@@ -1098,9 +1221,61 @@ namespace System.Data {
 		}
 		#endregion
 
-		#region Private Xml Serialisation
+		#region Private Methods
 
-		private string WriteObjectXml (object o)
+		private XmlReadMode ReadXmlIgnoreSchema (XmlReader input, XmlReadMode mode, bool checkRecurse)
+		{
+			if (input.LocalName == "schema" &&
+				input.NamespaceURI == XmlSchema.Namespace) {
+				input.Skip ();
+			}
+			else if (input.LocalName == "diffgram" &&
+				input.NamespaceURI == XmlConstants.DiffgrNamespace) {
+				XmlDiffLoader DiffLoader = new XmlDiffLoader (this);
+				DiffLoader.Load (input);
+			}
+			else if (checkRecurse ||
+				input.LocalName == DataSetName &&
+				input.NamespaceURI == Namespace) {
+				XmlDataReader.ReadXml (this, input, mode);
+			}
+			else if (checkRecurse && !input.IsEmptyElement) {
+				int depth = input.Depth;
+				input.Read ();
+				input.MoveToContent ();
+				ReadXmlIgnoreSchema (input, mode, false);
+				while (input.Depth > depth)
+					input.Skip ();
+				if (input.NodeType == XmlNodeType.EndElement)
+					input.ReadEndElement ();
+			}
+			input.MoveToContent ();
+			return XmlReadMode.IgnoreSchema;
+		}
+
+		private XmlReadMode ReadXmlReadSchema (XmlReader input, XmlReadMode mode, bool checkRecurse)
+		{
+			if (input.LocalName == "schema" &&
+				input.NamespaceURI == XmlSchema.Namespace) {
+				ReadXmlSchema (input);
+			}
+			else if (checkRecurse && !input.IsEmptyElement) {
+				int depth = input.Depth;
+				input.Read ();
+				input.MoveToContent ();
+				ReadXmlReadSchema (input, mode, false);
+				while (input.Depth > depth)
+					input.Skip ();
+				if (input.NodeType == XmlNodeType.EndElement)
+					input.ReadEndElement ();
+			}
+			else
+				input.Skip ();
+			input.MoveToContent ();
+			return XmlReadMode.ReadSchema;
+		}
+
+		internal static string WriteObjectXml (object o)
 		{
 			switch (Type.GetTypeCode (o.GetType ())) {
 				case TypeCode.Boolean:
@@ -1165,7 +1340,10 @@ namespace System.Data {
 			WriteTable (writer, rows, mode, version, true);
 		}
 
-		private void WriteTable (XmlWriter writer, DataRow[] rows, XmlWriteMode mode, DataRowVersion version, bool skipIfNested)
+		private void WriteTable (XmlWriter writer,
+			DataRow [] rows,
+			XmlWriteMode mode,
+			DataRowVersion version, bool skipIfNested)
 		{
 			//The columns can be attributes, hidden, elements, or simple content
 			//There can be 0-1 simple content cols or 0-* elements
@@ -1188,16 +1366,22 @@ namespace System.Data {
 					case 0:
 						break;
 					case 1:
+						if (!oneRel.Nested)
+							break;
 						if (row.GetParentRow (oneRel) != null)
 							continue;
 						break;
 					case 2:
 						bool skip = false;
-						for (int i = 0; i < table.ParentRelations.Count; i++)
-							if (row.GetParentRow (table.ParentRelations [i]) != null) {
+						for (int i = 0; i < table.ParentRelations.Count; i++) {
+							DataRelation prel = table.ParentRelations [i];
+							if (!prel.Nested)
+								continue;
+							if (row.GetParentRow (prel) != null) {
 								skip = true;
 								continue;
 							}
+						}
 						if (skip)
 							continue;
 						break;
@@ -1270,7 +1454,8 @@ namespace System.Data {
 
 		private void WriteColumnAsAttribute (XmlWriter writer, XmlWriteMode mode, DataColumn col, DataRow row, DataRowVersion version)
 		{
-			WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), WriteObjectXml (row[col, version]));
+			if (!row.IsNull (col))
+				WriteAttributeString (writer, mode, col.Namespace, col.Prefix, XmlConvert.EncodeLocalName (col.ColumnName), WriteObjectXml (row[col, version]));
 		}
 
 		private void WriteTableElement (XmlWriter writer, XmlWriteMode mode, DataTable table, DataRow row, DataRowVersion version)
@@ -1316,8 +1501,6 @@ namespace System.Data {
 		
 		internal void WriteIndividualTableContent (XmlWriter writer, DataTable table, XmlWriteMode mode)
 		{
-			((XmlTextWriter)writer).Formatting = Formatting.Indented;
-
 			if (mode == XmlWriteMode.DiffGram) {
 				SetTableRowsID (table);
 				WriteDiffGramElement (writer);
@@ -1340,493 +1523,18 @@ namespace System.Data {
 			writer.WriteEndElement (); // DataSet name or diffgr:diffgram
 		}
 		
-
-		private void CheckNamespace (string prefix, string ns, XmlNamespaceManager nsmgr, XmlSchema schema)
-		{
-			if (ns == String.Empty)
-				return;
-			if (ns != nsmgr.DefaultNamespace) {
-				if (nsmgr.LookupNamespace (nsmgr.NameTable.Get (prefix)) != ns) {
-					for (int i = 1; i < int.MaxValue; i++) {
-						string p = nsmgr.NameTable.Add ("app" + i);
-						if (!nsmgr.HasNamespace (p)) {
-							nsmgr.AddNamespace (p, ns);
-							HandleExternalNamespace (p, ns, schema);
-							break;
-						}
-					}
-				}
-			}
-		}
-		
-		XmlSchema BuildSchema ()
-		{
-			return BuildSchema (Tables, Relations);
-		}
-		
-		internal XmlSchema BuildSchema (DataTableCollection tables, DataRelationCollection relations)
-		{
-			string constraintPrefix = "";
-			XmlSchema schema = new XmlSchema ();
-			XmlNamespaceManager nsmgr = new XmlNamespaceManager (new NameTable ());
-			
-			if (Namespace != "") {
-				schema.AttributeFormDefault = XmlSchemaForm.Qualified;
-				schema.ElementFormDefault = XmlSchemaForm.Qualified;
-				schema.TargetNamespace = Namespace;
-				constraintPrefix = XmlConstants.TnsPrefix + ":";
-			}
-
-			// set the schema id
-			string xmlNSURI = "http://www.w3.org/2000/xmlns/";
-			schema.Id = DataSetName;
-			XmlDocument doc = new XmlDocument ();
-			XmlAttribute attr = null;
-			ArrayList atts = new ArrayList ();
-
-			nsmgr.AddNamespace ("xs", XmlSchema.Namespace);
-			nsmgr.AddNamespace (XmlConstants.MsdataPrefix, XmlConstants.MsdataNamespace);
-			if (Namespace != "") {
-				nsmgr.AddNamespace (XmlConstants.TnsPrefix, Namespace);
-				nsmgr.AddNamespace (String.Empty, Namespace);
-			}
-			if (CheckExtendedPropertyExists ())
-				nsmgr.AddNamespace (XmlConstants.MspropPrefix, XmlConstants.MspropNamespace);
-
-			if (atts.Count > 0)
-				schema.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
-
-			XmlSchemaElement elem = new XmlSchemaElement ();
-			elem.Name = XmlConvert.EncodeName (DataSetName);
-
-			// Add namespaces used in DataSet components (tables, columns, ...)
-			foreach (DataTable dt in Tables) {
-				foreach (DataColumn col in dt.Columns)
-					CheckNamespace (col.Prefix, col.Namespace, nsmgr, schema);
-				CheckNamespace (dt.Prefix, dt.Namespace, nsmgr, schema);
-			}
-
-			// Attributes for DataSet element
-			atts.Clear ();
-			attr = doc.CreateAttribute (XmlConstants.MsdataPrefix,  XmlConstants.IsDataSet, XmlConstants.MsdataNamespace);
-			attr.Value = "true";
-			atts.Add (attr);
-
-			attr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.Locale, XmlConstants.MsdataNamespace);
-			attr.Value = locale.Name;
-			atts.Add (attr);
-
-			elem.UnhandledAttributes = atts.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
-
-			AddExtendedPropertyAttributes (elem, ExtendedProperties, doc);
-
-			XmlSchemaComplexType complex = new XmlSchemaComplexType ();
-			elem.SchemaType = complex;
-
-			XmlSchemaChoice choice = new XmlSchemaChoice ();
-			complex.Particle = choice;
-			choice.MaxOccursString = XmlConstants.Unbounded;
-			
-			//Write out schema for each table in order
-			foreach (DataTable table in tables) {		
-				bool isTopLevel = true;
-				foreach (DataRelation rel in table.ParentRelations) {
-					if (rel.Nested) {
-						isTopLevel = false;
-						break;
-					}
-				}
-				
-				if (isTopLevel) {
-					if (table.Namespace != SafeNS (schema.TargetNamespace)) {
-						XmlSchemaElement extElem = new XmlSchemaElement ();
-						extElem.RefName = new XmlQualifiedName (table.TableName, table.Namespace);
-						choice.Items.Add (extElem);
-					}
-					else
-						choice.Items.Add (GetTableSchema (doc, table, schema, nsmgr));
-				}
-			}
-
-			schema.Items.Add (elem);
-			
-			AddConstraintsToSchema (elem, constraintPrefix, tables, relations, doc);
-			foreach (string prefix in nsmgr) {
-				string ns = nsmgr.LookupNamespace (nsmgr.NameTable.Get (prefix));
-				if (prefix != "xmlns" && prefix != "xml" && ns != null && ns != String.Empty)
-					schema.Namespaces.Add (prefix, ns);
-			}
-			return schema;
-		}
-		
-		private bool CheckExtendedPropertyExists ()
-		{
-			if (ExtendedProperties.Count > 0)
-				return true;
-			foreach (DataTable dt in Tables) {
-				if (dt.ExtendedProperties.Count > 0)
-					return true;
-				foreach (DataColumn col in dt.Columns)
-					if (col.ExtendedProperties.Count > 0)
-						return true;
-				foreach (Constraint c in dt.Constraints)
-					if (c.ExtendedProperties.Count > 0)
-						return true;
-			}
-			foreach (DataRelation rel in Relations)
-				if (rel.ExtendedProperties.Count > 0)
-					return true;
-			return false;
-		}
-
-		// Add all constraints in all tables to the schema.
-		private void AddConstraintsToSchema (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, DataRelationCollection relations, XmlDocument doc)
-		{
-			// first add all unique constraints.
-			Hashtable uniqueNames = AddUniqueConstraints (elem, constraintPrefix, tables, doc);
-			// Add all foriegn key constraints.
-			AddForeignKeys (uniqueNames, elem, constraintPrefix, relations, doc);
-		}
-		
-		// Add unique constaraints to the schema.
-		// return hashtable with the names of all XmlSchemaUnique elements we created.
-		private Hashtable AddUniqueConstraints (XmlSchemaElement elem, string constraintPrefix, DataTableCollection tables, XmlDocument doc)
-		{
-			Hashtable uniqueNames = new Hashtable();
-			foreach (DataTable table in tables) {
-				
-				foreach (Constraint constraint in table.Constraints) {
-					
-					if (constraint is UniqueConstraint) {
-						ArrayList attrs = new ArrayList ();
-						XmlAttribute attrib;
-						UniqueConstraint uqConst = (UniqueConstraint) constraint;
-						XmlSchemaUnique uniq = new XmlSchemaUnique ();
-						
-						// if column of the constraint is hidden do not write the constraint.
-						bool isHidden = false;
-						foreach (DataColumn column in uqConst.Columns) {
-							if (column.ColumnMapping == MappingType.Hidden) {
-								isHidden = true;
-								break;
-							}
-						}
-
-						if (isHidden)
-							continue;
-
-						// if constaraint name do not exist in the hashtable we can use it.
-						if (!uniqueNames.ContainsKey (uqConst.ConstraintName)) {
-							uniq.Name = uqConst.ConstraintName;
-						}
-						// generate new constraint name for the XmlSchemaUnique element.
-						else {
-							uniq.Name = uqConst.Table.TableName + "_" + uqConst.ConstraintName;
-							attrib = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.ConstraintName, XmlConstants.MsdataNamespace);
-							attrib.Value = uqConst.ConstraintName;
-							attrs.Add (attrib);
-						}
-						if (uqConst.IsPrimaryKey) {
-							attrib = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.PrimaryKey, XmlConstants.MsdataNamespace);
-							attrib.Value = "true";
-							attrs.Add (attrib);
-						}
-		
-						uniq.UnhandledAttributes = (XmlAttribute[])attrs.ToArray (typeof (XmlAttribute));
-
-						uniq.Selector = new XmlSchemaXPath();
-						uniq.Selector.XPath = ".//"+constraintPrefix + uqConst.Table.TableName;
-						XmlSchemaXPath field;
-						foreach (DataColumn column in uqConst.Columns) {
-				 			field = new XmlSchemaXPath();
-							field.XPath = constraintPrefix+column.ColumnName;
-							uniq.Fields.Add(field);
-						}
-				
-						AddExtendedPropertyAttributes (uniq, constraint.ExtendedProperties, doc);
-
-						elem.Constraints.Add (uniq);
-						uniqueNames.Add (uniq.Name, null);
-					}
-				}
-			}
-			return uniqueNames;
-		}
-		
-		// Add the foriegn keys to the schema.
-		private void AddForeignKeys (Hashtable uniqueNames, XmlSchemaElement elem, string constraintPrefix, DataRelationCollection relations, XmlDocument doc)
-		{
-			if (relations == null) return;
-			
-			foreach (DataRelation rel in relations) {
-				
-				if (rel.ParentKeyConstraint == null || rel.ChildKeyConstraint == null)
-					continue;
-
-				bool isHidden = false;
-				foreach (DataColumn col in rel.ParentColumns) {
-					if (col.ColumnMapping == MappingType.Hidden) {
-						isHidden = true;
-						break;
-					}
-				}
-				foreach (DataColumn col in rel.ChildColumns) {
-					if (col.ColumnMapping == MappingType.Hidden) {
-						isHidden = true;
-						break;
-					}
-				}
-				if (isHidden)
-					continue;
-				
-				ArrayList attrs = new ArrayList ();
-				XmlAttribute attrib;
-				XmlSchemaKeyref keyRef = new XmlSchemaKeyref();
-				keyRef.Name = rel.RelationName;
-				ForeignKeyConstraint fkConst = rel.ChildKeyConstraint;
-				UniqueConstraint uqConst = rel.ParentKeyConstraint;
-				
-				string concatName = rel.ParentTable.TableName + "_" + uqConst.ConstraintName;
-				// first try to find the concatenated name. If we didn't find it - use constraint name.
-				if (uniqueNames.ContainsKey (concatName)) {
-					keyRef.Refer = new XmlQualifiedName(concatName);
-				}
-				else {
-					keyRef.Refer = new XmlQualifiedName(uqConst.ConstraintName);
-				}
-
-				if (rel.Nested)	{
-					attrib = doc.CreateAttribute (XmlConstants.MsdataPrefix,  XmlConstants.IsNested, XmlConstants.MsdataNamespace);
-					attrib.Value = "true";
-					attrs.Add (attrib);
-				}
-
-				keyRef.Selector = new XmlSchemaXPath();
-				keyRef.Selector.XPath = ".//" + constraintPrefix + rel.ChildTable.TableName;
-				XmlSchemaXPath field;
-				foreach (DataColumn column in rel.ChildColumns)	{
-				 	field = new XmlSchemaXPath();
-					field.XPath = constraintPrefix+column.ColumnName;
-					keyRef.Fields.Add(field);
-				}
-
-				keyRef.UnhandledAttributes = (XmlAttribute[])attrs.ToArray (typeof (XmlAttribute));
-				AddExtendedPropertyAttributes (keyRef, rel.ExtendedProperties, doc);
-
-				elem.Constraints.Add (keyRef);
-			}
-		}
-
-		private XmlSchemaElement GetTableSchema (XmlDocument doc, DataTable table, XmlSchema schemaToAdd, XmlNamespaceManager nsmgr)
-		{
-			ArrayList elements;
-			ArrayList atts;
-			DataColumn simple;
-			
-			SplitColumns (table, out atts, out elements, out simple);
-
-			XmlSchemaElement elem = new XmlSchemaElement ();
-			elem.Name = table.TableName;
-
-			XmlSchemaComplexType complex = new XmlSchemaComplexType ();
-			elem.SchemaType = complex;
-
-			XmlSchemaObjectCollection schemaAttributes = null;
-
-			if (simple != null) {
-				// add simpleContent
-				XmlSchemaSimpleContent simpleContent = new XmlSchemaSimpleContent();
-				complex.ContentModel = simpleContent;
-
-				// add column name attribute
-				XmlAttribute[] xlmAttrs = new XmlAttribute [2];
-				xlmAttrs[0] = doc.CreateAttribute (XmlConstants.MsdataPrefix,  XmlConstants.ColumnName, XmlConstants.MsdataNamespace);
-				xlmAttrs[0].Value = simple.ColumnName;
-				
-				// add ordinal attribute
-				xlmAttrs[1] = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.Ordinal, XmlConstants.MsdataNamespace);
-				xlmAttrs[1].Value = XmlConvert.ToString (simple.Ordinal);
-				simpleContent.UnhandledAttributes = xlmAttrs;
-				
-			        
-				// add extension
-				XmlSchemaSimpleContentExtension extension = new XmlSchemaSimpleContentExtension();
-				simpleContent.Content = extension;
-				extension.BaseTypeName = MapType (simple.DataType);
-				schemaAttributes = extension.Attributes;
-			} else {
-				schemaAttributes = complex.Attributes;
-				//A sequence of element types or a simple content node
-				//<xs:sequence>
-				XmlSchemaSequence seq = new XmlSchemaSequence ();
-
-				foreach (DataColumn col in elements) {
-					
-					// Add element for the column.
-					XmlSchemaElement colElem = new XmlSchemaElement ();
-					ArrayList xattrs = new ArrayList();
-					XmlAttribute xattr;
-					colElem.Name = col.ColumnName;
-				
-					if (col.ColumnName != col.Caption && col.Caption != String.Empty) {
-						xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.Caption, XmlConstants.MsdataNamespace);
-						xattr.Value = col.Caption;
-						xattrs.Add (xattr);
-					}
-
-					if (col.AutoIncrement == true) {
-						xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.AutoIncrement, XmlConstants.MsdataNamespace);
-						xattr.Value = "true";
-						xattrs.Add (xattr);
-					}
-
-					if (col.AutoIncrementSeed != 0) {
-						xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.AutoIncrementSeed, XmlConstants.MsdataNamespace);
-						xattr.Value = XmlConvert.ToString (col.AutoIncrementSeed);
-						xattrs.Add (xattr);
-					}
-
-					if (col.DefaultValue.ToString () != String.Empty)
-						colElem.DefaultValue = WriteObjectXml (col.DefaultValue);
-					
-					if (col.MaxLength < 0)
-						colElem.SchemaTypeName = MapType (col.DataType);
-					
-					if (colElem.SchemaTypeName == XmlConstants.QnString && col.DataType != typeof (string) 
-						&& col.DataType != typeof (char)) {
-						xattr = doc.CreateAttribute (XmlConstants.MsdataPrefix, XmlConstants.DataType, XmlConstants.MsdataNamespace);
-						xattr.Value = col.DataType.AssemblyQualifiedName;
-						xattrs.Add (xattr);
-					}
-
-					if (col.AllowDBNull) {
-						colElem.MinOccurs = 0;
-					}
-
-					//writer.WriteAttributeString (XmlConstants.MsdataPrefix, 
-					//                            XmlConstants.Ordinal, 
-					//                            XmlConstants.MsdataNamespace, 
-					//                            col.Ordinal.ToString ());
-
-					// Write SimpleType if column have MaxLength
-					if (col.MaxLength > -1) {
-						colElem.SchemaType = GetTableSimpleType (doc, col);
-					}
-					
-					colElem.UnhandledAttributes = (XmlAttribute[])xattrs.ToArray(typeof (XmlAttribute));
-					AddExtendedPropertyAttributes (colElem, col.ExtendedProperties, doc);
-					seq.Items.Add (colElem);
-				}
-
-				foreach (DataRelation rel in table.ChildRelations) {
-					if (rel.Nested) {
-						if (rel.ChildTable.Namespace != SafeNS (schemaToAdd.TargetNamespace)) {
-							XmlSchemaElement el = new XmlSchemaElement ();
-							el.RefName = new XmlQualifiedName (rel.ChildTable.TableName, rel.ChildTable.Namespace);
-						} else {
-							XmlSchemaElement el = GetTableSchema (doc, rel.ChildTable, schemaToAdd, nsmgr);
-							el.MinOccurs = 0;
-							el.MaxOccursString = "unbounded";
-							XmlSchemaComplexType ct = (XmlSchemaComplexType) el.SchemaType;
-							ct.Name = el.Name;
-							el.SchemaType = null;
-							el.SchemaTypeName = new XmlQualifiedName (ct.Name, schemaToAdd.TargetNamespace);
-							schemaToAdd.Items.Add (ct);
-							seq.Items.Add (el);
-						}
-					}
-				}
-
-				if (seq.Items.Count > 0)
-					complex.Particle = seq;
-			}
-
-			//Then a list of attributes
-			foreach (DataColumn col in atts) {
-				//<xs:attribute name=col.ColumnName form="unqualified" type=MappedType/>
-				XmlSchemaAttribute att = new XmlSchemaAttribute ();
-				att.Name = col.ColumnName;
-				if (col.Namespace != String.Empty) {
-					att.Form = XmlSchemaForm.Qualified;
-					string prefix = col.Prefix == String.Empty ? "app" + schemaToAdd.Namespaces.Count : col.Prefix;
-					att.Name = prefix + ":" + col.ColumnName;
-					// FIXME: Handle prefix mapping correctly.
-					schemaToAdd.Namespaces.Add (prefix, col.Namespace);
-				}
-				att.SchemaTypeName = MapType (col.DataType);
-				// FIXME: what happens if extended properties are set on attribute columns??
-				schemaAttributes.Add (att);
-			}
-
-			AddExtendedPropertyAttributes (elem, table.ExtendedProperties, doc);
-
-			return elem;
-		}
-
-		private void AddExtendedPropertyAttributes (XmlSchemaAnnotated xsobj, PropertyCollection props, XmlDocument doc)
-		{
-			ArrayList attList = new ArrayList ();
-			XmlAttribute xmlAttr;
-
-			if (xsobj.UnhandledAttributes != null)
-				attList.AddRange (xsobj.UnhandledAttributes);
-
-			// add extended properties to xs:element
-			foreach (DictionaryEntry de in props) {
-				xmlAttr = doc.CreateAttribute (XmlConstants.MspropPrefix, XmlConvert.EncodeName (de.Key.ToString ()), XmlConstants.MspropNamespace);
-				xmlAttr.Value = de.Value != null ? WriteObjectXml (de.Value) : String.Empty;
-				attList.Add (xmlAttr);
-			}
-			if (attList.Count > 0)
-				xsobj.UnhandledAttributes = attList.ToArray (typeof (XmlAttribute)) as XmlAttribute [];
- 		}
-
-		private string SafeNS (string ns)
-		{
-			return ns != null ? ns : String.Empty;
-		}
-
-		private void HandleExternalNamespace (string prefix, string ns, XmlSchema schema)
-		{
-			foreach (XmlSchemaExternal ext in schema.Includes) {
-				XmlSchemaImport imp = ext as XmlSchemaImport;
-				if (imp != null && imp.Namespace == ns)
-					return; // nothing to do
-			}
-			XmlSchemaImport i = new XmlSchemaImport ();
-			i.Namespace = ns;
-			i.SchemaLocation = "_" + prefix + ".xsd";
-			schema.Includes.Add (i);
-		}
-
-		private XmlSchemaSimpleType GetTableSimpleType (XmlDocument doc, DataColumn col)
-		{
-			// SimpleType
-			XmlSchemaSimpleType simple = new XmlSchemaSimpleType ();
-
-			// Restriction
-			XmlSchemaSimpleTypeRestriction restriction = new XmlSchemaSimpleTypeRestriction ();
-			restriction.BaseTypeName = MapType (col.DataType);
-			
-			// MaxValue
-			XmlSchemaMaxLengthFacet max = new XmlSchemaMaxLengthFacet ();
-			max.Value = XmlConvert.ToString (col.MaxLength);
-			restriction.Facets.Add (max);
-			
-			simple.Content = restriction;
-			return simple;
-		}
-
 		private void DoWriteXmlSchema (XmlWriter writer)
 		{
-			BuildSchema ().Write (writer);
+			if (writer.WriteState == WriteState.Start)
+				writer.WriteStartDocument ();
+			XmlSchemaWriter.WriteXmlSchema (this, writer);
 		}
 		
 		///<summary>
 		/// Helper function to split columns into attributes elements and simple
 		/// content
 		/// </summary>
-		private void SplitColumns (DataTable table, 
+		internal static void SplitColumns (DataTable table, 
 			out ArrayList atts, 
 			out ArrayList elements, 
 			out DataColumn simple)
@@ -1879,40 +1587,6 @@ namespace System.Data {
 				dataRowID++;
 			}
 		}
-
-		
-		private XmlQualifiedName MapType (Type type)
-		{
-			switch (Type.GetTypeCode (type)) {
-				case TypeCode.String: return XmlConstants.QnString;
-				case TypeCode.Int16: return XmlConstants.QnShort;
-				case TypeCode.Int32: return XmlConstants.QnInt;
-				case TypeCode.Int64: return XmlConstants.QnLong;
-				case TypeCode.Boolean: return XmlConstants.QnBoolean;
-				case TypeCode.Byte: return XmlConstants.QnUnsignedByte;
-				//case TypeCode.Char: return XmlConstants.QnChar;
-				case TypeCode.DateTime: return XmlConstants.QnDateTime;
-				case TypeCode.Decimal: return XmlConstants.QnDecimal;
-				case TypeCode.Double: return XmlConstants.QnDouble;
-				case TypeCode.SByte: return XmlConstants.QnSbyte;
-				case TypeCode.Single: return XmlConstants.QnFloat;
-				case TypeCode.UInt16: return XmlConstants.QnUsignedShort;
-				case TypeCode.UInt32: return XmlConstants.QnUnsignedInt;
-				case TypeCode.UInt64: return XmlConstants.QnUnsignedLong;
-			}
-			
-			if (typeof (TimeSpan) == type)
-				return XmlConstants.QnDuration;
-			else if (typeof (System.Uri) == type)
-				return XmlConstants.QnUri;
-			else if (typeof (byte[]) == type)
-				return XmlConstants.QnBase64Binary;
-			else if (typeof (XmlQualifiedName) == type)
-				return XmlConstants.QnXmlQualifiedName;
-			else
-				return XmlConstants.QnString;
-		}
-
 		#endregion //Private Xml Serialisation
 	}
 }
