@@ -30,7 +30,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
+
 
 namespace System.Data {
 	public enum FillOptions 
@@ -42,4 +42,4 @@ namespace System.Data {
 	}
 }
 
-#endif // NET_2_0
+
