@@ -22,6 +22,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 using System;
+using System.Collections;
 
 namespace System.Data
 {
@@ -37,11 +38,13 @@ namespace System.Data
 			if(sourceSet == null)
 				throw new ArgumentNullException("sourceSet");
 
+			bool prevEC = targetSet.EnforceConstraints;
+			targetSet.EnforceConstraints = false;
 			foreach (DataTable t in sourceSet.Tables)
 				MergeManager.Merge(targetSet, t, preserveChanges, missingSchemaAction);
 
 			AdjustSchema(targetSet,sourceSet,missingSchemaAction);
-
+			targetSet.EnforceConstraints = prevEC;
 		}
 
 		internal static void Merge(DataSet targetSet, DataTable sourceTable, bool preserveChanges, MissingSchemaAction missingSchemaAction)
@@ -51,6 +54,9 @@ namespace System.Data
 			if(sourceTable == null)
 				throw new ArgumentNullException("sourceTable");
 
+			bool savedEnfoceConstraints = targetSet.EnforceConstraints;
+			targetSet.EnforceConstraints = false;
+
 			DataTable targetTable = null;
 			if (!AdjustSchema(targetSet, sourceTable, missingSchemaAction,ref targetTable)) {
 				return;
@@ -58,7 +64,13 @@ namespace System.Data
 			if (targetTable != null) {
 				checkColumnTypes(targetTable, sourceTable); // check that the colums datatype is the same
 				fillData(targetTable, sourceTable, preserveChanges);
-			}			
+			}
+			targetSet.EnforceConstraints = savedEnfoceConstraints;
+			
+			if (!targetSet.EnforceConstraints && targetTable != null) {
+				// indexes are still outdated
+				targetTable.ResetIndexes();
+			}
 		}
 
 		internal static void Merge(DataSet targetSet, DataRow[] sourceRows, bool preserveChanges, MissingSchemaAction missingSchemaAction)
@@ -68,6 +80,10 @@ namespace System.Data
 			if(sourceRows == null)
 				throw new ArgumentNullException("sourceRows");
 
+			bool savedEnfoceConstraints = targetSet.EnforceConstraints;
+			targetSet.EnforceConstraints = false;
+
+			ArrayList targetTables = new ArrayList();
 			for (int i = 0; i < sourceRows.Length; i++) {
 				DataRow row = sourceRows[i];
 				DataTable sourceTable = row.Table;
@@ -78,8 +94,17 @@ namespace System.Data
 				if (targetTable != null) {
 					checkColumnTypes(targetTable, row.Table);
 					MergeRow(targetTable, row, preserveChanges);
+					if (!(targetTables.IndexOf(targetTable) >= 0)) {
+						targetTables.Add(targetTable);
+					}
 				}
 			}
+
+			targetSet.EnforceConstraints = savedEnfoceConstraints;
+
+			foreach(DataTable table in targetTables) {
+				table.ResetIndexes();
+			}
 		}
 
 		// merge a row into a target table.
@@ -103,15 +128,17 @@ namespace System.Data
 				}
 			
 				// find the row in the target table.
-				targetRow = targetTable.Rows.Find(keyValues);
+				targetRow = targetTable.Rows.Find(keyValues, DataViewRowState.OriginalRows);
+				if (targetRow == null)
+					targetRow = targetTable.Rows.Find(keyValues);
 			}
 			// row doesn't exist in target table, or there are no primary keys.
 			// create new row and copy values from source row to the new row.
 			if (targetRow == null)
 			{ 
-				targetRow = targetTable.NewRow();
-				row.CopyValuesToRow(targetRow);
-				targetTable.Rows.Add(targetRow);
+				DataRow newRow = targetTable.NewNotInitializedRow();
+				row.CopyValuesToRow(newRow);
+				targetTable.Rows.AddInternal (newRow);
 			}
 			// row exists in target table, and presere changes is false - 
 			// change the values of the target row to the values of the source row.
@@ -119,7 +146,6 @@ namespace System.Data
 			{
 				row.CopyValuesToRow(targetRow);
 			}
-
 		}
 			
 
@@ -138,6 +164,7 @@ namespace System.Data
 						DataColumn[] childColumns = ResolveColumns(sourceSet,targetTable,relation.ChildColumns);
 						if (parentColumns != null && childColumns != null) {
 							DataRelation newRelation = new DataRelation(relation.RelationName,parentColumns,childColumns);
+							newRelation.Nested = relation.Nested; 
 							targetSet.Relations.Add(newRelation);
 						}
 					}
@@ -150,33 +177,37 @@ namespace System.Data
 					DataTable targetTable = targetSet.Tables[sourceTable.TableName];
 
 					if (targetTable != null) {
-						foreach(UniqueConstraint uc in sourceTable.Constraints.UniqueConstraints) {
-							// TODO : add more precise condition (columns)
-							if ( !targetTable.Constraints.Contains(uc.ConstraintName) ) {		
-								DataColumn[] columns = ResolveColumns(sourceSet,targetTable,uc.Columns);
-								if (columns != null) {
-									UniqueConstraint newConstraint = new UniqueConstraint(uc.ConstraintName,columns,uc.IsPrimaryKey);
-									targetTable.Constraints.Add(newConstraint);
-								}
-							}
-							else {
-								// TODO : should we throw an exeption ?
-							}
-						}
+						foreach(Constraint constraint in sourceTable.Constraints) {
 
-						foreach(ForeignKeyConstraint fc in sourceTable.Constraints.ForeignKeyConstraints) {
-							// TODO : add more precise condition (columns)
-							if (!targetTable.Constraints.Contains(fc.ConstraintName)) {
-								DataColumn[] columns = ResolveColumns(sourceSet,targetTable,fc.Columns);
-								DataTable relatedTable = targetSet.Tables[fc.RelatedTable.TableName];
-								DataColumn[] relatedColumns = ResolveColumns(sourceSet,relatedTable,fc.RelatedColumns);
-								if (columns != null && relatedColumns != null) {
-									ForeignKeyConstraint newConstraint = new ForeignKeyConstraint(fc.ConstraintName,relatedColumns,columns);
-									targetTable.Constraints.Add(newConstraint);
+							if (constraint is UniqueConstraint) {
+								UniqueConstraint uc = (UniqueConstraint)constraint;
+								// FIXME : add more precise condition (columns)
+								if ( !targetTable.Constraints.Contains(uc.ConstraintName) ) {		
+									DataColumn[] columns = ResolveColumns(sourceSet,targetTable,uc.Columns);
+									if (columns != null) {
+										UniqueConstraint newConstraint = new UniqueConstraint(uc.ConstraintName,columns,uc.IsPrimaryKey);
+										targetTable.Constraints.Add(newConstraint);
+									}
+								}
+								else {
+									// FIXME : should we throw an exception ?
 								}
 							}
 							else {
-								// TODO : should we throw an exeption ?
+								ForeignKeyConstraint fc = (ForeignKeyConstraint)constraint;
+								// FIXME : add more precise condition (columns)
+								if (!targetTable.Constraints.Contains(fc.ConstraintName)) {
+									DataColumn[] columns = ResolveColumns(sourceSet,targetTable,fc.Columns);
+									DataTable relatedTable = targetSet.Tables[fc.RelatedTable.TableName];
+									DataColumn[] relatedColumns = ResolveColumns(sourceSet,relatedTable,fc.RelatedColumns);
+									if (columns != null && relatedColumns != null) {
+										ForeignKeyConstraint newConstraint = new ForeignKeyConstraint(fc.ConstraintName,relatedColumns,columns);
+										targetTable.Constraints.Add(newConstraint);
+									}
+								}
+								else {
+									// FIXME : should we throw an exception ?
+								}
 							}
 						}
 					}
