@@ -256,15 +256,11 @@ namespace System.Data
 			XmlSchema s = new XmlSchema ();
 			XmlSchemaAttribute a = new XmlSchemaAttribute ();
 			a.Name = "foo";
-			// FIXME: mcs looks to have a bug around static 
-			// reference resolution. XmlSchema.Namespace should work.
-			a.SchemaTypeName = new XmlQualifiedName ("integer", System.Xml.Schema.XmlSchema.Namespace);
+			a.SchemaTypeName = new XmlQualifiedName ("integer", XmlSchema.Namespace);
 			s.Items.Add (a);
 			XmlSchemaAttribute b = new XmlSchemaAttribute ();
 			b.Name = "bar";
-			// FIXME: mcs looks to have a bug around static 
-			// reference resolution. XmlSchema.Namespace should work.
-			b.SchemaTypeName = new XmlQualifiedName ("decimal", System.Xml.Schema.XmlSchema.Namespace);
+			b.SchemaTypeName = new XmlQualifiedName ("decimal", XmlSchema.Namespace);
 			s.Items.Add (b);
 			XmlSchemaElement e = new XmlSchemaElement ();
 			e.Name = "bar";
@@ -302,8 +298,7 @@ namespace System.Data
 			this.dataset = dataset;
 			dataset.DataSetName = "NewDataSet"; // Initialize always
 			schema = XmlSchema.Read (reader, null);
-			// FIXME: Just XmlSchema.Namespace should work (mcs bug)
-			if (reader.NodeType == XmlNodeType.EndElement && reader.LocalName == "schema" && reader.NamespaceURI == System.Xml.Schema.XmlSchema.Namespace)
+			if (reader.NodeType == XmlNodeType.EndElement && reader.LocalName == "schema" && reader.NamespaceURI == XmlSchema.Namespace)
 				reader.ReadEndElement ();
 			schema.Compile (null);
 		}
@@ -384,13 +379,10 @@ el.ElementType != schemaAnyType)
 					return true; // column element
 				if (ct.AttributeUses.Count > 0)
 					return false; // table element
-				switch (ct.ContentType) {
-				case XmlSchemaContentType.Empty:
-				case XmlSchemaContentType.TextOnly:
+				if (ct.ContentType == XmlSchemaContentType.TextOnly)
 					return true; // column element
-				default:
+				else
 					return false; // table element
-				}
 			}
 			XmlSchemaGroupBase gb = p as XmlSchemaGroupBase;
 			for (int i = 0; i < gb.Items.Count; i++) {
@@ -569,8 +561,9 @@ el.ElementType != schemaAnyType)
 				XmlSchemaElement el = p as XmlSchemaElement;
 				if (el != null)
 					ImportColumnElement (parent, el);
-				else
+				else if (p is XmlSchemaGroupBase)
 					ImportColumnGroupBase (parent, (XmlSchemaGroupBase) p);
+				// otherwise p is xs:any
 			}
 		}
 
@@ -654,6 +647,12 @@ el.ElementType != schemaAnyType)
 					case XmlConstants.AutoIncrementSeed:
 						col.AutoIncrementSeed = int.Parse (attr.Value);
 						break;
+					case XmlConstants.AutoIncrementStep:
+						col.AutoIncrementStep = int.Parse (attr.Value);
+						break;
+					case XmlConstants.ReadOnly:
+						col.ReadOnly = XmlConvert.ToBoolean (attr.Value);
+						break;
 					case XmlConstants.Ordinal:
 						ordinal = int.Parse (attr.Value);
 						break;
