@@ -229,22 +229,18 @@ namespace MonoTests.System.Data
                                 table2.Constraints.Add (fkc);
                                 throw new ApplicationException ("An Exception was expected");
                         }
-#if NET_1_1
-                        catch (InvalidOperationException) {
-			}
-#else
+                        // LAMESPEC : spec says InvalidConstraintException but throws this
                         catch (ArgumentException) {
                         }
-#endif
-                                                                                                    
+
+#if false // FIXME: Here this test crashes under MS.NET.
                         // OK - So AddRange() is the only way!
-			// FIXME: Here this test crashes.
                         table2.Constraints.AddRange (constraints);
 			   // After AddRange(), Check the properties of ForeignKeyConstraint object
                         Assertion.Assert("#A04", fkc.RelatedColumns [0].ColumnName.Equals ("col1"));                        Assertion.Assert("#A05", fkc.RelatedColumns [1].ColumnName.Equals ("col2"));                        Assertion.Assert("#A06", fkc.RelatedColumns [2].ColumnName.Equals ("col3"));                        Assertion.Assert("#A07", fkc.Columns [0].ColumnName.Equals ("col4"));
                         Assertion.Assert("#A08", fkc.Columns [1].ColumnName.Equals ("col5"));
                         Assertion.Assert("#A09", fkc.Columns [2].ColumnName.Equals ("col6"));
-                                                                                                    
+#endif
                         // Try to add columns with names which do not exist in the table
                         parentColumnNames [2] = "noColumn";
                         ForeignKeyConstraint foreignKeyConstraint = new ForeignKeyConstraint ("hello world", parentTableName, parentColumnNames, childColumnNames, AcceptRejectRule.Cascade, Rule.Cascade, Rule.Cascade);
@@ -255,11 +251,15 @@ namespace MonoTests.System.Data
                                 table2.Constraints.AddRange (constraints);
                                 throw new ApplicationException ("An Exception was expected");
                         }
-                        catch (Exception e){
-                                Assertion.AssertEquals ("#A04", "System.Data.InvalidConstraintException", e.GetType().ToString());
+                        catch (ArgumentException e) {
                         }
+                        catch (InvalidConstraintException e){ // Could not test on ms.net, as ms.net does not reach here so far.        
+                        }
+                        
+#if false // FIXME: Here this test crashes under MS.NET.
                         // Check whether the child table really contains the foreign key constraint named "hello world"
-                        Assertion.Assert("#A10 ", table2.Constraints.Contains ("hello world"));
+                        Assertion.Assert("#A11 ", table2.Constraints.Contains ("hello world"));
+#endif
                 }
 
 
