@@ -44,10 +44,10 @@ namespace System.DirectoryServices
 	/// </summary>
 	public class DirectorySearcher : Component	
 	{
-
+		private static readonly TimeSpan DefaultTimeSpan = new TimeSpan(-TimeSpan.TicksPerSecond);
 		private DirectoryEntry _SearchRoot=null;
 		private bool _CacheResults=true;
-		private TimeSpan _ClientTimeout = new TimeSpan(-1);
+		private TimeSpan _ClientTimeout = DefaultTimeSpan;
 		private string _Filter="(objectClass=*)";
 		private int _PageSize=0;
 		private StringCollection _PropertiesToLoad=new StringCollection();
@@ -56,23 +56,22 @@ namespace System.DirectoryServices
 						System.DirectoryServices.ReferralChasingOption.External;
 		private SearchScope _SearchScope=
 						System.DirectoryServices.SearchScope.Subtree;
-		private TimeSpan _ServerPageTimeLimit=new TimeSpan(-1);
-		private int _SizeLimit=1000;
+		private TimeSpan _ServerPageTimeLimit = DefaultTimeSpan;
+		private TimeSpan _serverTimeLimit = DefaultTimeSpan;
+		private int _SizeLimit=0;
 		private LdapConnection _conn = null;
 		private string _Host=null;
 		private int _Port=389;
 		private SearchResultCollection _SrchColl=null;
-		private bool emptycoll=true;
 
 		internal SearchResultCollection SrchColl 
 		{
 			get
 			{
-				if (emptycoll)
+				if (_SrchColl == null)
 				{
 					_SrchColl =  new SearchResultCollection();
 					DoSearch();
-					emptycoll=false;
 				}
 				return _SrchColl;
 			}
@@ -111,7 +110,7 @@ namespace System.DirectoryServices
 			}
 			set
 			{
-				CacheResults = value;
+				_CacheResults = value;
 			}
 		}
 
@@ -177,6 +176,7 @@ namespace System.DirectoryServices
 			set
 			{
 				_Filter = value;
+				ClearCachedResults();
 			}
 		}
 
@@ -299,6 +299,7 @@ namespace System.DirectoryServices
 			set
 			{
 				_SearchRoot = value;
+				ClearCachedResults();
 			}
 		}
 
@@ -321,6 +322,7 @@ namespace System.DirectoryServices
 			set
 			{
 				_SearchScope =  value;
+				ClearCachedResults();
 			}
 		}
 
@@ -371,12 +373,12 @@ namespace System.DirectoryServices
 			[MonoTODO]
 			get
 			{
-				throw new NotImplementedException();
+				return _serverTimeLimit;
 			}
 			[MonoTODO]
 			set
 			{
-				throw new NotImplementedException();
+				_serverTimeLimit = value;
 			}
 		}
 
@@ -406,6 +408,9 @@ namespace System.DirectoryServices
 			}
 			set
 			{
+				if (value < 0) {
+					throw new ArgumentException();
+				}
 				_SizeLimit =  value;
 			}
 		}
@@ -597,6 +602,10 @@ namespace System.DirectoryServices
 		/// </returns>
 		public SearchResult FindOne()
 		{
+			// TBD : should search for no more than single result
+			if (SrchColl.Count == 0) {
+				return null;
+			}
 			return SrchColl[0];
 		}
 
@@ -614,17 +623,47 @@ namespace System.DirectoryServices
 		private void DoSearch()
 		{
 			InitBlock();
+			if (!PropertiesToLoad.Contains("ADsPath")) {
+				PropertiesToLoad.Add("ADsPath");
+			}
 			String[] attrs= new String[PropertiesToLoad.Count];
 			PropertiesToLoad.CopyTo(attrs,0);
+			
+			LdapSearchConstraints cons = _conn.SearchConstraints;
+			if (SizeLimit > 0) {
+				cons.MaxResults = SizeLimit;
+			}
+			if (ServerTimeLimit != DefaultTimeSpan) {
+				cons.ServerTimeLimit = (int)ServerTimeLimit.TotalSeconds;
+			}
+
+			int connScope = LdapConnection.SCOPE_SUB;
+			switch (_SearchScope)
+			{
+			case SearchScope.Base:
+			  connScope = LdapConnection.SCOPE_BASE;
+			  break;
+
+			case SearchScope.OneLevel:
+			  connScope = LdapConnection.SCOPE_ONE;
+			  break;
+
+			case SearchScope.Subtree:
+			  connScope = LdapConnection.SCOPE_SUB;
+			  break;
+
+			default:
+			  connScope = LdapConnection.SCOPE_SUB;
+			  break;
+			}
 			LdapSearchResults lsc=_conn.Search(	SearchRoot.Fdn,
-												LdapConnection.SCOPE_SUB,
+												connScope,
 												Filter,
 												attrs,
-												false);
+												false,cons);
 
 			while(lsc.hasMore())						
 			{
-
 				LdapEntry nextEntry = null;
 				try 							
 				{
@@ -632,9 +671,14 @@ namespace System.DirectoryServices
 				}
 				catch(LdapException e) 							
 				{
-					Console.WriteLine("Error: " + e.LdapErrorMessage);
-					// Exception is thrown, go for next entry
-					throw e;
+					switch (e.ResultCode) {
+						// in case of this return codes exception should not be thrown
+						case LdapException.SIZE_LIMIT_EXCEEDED:
+						case LdapException.TIME_LIMIT_EXCEEDED:
+							continue;
+						default :
+							throw e;
+					}
 				}
 				DirectoryEntry de = new DirectoryEntry(_conn);
 				PropertyCollection pcoll = new PropertyCollection();
@@ -653,16 +697,29 @@ namespace System.DirectoryServices
 //						de.Properties[attributeName].Mbit=false;
 					}
 				}
+				if (!pcoll.Contains("ADsPath")) {
+					pcoll["ADsPath"].Add(de.Path);
+				}
 //				_SrchColl.Add(new SearchResult(de,PropertiesToLoad));
 				_SrchColl.Add(new SearchResult(de,pcoll));
 			}
-		return;
+			return;
 		}
 
 		[MonoTODO]
 		protected override void Dispose(bool disposing)
 		{
-			throw new NotImplementedException();
+			if (disposing) {
+				if(_conn.Connected) {
+					_conn.Disconnect();
+				}
+			}
+			base.Dispose(disposing);
+		}
+
+		private void ClearCachedResults()
+		{
+			_SrchColl = null;
 		}
 
 	}
