@@ -51,34 +51,41 @@ namespace System.Runtime.Remoting.Channels.Http
 
 	internal class RequestArguments
 	{
+		static int count;
+		Socket socket;
+		
 		public RequestArguments (Socket socket, HttpServerTransportSink sink)
 		{
+			Id = count++;
 			NetworkStream ns = new NetworkStream (socket);
-			InputStream = ns;
-			OutputStream = ns;
+			this.Stream = ns;
+			this.socket = socket;
 			Sink = sink;
 		}
 		
-		public RequestArguments (Stream inputStream, Stream outputStream, HttpServerTransportSink sink)
+		public void Process ()
 		{
-			InputStream = inputStream;
-			OutputStream = outputStream;
-			Sink = sink;
+			HttpServer.ProcessRequest (this);
 		}
-		public Stream InputStream;
-		public Stream OutputStream;
+		
+		public IPAddress ClientAddress
+		{
+			get {
+				IPEndPoint ep = socket.RemoteEndPoint as IPEndPoint;
+				if (ep != null) return ep.Address;
+				else return null;
+			}
+		}
+		
+		public int Id;
+		public Stream Stream;
 		public HttpServerTransportSink Sink;
 	}
 
 	internal sealed class HttpServer
 	{
-		public static void ProcessRequest (object reqInfo)
+		public static void ProcessRequest (RequestArguments reqArg)
 		{
-			if(reqInfo as RequestArguments == null)
-				return;
-
-			RequestArguments reqArg = (RequestArguments)reqInfo;
-		
 			//Step (1) Start Reciceve the header
 			ArrayList  Headers = RecieveHeader (reqArg);
 				
@@ -105,9 +112,8 @@ namespace System.Runtime.Remoting.Channels.Http
 			else
 				buffer = new byte [0];
 				
-			
 			//Step (5)
-		    SendRequestForChannel (reqArg, HeaderFields, CustomHeaders, buffer);
+			SendRequestForChannel (reqArg, HeaderFields, CustomHeaders, buffer);
 		}
 
 		private static ArrayList RecieveHeader (RequestArguments reqArg)
@@ -118,7 +124,7 @@ namespace System.Runtime.Remoting.Channels.Http
 			byte[] buffer = new byte[1024];
 			ArrayList  Headers = new ArrayList();
 			
-			Stream ist = reqArg.InputStream;
+			Stream ist = reqArg.Stream;
 
 			int index =0;
 			while (!bLastLine)
@@ -205,7 +211,7 @@ namespace System.Runtime.Remoting.Channels.Http
 			{
 				int nr = 0;
 				while (nr < buffer.Length)
-					nr += reqArg.InputStream.Read (buffer, nr, buffer.Length - nr);
+					nr += reqArg.Stream.Read (buffer, nr, buffer.Length - nr);
 			}
 			catch (SocketException e)
 			{
@@ -215,8 +221,7 @@ namespace System.Runtime.Remoting.Channels.Http
 						SendResponse (reqArg, 408, null, null);
 						break;
 					default :
-						//<Exception>					
-						break;
+						throw e;
 				}
 				
 				return false;
@@ -241,6 +246,8 @@ namespace System.Runtime.Remoting.Channels.Http
 			THeaders[CommonTransportKeys.UserAgent] = HeaderFields["user-agent"];
 			THeaders[CommonTransportKeys.Host] = HeaderFields["host"];
 			THeaders[CommonTransportKeys.SoapAction] = HeaderFields["SOAPAction"];
+			THeaders[CommonTransportKeys.IPAddress] = reqArg.ClientAddress;
+			THeaders[CommonTransportKeys.ConnectionId] = reqArg.Id;
 
 			foreach(DictionaryEntry DictEntry in CustomHeaders)
 			{
@@ -261,7 +268,7 @@ namespace System.Runtime.Remoting.Channels.Http
 			
 		}
 		
-		public static bool SendResponse (RequestArguments reqArg, int httpStatusCode, ITransportHeaders headers, Stream responseStream)
+		public static void SendResponse (RequestArguments reqArg, int httpStatusCode, ITransportHeaders headers, Stream responseStream)
 		{
 			byte [] headersBuffer = null;
 			byte [] entityBuffer = null;
@@ -309,22 +316,11 @@ namespace System.Runtime.Remoting.Channels.Http
 		
 		   	headersBuffer = Encoding.ASCII.GetBytes (responseStr.ToString());
 
-			try
-			{
-				//send headersBuffer
-				reqArg.OutputStream.Write (headersBuffer, 0, headersBuffer.Length);
+			//send headersBuffer
+			reqArg.Stream.Write (headersBuffer, 0, headersBuffer.Length);
 
-				if (entityBuffer != null)
-					reqArg.OutputStream.Write (entityBuffer, 0, entityBuffer.Length);
-			}
-			catch
-			{
-				//<EXCEPTION>
-				//may be its the client's fault so just return with false
-				return false;
-			}
-
-			return true;
+			if (entityBuffer != null)
+				reqArg.Stream.Write (entityBuffer, 0, entityBuffer.Length);
 		}
 
 		internal static string GetReasonPhrase (int HttpStatusCode)
