@@ -2,11 +2,10 @@
 // X509KeyUsageFlags.cs - System.Security.Cryptography.X509Certificates.X509KeyUsageFlags
 //
 // Author:
-//	Sebastien Pouliot (spouliot@motus.com)
+//	Sebastien Pouliot  <sebastien@ximian.com>
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-//
-
+// Copyright (C) 2004-2005 Novell Inc. (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,23 +29,21 @@
 
 #if NET_2_0
 
-using System;
-
 namespace System.Security.Cryptography.X509Certificates {
 
-	// Note: Match the definition of framework version 1.2.3400.0 on http://longhorn.msdn.microsoft.com
-
+	[Flags]
 	public enum X509KeyUsageFlags {
-		EncipherOnly,
-		CRLSign,
-		KeyCertSign,
-		KeyAgreement,
-		DataEncipherment,
-		KeyEncipherment,
-		NonRepudiation,
-		DigitalSignature,
-		DecipherOnly
+		None = 0,
+		EncipherOnly = 1,
+		CrlSign = 2,
+		KeyCertSign = 4,
+		KeyAgreement = 8,
+		DataEncipherment = 16,
+		KeyEncipherment = 32,
+		NonRepudiation = 64,
+		DigitalSignature = 128,
+		DecipherOnly = 32768
 	}
 }
 
-#endif
\ No newline at end of file
+#endif
