@@ -4,11 +4,11 @@
 // Author:
 //	Sebastien Pouliot <sebastien@ximian.com>
 //	Atsushi Enomoto <atsushi@ximian.com>
+//      Tim Coleman <tim@timcoleman.com>
 //
 // (C) 2002, 2003 Motus Technologies Inc. (http://www.motus.com)
-// (C) 2004 Novell (http://www.novell.com)
-//
-
+// Copyright (C) Tim Coleman, 2004
+// Copyright (C) 2004-2005 Novell Inc. (http://www.novell.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,6 +30,8 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
+using System.Collections;
+using System.IO;
 using System.Runtime.InteropServices;
 using System.Security.Policy;
 using System.Xml;
@@ -51,6 +53,8 @@ namespace System.Security.Cryptography.Xml {
 #endif
 		}
 
+		#region Properties
+
 		public string Algorithm {
 			get { return algo; }
 			set { algo = value; }
@@ -64,6 +68,39 @@ namespace System.Security.Cryptography.Xml {
 			get;
 		}
 
+#if NET_1_1
+		[ComVisible(false)]
+		public XmlResolver Resolver {
+			set { xmlResolver = value; }
+		}
+#endif
+
+#if NET_2_0
+		[MonoTODO]
+		[ComVisible (false)]
+		public XmlElement Context {
+			get { throw new NotImplementedException (); }
+			set { throw new NotImplementedException (); }
+		}
+
+		[MonoTODO]
+		[ComVisible (false)]
+		public Hashtable PropagatedNamespaces {
+			get { throw new NotImplementedException (); }
+		}
+#endif
+
+		#endregion // Properties
+
+		#region Methods
+#if NET_2_0
+		[ComVisible (false)]
+		public virtual byte[] GetDigestedOutput (HashAlgorithm hash)
+		{
+			return hash.ComputeHash ((Stream) GetOutput (typeof (Stream)));
+		}
+#endif
+
 		protected abstract XmlNodeList GetInnerXml ();
 
 		public abstract object GetOutput ();
@@ -95,11 +132,6 @@ namespace System.Security.Cryptography.Xml {
 			return xmlResolver;
 		}
 
-#if NET_1_1
-		[ComVisible(false)]
-		public XmlResolver Resolver {
-			set { xmlResolver = value; }
-		}
-#endif
+		#endregion // Methods
 	}
 }
