@@ -32,14 +32,23 @@ using System.Web.Services.Configuration;
 using System.Xml;
 using System.Xml.Serialization;
 
-namespace System.Web.Services.Description {
+namespace System.Web.Services.Description 
+{
 	[XmlFormatExtensionPoint ("Extensions")]
-	public sealed class Binding : DocumentableItem {
+	public sealed class Binding :
+#if NET_2_0
+		NamedItem
+#else
+		DocumentableItem 
+#endif
+	{
 
 		#region Fields
 
 		ServiceDescriptionFormatExtensionCollection extensions;
+#if !NET_2_0
 		string name;
+#endif
 		OperationBindingCollection operations;
 		ServiceDescription serviceDescription;
 		XmlQualifiedName type;
@@ -51,7 +60,9 @@ namespace System.Web.Services.Description {
 		public Binding ()
 		{
 			extensions = new ServiceDescriptionFormatExtensionCollection (this);
+#if !NET_2_0
 			name = String.Empty;
+#endif
 			operations = new OperationBindingCollection (this);
 			serviceDescription = null;
 			type = XmlQualifiedName.Empty;
@@ -61,23 +72,24 @@ namespace System.Web.Services.Description {
 
 		#region Properties
 
-//		[XmlIgnore]
+		[XmlIgnore]
 		public ServiceDescriptionFormatExtensionCollection Extensions { 	
 			get { return extensions; }
 		}
 
+#if !NET_2_0
 		[XmlAttribute ("name", DataType = "NCName")]	
 		public string Name {
 			get { return name; }
 			set { name = value; }
 		}
+#endif
 
 		[XmlElement ("operation")]
 		public OperationBindingCollection Operations {
 			get { return operations; }
 		}
 
-		[XmlIgnore]
 		public ServiceDescription ServiceDescription {
 			get { return serviceDescription; }
 		}
