@@ -77,8 +77,18 @@ namespace System.Web.Services.Description {
 			if (WSConfig.IsSupported (WSProtocol.HttpGet))
 				new HttpGetProtocolReflector ().Reflect (this, type, url, schemaExporter, soapSchemaExporter);
 			
+#if NET_1_1
+			if (WSConfig.IsSupported (WSProtocol.HttpPost) || WSConfig.IsSupported (WSProtocol.HttpPostLocalhost))
+#else
 			if (WSConfig.IsSupported (WSProtocol.HttpPost))
+#endif
 				new HttpPostProtocolReflector ().Reflect (this, type, url, schemaExporter, soapSchemaExporter);
+				
+			int i=0;
+			while (i < types.Schemas.Count) {
+				if (types.Schemas[i].Items.Count == 0) types.Schemas.RemoveAt (i);
+				else i++;
+			}
 			
 			if (serviceDescriptions.Count == 1)
 				serviceDescriptions[0].Types = types;
