@@ -126,10 +126,15 @@ namespace System.Web.Services.Protocols {
 
 				Action = rma.Action;
 				Binding = rma.Binding;
-				RequestName = rma.RequestElementName;
+				
+				// When using RPC, MS.NET seems to ignore RequestElementName and
+				// MessageName, and it always uses the method name
+				RequestName = source.Name;
+				ResponseName = source.Name + "Response";
+//				RequestName = rma.RequestElementName;
+//				ResponseName = rma.ResponseElementName;
 				RequestNamespace = rma.RequestNamespace;
 				ResponseNamespace = rma.ResponseNamespace;
-				ResponseName = rma.ResponseElementName;
 				ParameterStyle = SoapParameterStyle.Wrapped;
 				OneWay = rma.OneWay;
 				SoapBindingStyle = SoapBindingStyle.Rpc;
@@ -156,7 +161,7 @@ namespace System.Web.Services.Protocols {
 			if (RequestName == "") RequestName = Name;
 			if (ResponseName == "")	ResponseName = Name + "Response";
 			if (Action == null || Action == "")
-				Action = RequestNamespace.EndsWith("/") ? (RequestNamespace + Name) : (RequestNamespace + "/" + Name);
+				Action = serviceNamespace.EndsWith("/") ? (serviceNamespace + Name) : (serviceNamespace + "/" + Name);
 			
 			bool hasWrappingElem = (ParameterStyle == SoapParameterStyle.Wrapped);
 			bool writeAccessors = (SoapBindingStyle == SoapBindingStyle.Rpc);
@@ -179,16 +184,20 @@ namespace System.Web.Services.Protocols {
 			responseSerializerId = parent.RegisterSerializer (OutputMembersMapping);
 
 			object[] o = source.GetCustomAttributes (typeof (SoapHeaderAttribute));
-			Headers = new HeaderInfo[o.Length];
+			ArrayList headerList = new ArrayList (o.Length);
+			
 			for (int i = 0; i < o.Length; i++) {
 				SoapHeaderAttribute att = (SoapHeaderAttribute) o[i];
 				MemberInfo[] mems = source.DeclaringType.GetMember (att.MemberName);
 				if (mems.Length == 0) throw new InvalidOperationException ("Member " + att.MemberName + " not found in class " + source.DeclaringType.FullName + ".");
 				
-				Type headerType = (mems[0] is FieldInfo) ? ((FieldInfo)mems[0]).FieldType : ((PropertyInfo)mems[0]).PropertyType;
-				Headers [i] = new HeaderInfo (mems[0], att);
-				parent.RegisterHeaderType (headerType, serviceNamespace, Use);
+				HeaderInfo header = new HeaderInfo (mems[0], att);
+				headerList.Add (header);
+
+				if (!header.IsUnknownHeader)
+					parent.RegisterHeaderType (header.HeaderType, serviceNamespace, Use);
 			}
+			Headers = (HeaderInfo[]) headerList.ToArray (typeof(HeaderInfo));
 
 			SoapExtensions = SoapExtension.GetMethodExtensions (source);
 		}
@@ -272,6 +281,7 @@ namespace System.Web.Services.Protocols {
 		internal MemberInfo Member;
 		internal SoapHeaderAttribute AttributeInfo;
 		internal Type HeaderType;
+		internal bool IsUnknownHeader;
 
 		public HeaderInfo (MemberInfo member, SoapHeaderAttribute attributeInfo)
 		{
@@ -279,6 +289,14 @@ namespace System.Web.Services.Protocols {
 			AttributeInfo = attributeInfo;
 			if (Member is PropertyInfo) HeaderType = ((PropertyInfo)Member).PropertyType;
 			else HeaderType = ((FieldInfo)Member).FieldType;
+			
+			if (HeaderType == typeof(SoapHeader) || HeaderType == typeof(SoapUnknownHeader) ||
+				HeaderType == typeof(SoapHeader[]) || HeaderType == typeof(SoapUnknownHeader[]))
+			{
+				IsUnknownHeader = true;
+			}
+			else if (!typeof(SoapHeader).IsAssignableFrom (HeaderType))
+				throw new InvalidOperationException (string.Format ("Header members type must be a SoapHeader subclass"));
 		}
 		
 		public object GetHeaderValue (object ob)
@@ -287,12 +305,28 @@ namespace System.Web.Services.Protocols {
 			else return ((FieldInfo)Member).GetValue (ob);
 		}
 
-		public void SetHeaderValue (object ob, object value)
+		public void SetHeaderValue (object ob, SoapHeader header)
 		{
+			object value = header;
+			if (IsUnknownHeader && HeaderType.IsArray)
+			{
+				SoapUnknownHeader uheader = header as SoapUnknownHeader;
+				SoapUnknownHeader[] array = (SoapUnknownHeader[]) GetHeaderValue (ob);
+				if (array == null || array.Length == 0) {
+					value = new SoapUnknownHeader[] { uheader };
+				}
+				else {
+					SoapUnknownHeader[] newArray = new SoapUnknownHeader [array.Length+1];
+					Array.Copy (array, newArray, array.Length);
+					newArray [array.Length] = uheader;
+					value = newArray;
+				}
+			}
+			
 			if (Member is PropertyInfo) ((PropertyInfo)Member).SetValue (ob, value, null);
 			else ((FieldInfo)Member).SetValue (ob, value);
 		}
-
+		
 		public SoapHeaderDirection Direction
 		{
 			get { return AttributeInfo.Direction; }
@@ -463,8 +497,11 @@ namespace System.Web.Services.Protocols {
 		{
 			Hashtable table = header_serializers_byname [(int)use];
 			if (table == null) return null;
+			
+			object serId = table [qname];
+			if (serId == null) return null;
 				
-			return GetSerializer ((int) table [qname]);
+			return GetSerializer ((int) serId);
 		}		
 		
 		public SoapMethodStubInfo GetMethodForSoapAction (string name)
