@@ -151,6 +151,9 @@ namespace System.Web.Services.Protocols
 			catch (Exception ex) {
 				ainfo.SetCompleted (null, ex, true);
 			}
+			finally {
+				response.Close();
+			}
 		}
 
 		protected object[] EndInvoke (IAsyncResult asyncResult)
@@ -244,6 +247,9 @@ namespace System.Web.Services.Protocols
 					msg = String.Format (msg, (int) code, code);
 					throw new WebException (msg, null, WebExceptionStatus.ProtocolError, http_response);
 				}
+				if (message.OneWay && response.ContentLength == 0 && (code == HttpStatusCode.Accepted || code == HttpStatusCode.OK)) {
+					return new object[0];
+				}
 			}
 			
 			//
@@ -269,13 +275,16 @@ namespace System.Web.Services.Protocols
 			
 			// Deserialize the response
 
-			StreamReader reader = new StreamReader (stream, encoding, false);
-			XmlTextReader xml_reader = new XmlTextReader (reader);
-
 			SoapHeaderCollection headers;
 			object content;
 
-			WebServiceHelper.ReadSoapMessage (xml_reader, type_info, msi.Use, msi.ResponseSerializer, out content, out headers);
+			using (StreamReader reader = new StreamReader (stream, encoding, false)) {
+				XmlTextReader xml_reader = new XmlTextReader (reader);
+
+				WebServiceHelper.ReadSoapMessage (xml_reader, type_info, msi.Use, msi.ResponseSerializer,
+								out content, out headers);
+			}
+
 			
 			if (content is Fault)
 			{
@@ -324,7 +333,12 @@ namespace System.Web.Services.Protocols
 					throw ex;
 			}
 
-			return ReceiveResponse (response, message, extensions);
+			try {
+				return ReceiveResponse (response, message, extensions);
+			}
+			finally {
+				response.Close();
+			}
 		}
 		
 #if NET_2_0
