@@ -93,6 +93,14 @@ namespace System.Web.Services.Protocols {
 			get { return url; }
 		}
 
+#if NET_2_0
+		[MonoTODO]
+		[System.Runtime.InteropServices.ComVisible(false)]
+		public override SoapProtocolVersion SoapVersion {
+			get { throw new NotImplementedException (); }
+		}
+#endif
+
 		#endregion // Properties
 
 		#region Methods
