@@ -71,7 +71,7 @@ namespace System.Web.Caching {
 			} while (bytePos < 60);
 
 			// GC Bucket controller
-			_intFlush = System.DateTime.Now.Minute - 1;
+			_intFlush = System.DateTime.UtcNow.Minute - 1;
 			_objTimer = new System.Threading.Timer (new System.Threading.TimerCallback (GarbageCleanup), null, 10000, 60000);
 		}
 
@@ -80,7 +80,7 @@ namespace System.Web.Caching {
 		/// </summary>
 		/// <param name="objEntry">Cache entry to add.</param>
 		internal void Add (CacheEntry objEntry) {
-			long now = DateTime.Now.Ticks;
+			long now = DateTime.UtcNow.Ticks;
 			if (objEntry.Expires < now)
 				objEntry.Expires = now;
 
@@ -92,25 +92,7 @@ namespace System.Web.Caching {
 				_arrBuckets [objEntry.ExpiresBucket].Remove (objEntry);
 		}
 
-		internal void Update (CacheEntry objEntry, long ticksExpires) {
-			// If the entry doesn't have a expires time we assume that the entry is due to expire now.
-			int oldBucket = objEntry.ExpiresBucket;
-			int newBucket = GetHashBucket (ticksExpires);
-
-			if (oldBucket == CacheEntry.NoBucketHash)
-				return;
-
-			// Check if we need to move the item
-			if (oldBucket != newBucket) {
-				_arrBuckets [oldBucket].Remove (objEntry);
-				objEntry.Expires = ticksExpires;
-				_arrBuckets [newBucket].Add (objEntry);
-			} else
-				_arrBuckets [oldBucket].Update (objEntry, ticksExpires);
-		}
-
 		internal void GarbageCleanup (object State) {
-			ExpiresBucket objBucket;
 			int bucket;
 
 			// We lock here if FlushExpiredItems take time
