@@ -30,6 +30,7 @@
 
 using System;
 using System.Collections;
+using System.Globalization;
 
 namespace System.Web.Compilation
 {
@@ -65,6 +66,10 @@ namespace System.Web.Compilation
 
 		static string [] application_atts = { "description", "inherits", "codebehind" };
 
+#if NET_2_0
+		static string [] mastertype_atts = { "virtualpath", "typename" };
+#endif
+		
 		static Directive ()
 		{
 			InitHash ();
@@ -72,8 +77,8 @@ namespace System.Web.Compilation
 		
 		private static void InitHash ()
 		{
-			CaseInsensitiveHashCodeProvider provider = new CaseInsensitiveHashCodeProvider ();
-			CaseInsensitiveComparer comparer =  new CaseInsensitiveComparer ();
+			CaseInsensitiveHashCodeProvider provider = new CaseInsensitiveHashCodeProvider (CultureInfo.InvariantCulture);
+			CaseInsensitiveComparer comparer =  new CaseInsensitiveComparer (CultureInfo.InvariantCulture);
 
 			directivesHash = new Hashtable (provider, comparer); 
 
@@ -122,6 +127,16 @@ namespace System.Web.Compilation
 			valid_attributes = new Hashtable (provider, comparer);
 			foreach (string att in application_atts) valid_attributes.Add (att, null);
 			directivesHash.Add ("APPLICATION", valid_attributes);
+
+#if NET_2_0
+			valid_attributes = new Hashtable (provider, comparer);
+			foreach (string att in mastertype_atts) valid_attributes.Add (att, null);
+			directivesHash.Add ("MASTERTYPE", valid_attributes);
+			
+			valid_attributes = new Hashtable (provider, comparer);
+			foreach (string att in control_atts) valid_attributes.Add (att, null);
+			directivesHash.Add ("MASTER", valid_attributes);
+#endif
 		}
 		
 		private Directive () { }
