@@ -67,8 +67,8 @@ namespace System.Web.Configuration
 			if (assemblies == null)
 				assemblies = new ArrayList ();
 
-			if (temp_directory == null)
-				temp_directory = Path.GetTempPath ();
+			if (temp_directory == null || temp_directory == "")
+				temp_directory = AppDomain.CurrentDomain.SetupInformation.DynamicBase;
 		}
 
 		static public CompilationConfiguration GetInstance (HttpContext context)
@@ -87,21 +87,21 @@ namespace System.Web.Configuration
 		
 		public CodeDomProvider GetProvider (string language)
 		{
-			WebCompiler compiler = Compilers [language];
+			Compiler compiler = Compilers [language];
 			if (compiler == null)
 				return null;
 
 			if (compiler.Provider != null)
 				return compiler.Provider;
 
-			Type t = Type.GetType (compiler.Type);
+			Type t = Type.GetType (compiler.Type, true);
 			compiler.Provider = Activator.CreateInstance (t) as CodeDomProvider;
 			return compiler.Provider;
 		}
 
 		public string GetCompilerOptions (string language)
 		{
-			WebCompiler compiler = Compilers [language];
+			Compiler compiler = Compilers [language];
 			if (compiler == null)
 				return null;
 
@@ -110,7 +110,7 @@ namespace System.Web.Configuration
 
 		public int GetWarningLevel (string language)
 		{
-			WebCompiler compiler = Compilers [language];
+			Compiler compiler = Compilers [language];
 			if (compiler == null)
 				return 0;
 
@@ -187,6 +187,7 @@ namespace System.Web.Configuration
 				if (value != null && !Directory.Exists (value))
 					throw new ArgumentException ("Directory does not exist");
 
+				Console.WriteLine ("Dos: '{0}'\n{1}", value, Environment.StackTrace);
 				temp_directory = value;
 			}
 		}
