@@ -45,14 +45,15 @@ namespace System.Web.Configuration
 		static WebDefaultConfig config;
 		static string machineConfigPath;
 		const BindingFlags privStatic = BindingFlags.NonPublic | BindingFlags.Static;
-			
+		static readonly object lockobj = new object ();
+		
 		private WebConfigurationSettings ()
 		{
 		}
 
 		public static void Init ()
 		{
-			lock (typeof (WebConfigurationSettings)) {
+			lock (lockobj) {
 				if (config != null)
 					return;
 
@@ -88,7 +89,7 @@ namespace System.Web.Configuration
 
 		public static string MachineConfigPath {
 			get {
-				lock (typeof (WebConfigurationSettings)) {
+				lock (lockobj) {
 					if (machineConfigPath != null)
 						return machineConfigPath;
 
@@ -148,7 +149,7 @@ namespace System.Web.Configuration
 			if (context == null)
 				return null;
 
-			ConfigurationData config = GetConfigFromFileName (context.Request.FilePath, context);
+			ConfigurationData config = GetConfigFromFileName (context.Request.CurrentExecutionFilePath, context);
 			if (config == null)
 				return null;
 
@@ -213,6 +214,9 @@ namespace System.Web.Configuration
 
 		public void Init (HttpContext context)
 		{
+			if (initCalled)
+				return;
+
 			lock (this) {
 				if (initCalled)
 					return;
@@ -226,18 +230,6 @@ namespace System.Web.Configuration
 				initCalled = true;
 			}
 		}
-
-		static string GetAppConfigPath ()
-		{
-			AppDomainSetup currentInfo = AppDomain.CurrentDomain.SetupInformation;
-
-			string configFile = currentInfo.ConfigurationFile;
-			if (configFile == null || configFile.Length == 0)
-				return null;
-
-			return configFile;
-
-		}
 	}
 
         class FileWatcherCache
@@ -251,7 +243,7 @@ namespace System.Web.Configuration
                 public FileWatcherCache (ConfigurationData data)
                 {
 			this.data = data;
-                        cacheTable = Hashtable.Synchronized (new Hashtable ());
+			cacheTable = new Hashtable ();
 			this.path = Path.GetDirectoryName (data.FileName);
 			this.filename = Path.GetFileName (data.FileName);
 			if (!Directory.Exists (path))
@@ -268,10 +260,6 @@ namespace System.Web.Configuration
 		void SetChanged (object o, FileSystemEventArgs args)
 		{
 			lock (data) {
-				if (watcher.Filter == "*.config" &&
-				    String.Compare (Path.GetFileName (args.FullPath), "web.config", true) != 0)
-					return;
-				
 				cacheTable.Clear ();
 				data.Reset ();
 				if (args.ChangeType == WatcherChangeTypes.Created)
@@ -334,7 +322,6 @@ namespace System.Web.Configuration
 		Hashtable locations;
 		string fileName;
 		string dirname;
-		string realdir;
 		static object removedMark = new object ();
 		static object groupMark = new object ();
                 static object emptyMark = new object ();
@@ -396,7 +383,6 @@ namespace System.Web.Configuration
 		public ConfigurationData (ConfigurationData parent, string filename, string realdir)
 		{
 			this.parent = (parent == this) ? null : parent;
-			this.realdir = realdir;
 			if (filename == null) {
 				this.fileName = Path.Combine (realdir, "*.config");
 			} else {
@@ -532,17 +518,16 @@ namespace System.Web.Configuration
 			XmlDocument doc = GetDocumentForSection (sectionName);
 			if (doc == null || doc.DocumentElement == null)
 				return parentConfig;
-			
+
 			return iconf.Create (parentConfig, fileName, doc.DocumentElement);
 		}
 
 		public object GetConfig (string sectionName, HttpContext context)
 		{
 			if (locations != null && dirname != null) {
-				string reduced = UrlUtils.MakeRelative (context.Request.FilePath, dirname);
+				string reduced = UrlUtils.MakeRelative (context.Request.CurrentExecutionFilePath, dirname);
 				string [] parts = reduced.Split ('/');
 				Location location = null;
-				int length = parts.Length;
 
 				string target = null;
 				for (int i = 0; i < parts.Length; i++) {
