@@ -31,6 +31,9 @@ namespace System.Web.Mail
 
 	/// <summary>
 	/// </summary>
+#if NET_2_0
+        [Serializable]
+#endif
 	public enum MailFormat 
 	{
 
