@@ -4,6 +4,9 @@
 // Author:
 //    Lawrence Pit (loz@cable.a2000.nl)
 //    Per Arneng (pt99par@student.bth.se)
+//	Sanjay Gupta (gsanjay@novell.com)
+//
+//     (c)2004 Novell, Inc. (http://www.novell.com)
 //
 
 //
@@ -60,6 +63,9 @@ namespace System.Web.Mail
 			bodyEncoding = Encoding.Default;
 #if NET_1_1
 			fields = new Hashtable ();
+#endif
+#if NET_2_0
+			bodyParts = new ArrayList (2);
 #endif
 		}		
 	
@@ -136,6 +142,15 @@ namespace System.Web.Mail
 			}
 		}
 #endif
+
+#if NET_2_0
+		private ArrayList bodyParts;
+		
+		public IList RelatedBodyParts {
+			get { return (IList) bodyParts; }
+		}
+#endif
+
 	}
 	
 } //namespace System.Web.Mail
