@@ -3,9 +3,10 @@
 //
 // Author(s):
 //   Per Arneng <pt99par@student.bth.se>
+//   Sanjay Gupta <gsanjay@novell.com>
 //
+//   (C) 2004, Novell, Inc. (http://www.novell.com)
 //
-
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -51,8 +52,8 @@ namespace System.Web.Mail {
 	    this.message = message;
 	    
 	    if( message.From != null ) {
-		from = MailAddress.Parse( message.From );
-		header.From = from.ToString();
+			from = MailAddress.Parse( message.From );
+			header.From = from.ToString();
 	    }
 	    
 	    if( message.To != null ) {
@@ -69,8 +70,7 @@ namespace System.Web.Mail {
 		bcc = MailAddressCollection.Parse( message.Bcc );
 		header.Bcc = bcc.ToString();
 	    }
-
-	    
+   
 	    // set the subject
 	    if( message.Subject != null ) {
 		
@@ -163,7 +163,13 @@ namespace System.Web.Mail {
 		header.ContentTransferEncoding = "8bit";
 	    }
 
-	    
+	    // Add Date header, we were missing earlier 27/08/04
+	    // RFC822 requires date to be in format Fri, 27 Aug 2004 20:13:20 +0530
+	    //DateTime.Now gives in format 8/27/2004 8:13:00 PM
+	    // Need to explore further dateTime formats available or do we need
+	    // to write a function to convert.
+		//header.Data.Add ("Date", DateTime.Now.ToString()); 
+
 	    // Add the custom headers
 	    foreach( string key in message.Headers.Keys )
 		header.Data[ key ] = (string)this.message.Headers[ key ];
@@ -229,17 +235,23 @@ namespace System.Web.Mail {
 	}
 
 #if NET_1_1
-	public MailHeader Fields {
-		get {
-			MailHeader bodyHeaders = new MailHeader();
-			// Add Fields to MailHeader Object
-			foreach( string key in message.Fields.Keys )
-				bodyHeaders.Data[ key ] = (string)this.message.Fields[ key ];
-			return bodyHeaders;
+		public MailHeader Fields {
+			get {
+					MailHeader bodyHeaders = new MailHeader();
+					// Add Fields to MailHeader Object
+					foreach( string key in message.Fields.Keys )
+						bodyHeaders.Data[ key ] = (string)this.message.Fields[ key ];
+
+					return bodyHeaders;
+			}
+			
 		}
-	}
 #endif
 
-    }
-
+#if NET_2_0
+		public IList RelatedBodyParts {
+			get { return message.RelatedBodyParts; }
+		}
+#endif
+    }	 
 }
