@@ -34,70 +34,76 @@ using System.Collections.Specialized;
 using System.Text;
 
 namespace System.Web.Security {
-	public class SqlRoleProvider {
+	public class SqlRoleProvider: RoleProvider {
 		
 		[MonoTODO]
-		public void AddUsersToRoles (string [] usernames, string [] rolenames)
+		public override void AddUsersToRoles (string [] usernames, string [] rolenames)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public void CreateRole (string rolename)
+		public override void CreateRole (string rolename)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public void DeleteRole (string rolename)
+		public override bool DeleteRole (string rolename, bool throwOnPopulatedRole)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public string [] GetAllRoles ()
+		public override string[] FindUsersInRole (string roleName, string usernameToMatch)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public string [] GetRolesForUser (string username)
+		public override string [] GetAllRoles ()
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public string [] GetUsersInRole (string rolename)
+		public override string [] GetRolesForUser (string username)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public virtual void Initialize (string name, NameValueCollection config)
+		public override string [] GetUsersInRole (string rolename)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public bool IsUserInRole (string username, string rolename)
+		public override void Initialize (string name, NameValueCollection config)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public void RemoveUsersFromRoles (string [] usernames, string [] rolenames)
+		public override bool IsUserInRole (string username, string rolename)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public bool RoleExists (string rolename)
+		public override void RemoveUsersFromRoles (string [] usernames, string [] rolenames)
 		{
 			throw new NotImplementedException ();
 		}
 		
 		[MonoTODO]
-		public string ApplicationName {
+		public override bool RoleExists (string rolename)
+		{
+			throw new NotImplementedException ();
+		}
+		
+		[MonoTODO]
+		public override string ApplicationName {
 			get { throw new NotImplementedException (); }
 			set { throw new NotImplementedException (); }
 		}
@@ -106,11 +112,6 @@ namespace System.Web.Security {
 		public virtual string Description {
 			get { throw new NotImplementedException (); }
 		}
-		
-		[MonoTODO]
-		public virtual string Name {
-			get { throw new NotImplementedException (); }
-		}
 	}
 }
 #endif
