@@ -4,9 +4,11 @@
 // Authors:
 //   Gaurav Vaish (gvaish@iitk.ac.in)
 //   Andreas Nahr (ClassDevelopment@A-SoftTech.com)
+//   Sanjay Gupta (gsanjay@novell.com)
 //
 // (C) Gaurav Vaish (2001)
 // (C) 2003 Andreas Nahr
+// (C) 2004 Novell, Inc. (http://www.novell.com)
 //
 
 //
@@ -50,6 +52,13 @@ namespace System.Web.UI.WebControls
 
 		private DataKeyCollection dataKeys;
 		private object            dataSource;
+		
+#if NET_2_0
+		bool inited;
+		IDataSource currentSource;
+		DataSourceSelectArguments selectArguments = null;
+		bool requiresDataBinding;
+#endif
 
 		public BaseDataList() : base()
 		{
@@ -93,7 +102,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
 		[Bindable(true)]
+#endif
 		[DefaultValue(-1)]
 		[WebCategory("Layout")]
 		[WebSysDescription("BaseDataList_CellPadding")]
@@ -111,7 +122,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
 		[Bindable(true)]
+#endif
 		[DefaultValue(-1)]
 		[WebCategory("Layout")]
 		[WebSysDescription("BaseDataList_CellSpacing")]
@@ -129,6 +142,21 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+		[DefaultValue ("")]
+		[WebCategory ("Accessibility")]
+		[WebSysDescription ("Caption")]
+		public virtual string Caption {
+			get {
+				object o = ViewState ["Caption"];
+				return (o != null) ? (string) o : "";
+			}
+
+			set { ViewState ["Caption"] = value; }
+		}
+
+#if NET_2_0
+	    [ThemeableAttribute (false)]
+#endif
 		[DefaultValue("")]
 		[WebCategory("Data")]
 		[WebSysDescription("BaseDataList_DataKeyField")]
@@ -161,6 +189,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+	    [ThemeableAttribute (false)]
+#endif
 		[DefaultValue("")]
 		[WebCategory("Data")]
 		[WebSysDescription("BaseDataList_DataMember")]
@@ -179,6 +210,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+	    [ThemeableAttribute (false)]
+#endif
 		[Bindable(true)]
 		[DefaultValue(null)]
 		[DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
@@ -191,6 +225,9 @@ namespace System.Web.UI.WebControls
 			set {
 				if (value == null || value is IListSource || value is IEnumerable) {
 					dataSource = value;
+#if NET_2_0
+					if (inited) OnDataPropertyChanged ();
+#endif
 				} else {
 					throw new ArgumentException (HttpRuntime.FormatResourceString (
 								"Invalid_DataSource_Type", ID));
@@ -198,7 +235,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
 		[Bindable(true)]
+#endif
 		[DefaultValue(GridLines.Both)]
 		[WebCategory("Appearance")]
 		[WebSysDescription("BaseDataList_GridLines")]
@@ -219,7 +258,9 @@ namespace System.Web.UI.WebControls
 		// LAMESPEC HorizontalAlign has a Category attribute, this should obviously be a WebCategory attribute
 		// but is defined incorrectly in the MS framework
 
+#if !NET_2_0
 		[Bindable(true)]
+#endif
 		[DefaultValue(HorizontalAlign.NotSet)]
 		[Category("Layout")]
 		[WebSysDescription("BaseDataList_HorizontalAlign")]
@@ -296,67 +337,98 @@ namespace System.Web.UI.WebControls
 		protected abstract void CreateControlHierarchy(bool useDataSource);
 		
 		#if NET_2_0
-
 			
-			// should be `internal protected' (why, oh WHY did they do that !?!)
 			protected override void OnInit (EventArgs e)
 			{
 				base.OnInit(e);
+				Page.PreLoad += new EventHandler (OnPagePreLoad);
+			}
+			
+			void OnPagePreLoad (object sender, EventArgs e)
+			{
+				SubscribeSourceChangeEvent ();
 				inited = true;
-				if (!Page.IsPostBack)
-					RequiresDataBinding = true;
+			}
+		
+			void SubscribeSourceChangeEvent ()
+			{
+				IDataSource ds = GetDataSource ();
+				
+				if (currentSource != ds && currentSource != null) {
+					currentSource.DataSourceChanged -= new EventHandler (OnDataSourceViewChanged);
+					currentSource = ds;
+				}
+					
+				if (ds != null)
+					ds.DataSourceChanged += new EventHandler (OnDataSourceViewChanged);
 			}
 			
-			// should be `internal protected' (why, oh WHY did they do that !?!)
 			protected override void OnLoad (EventArgs e)
 			{
-				IDataSource ds = GetDataSourceObject () as IDataSource;
-				if (ds != null && DataSourceID != "")
-					ds.DataSourceChanged += new EventHandler (OnDataSourceChanged);
-				
+				if (IsBoundUsingDataSourceID && (!Page.IsPostBack || !EnableViewState))
+					RequiresDataBinding = true;
+	
 				base.OnLoad(e);
 			}
 			
-			// should be `internal protected' (why, oh WHY did they do that !?!)
 			protected override void OnPreRender (EventArgs e)
 			{
 				EnsureDataBound ();
 				base.OnPreRender (e);
 			}
 				
+			protected bool IsBoundUsingDataSourceID {
+				get { return DataSourceID.Length > 0; }
+			}
+			
 			protected void EnsureDataBound ()
 			{
-				if (RequiresDataBinding && DataSourceID != "")
+				if (RequiresDataBinding && IsBoundUsingDataSourceID)
 					DataBind ();
 			}
 			
-			protected virtual object GetDataSourceObject ()
+			IDataSource GetDataSource ()
 			{
-				if (DataSourceID != "")
-					return (IDataSource) NamingContainer.FindControl (DataSourceID);
-				
-				return DataSource;
+				if (IsBoundUsingDataSourceID) {
+					Control ctrl = NamingContainer.FindControl (DataSourceID);
+					if (ctrl == null)
+						throw new HttpException (string.Format ("A control with ID '{0}' could not be found.", DataSourceID));
+					if (!(ctrl is IDataSource))
+						throw new HttpException (string.Format ("The control with ID '{0}' is not a control of type IDataSource.", DataSourceID));
+					return (IDataSource) ctrl;
+				}
+				return DataSource as IDataSource;
 			}
 			
-			protected virtual IEnumerable GetResolvedDataSource ()
+			protected IEnumerable GetData ()
 			{
-				if (DataSource != null && DataSourceID != "")
-					throw new HttpException ();
+				if (DataSource != null && IsBoundUsingDataSourceID)
+					throw new HttpException ("Control bound using both DataSourceID and DataSource properties.");
 				
-				IDataSource ds = this.GetDataSourceObject () as IDataSource;
-				if (ds != null && DataSourceID != "")
-					return ds.GetView (DataMember).Select ();
-				else if (DataSource != null)
-					return DataSourceHelper.GetResolvedDataSource (DataSource, DataMember);
-				else
-					return null; 
+				IDataSource ds = GetDataSource ();
+				if (ds != null)
+					return ds.GetView (DataMember).ExecuteSelect (SelectArguments);
+				
+				IEnumerable ie = DataSourceHelper.GetResolvedDataSource (DataSource, DataMember);
+				if (ie != null) return ie;
+				
+				throw new HttpException (string.Format ("Unexpected data source type: {0}", DataSource.GetType()));
 			}
 			
-			protected void OnDataSourceChanged (object sender, EventArgs e)
+			protected virtual void OnDataSourceViewChanged (object sender, EventArgs e)
 			{
 				RequiresDataBinding = true;
 			}
-			
+
+			protected virtual void OnDataPropertyChanged ()
+			{
+				RequiresDataBinding = true;
+				SubscribeSourceChangeEvent ();
+			}
+
+			[DefaultValueAttribute ("")]
+			[IDReferencePropertyAttribute (typeof(System.Web.UI.DataSourceControl))]
+			[ThemeableAttribute (false)]
 			public virtual string DataSourceID {
 				get {
 					object o = ViewState ["DataSourceID"];
@@ -366,21 +438,38 @@ namespace System.Web.UI.WebControls
 					return String.Empty;
 				}
 				set {
-					if (inited)
-						RequiresDataBinding = true;
-					
 					ViewState ["DataSourceID"] = value;
+					if (inited) OnDataPropertyChanged ();
 				}
 			}
 			
-			bool requiresDataBinding;
+			protected bool Initialized {
+				get { return inited; }
+			}
+			
 			protected bool RequiresDataBinding {
 				get { return requiresDataBinding; }
 				set { requiresDataBinding = value; }
 			}
 			
-			protected bool inited;
-				
+			protected virtual DataSourceSelectArguments CreateDataSourceSelectArguments ()
+			{
+				return DataSourceSelectArguments.Empty;
+			}
+			
+			protected DataSourceSelectArguments SelectArguments {
+				get {
+					if (selectArguments == null)
+						selectArguments = CreateDataSourceSelectArguments ();
+					return selectArguments;
+				}
+			}
+			
+			internal IEnumerable GetResolvedDataSource ()
+			{
+				return GetData ();
+			}
+			
 		#else
 			internal IEnumerable GetResolvedDataSource ()
 			{
