@@ -60,6 +60,10 @@ namespace System.Web.UI.WebControls
 			renderUplevel = false;
 		}
 
+#if NET_2_0
+		[IDReferencePropertyAttribute (typeof(System.Web.UI.Control))]
+		[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (""), WebCategory ("Behavior")]
 		[TypeConverter (typeof (ValidatedControlConverter))]
 		[WebSysDescription ("The ID of the control to validate.")]
@@ -80,7 +84,12 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[DefaultValue (typeof (ValidatorDisplay), "Static"), Bindable (true), WebCategory ("Appearance")]
+#if NET_2_0
+		[ThemeableAttribute (false)]
+#else
+		[Bindable (true)]
+#endif
+		[DefaultValue (typeof (ValidatorDisplay), "Static"), WebCategory ("Appearance")]
 		[WebSysDescription ("Determines how the validator is displayed.")]
 		public ValidatorDisplay Display
 		{
@@ -103,6 +112,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+		[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (true), WebCategory ("Behavior")]
 		[WebSysDescription ("Determines if client script is activated on uplevel browsers.")]
 		public bool EnableClientScript
@@ -136,7 +148,12 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[DefaultValue (""), Bindable (true), WebCategory ("Appearance")]
+#if NET_2_0
+	    [Localizable (true)]
+#else
+		[Bindable (true)]
+#endif
+		[DefaultValue (""), WebCategory ("Appearance")]
 		[WebSysDescription ("An error message that is displayed if the control validates to false.")]
 		public string ErrorMessage
 		{
@@ -168,6 +185,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+		[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (true), Browsable (false), WebCategory ("Misc")]
 		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		[WebSysDescription ("Indicates if the control validated to false.")]
@@ -385,11 +405,12 @@ namespace System.Web.UI.WebControls
 		[MonoTODO("Damn_This_Is_Really_Frustrating___by_Gaurav")]
 		protected void RegisterValidatorCommonScript()
 		{
-			if(Page.IsClientScriptBlockRegistered("ValidatorIncludeScript"))
+			if(Page.ClientScript.IsClientScriptBlockRegistered("ValidatorIncludeScript"))
 				return;
 			
-			string jsDirectory = System.Web.UI.Utils.GetScriptLocation(Context);
-			string jsFile = jsDirectory + "/WebUIValidation.js";
+			//string jsDirectory = System.Web.UI.Utils.GetScriptLocation(Context);
+			//string jsFile = jsDirectory + "/WebUIValidation.js";
+                        
 			//TODO: Ok, now add the <script language="javascript"> etc
 			//FIXME: Should I check for 'Explorer'? MS-Net seems to do it!
 			throw new NotImplementedException();
@@ -403,7 +424,7 @@ namespace System.Web.UI.WebControls
 			//I'm confused! This will make it work, at least on IE
 			string val = "document.all[\"" + ClientID;
 			val += "\"]";
-			Page.RegisterArrayDeclaration("Page_Validators", val);
+			Page.ClientScript.RegisterArrayDeclaration("Page_Validators", val);
 		}
 
 		[MonoTODO("Render_ing_always_left")]
