@@ -39,6 +39,9 @@ using System.Web.UI;
 namespace System.Web.UI.WebControls
 {
 	public class DataGridItem : TableRow, INamingContainer
+#if NET_2_0
+		, IDataItemContainer
+#endif
 	{
 		private int itemIndex;
 		private int dataSetIndex;
@@ -87,6 +90,16 @@ namespace System.Web.UI.WebControls
 				return itemType;
 			}
 		}
+		
+#if NET_2_0
+		int IDataItemContainer.DataItemIndex {
+			get { return dataSetIndex; }
+		}
+
+		int IDataItemContainer.DisplayIndex {
+			get { return itemIndex; }
+		}
+#endif
 
 		protected override bool OnBubbleEvent(object source, EventArgs e)
 		{
