@@ -41,6 +41,9 @@ namespace System.Web.UI.WebControls
 {
 	[ValidationProperty("SelectedItem")]
 	public class DropDownList : ListControl, IPostBackDataHandler
+#if NET_2_0
+		, IEditableTextControl
+#endif
 	{
 		public DropDownList(): base()
 		{
@@ -105,6 +108,7 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
 		[Browsable (false), DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
 		[Bindable (false), EditorBrowsable (EditorBrowsableState.Never)]
 		public override string ToolTip
@@ -116,6 +120,7 @@ namespace System.Web.UI.WebControls
 			set {
 			}
 		}
+#endif
 
 		protected override void AddAttributesToRender(HtmlTextWriter writer)
 		{
@@ -128,7 +133,7 @@ namespace System.Web.UI.WebControls
 
 			if(AutoPostBack && Page != null)
 			{
-				writer.AddAttribute(HtmlTextWriterAttribute.Onchange, Page.GetPostBackClientEvent(this,""));
+				writer.AddAttribute(HtmlTextWriterAttribute.Onchange, Page.ClientScript.GetPostBackClientEvent(this,""));
 				writer.AddAttribute("language", "javascript");
 			}
 		}
@@ -138,6 +143,7 @@ namespace System.Web.UI.WebControls
 			return new EmptyControlCollection(this);
 		}
 
+#if !NET_2_0
 		protected override void RenderContents(HtmlTextWriter writer)
 		{
 			if(Items != null)
@@ -163,8 +169,24 @@ namespace System.Web.UI.WebControls
 				}
 			}
 		}
+#else
+		protected internal override void VerifyMultiSelect ()
+		{
+			throw new HttpException(HttpRuntime.FormatResourceString("Cannot_Multiselect_In_DropDownList"));
+		}
+#endif
 
-		bool IPostBackDataHandler.LoadPostData(string postDataKey, NameValueCollection postCollection)
+
+#if NET_2_0
+		bool IPostBackDataHandler.LoadPostData (string postDataKey, NameValueCollection postCollection)
+		{
+			return LoadPostData (postDataKey, postCollection);
+		}
+		
+		protected virtual bool LoadPostData (string postDataKey, NameValueCollection postCollection)
+#else
+		bool IPostBackDataHandler.LoadPostData (string postDataKey, NameValueCollection postCollection)
+#endif
 		{
 			string[] vals = postCollection.GetValues(postDataKey);
 			if(vals != null)
@@ -179,9 +201,24 @@ namespace System.Web.UI.WebControls
 			return false;
 		}
 
-		void IPostBackDataHandler.RaisePostDataChangedEvent()
+#if NET_2_0
+		void IPostBackDataHandler.RaisePostDataChangedEvent ()
 		{
-			OnSelectedIndexChanged(EventArgs.Empty);
+			RaisePostDataChangedEvent ();
 		}
+		
+		protected virtual void RaisePostDataChangedEvent ()
+		{
+			if (CausesValidation)
+				Page.Validate (ValidationGroup);
+
+			OnSelectedIndexChanged (EventArgs.Empty);
+		}
+#else
+		void IPostBackDataHandler.RaisePostDataChangedEvent ()
+		{
+			OnSelectedIndexChanged (EventArgs.Empty);
+		}
+#endif
 	}
 }
