@@ -42,6 +42,9 @@ using System.Web.UI;
 namespace System.Web.UI.WebControls
 {
 	[TypeConverter(typeof(FontUnitConverter))]
+#if NET_2_0
+	[Serializable]
+#endif
 	public struct FontUnit
 	{
 		public static readonly FontUnit Empty   = new FontUnit();
@@ -62,8 +65,8 @@ namespace System.Web.UI.WebControls
 
 		static FontUnit ()
 		{
-			sizeTable = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
-						   CaseInsensitiveComparer.Default);
+			sizeTable = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						   CaseInsensitiveComparer.DefaultInvariant);
 			sizeTable.Add ("smaller", 2);
 			sizeTable.Add ("larger", 3);
 			sizeTable.Add ("xx-small", 4);
