@@ -46,6 +46,9 @@ namespace System.Web.UI.WebControls
 	[ParseChildren(false)]
 	[ToolboxData("<{0}:LinkButton runat=\"server\">LinkButton</{0}:LinkButton>")]
 	public class LinkButton : WebControl, IPostBackEventHandler
+#if NET_2_0
+		, IButtonControl
+#endif
 	{
 		private static readonly object ClickEvent   = new object();
 		private static readonly object CommandEvent = new object();
@@ -54,7 +57,12 @@ namespace System.Web.UI.WebControls
 		{
 		}
 
-		[DefaultValue (true), Bindable (false), WebCategory ("Behavior")]
+#if NET_2_0
+		[ThemeableAttribute (false)]
+#else
+		[Bindable (false)]
+#endif
+		[DefaultValue (true), WebCategory ("Behavior")]
 		[WebSysDescription ("Determines if validation is performed when clicked.")]
 		public bool CausesValidation
 		{
@@ -66,6 +74,9 @@ namespace System.Web.UI.WebControls
 			set { ViewState ["CausesValidation"] = value; }
 		}
 
+#if NET_2_0
+		[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (""), Bindable (true), WebCategory ("Behavior")]
 		[WebSysDescription ("An argument for the Command of this control.")]
 		public string CommandArgument
@@ -78,6 +89,9 @@ namespace System.Web.UI.WebControls
 			set { ViewState ["CommandArgument"] = value; }
 		}
 
+#if NET_2_0
+		[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (""), WebCategory ("Behavior")]
 		[WebSysDescription ("The name of the Command of this control.")]
 		public string CommandName
@@ -90,6 +104,9 @@ namespace System.Web.UI.WebControls
 			set { ViewState ["CommandName"] = value; }
 		}
 
+#if NET_2_0
+		[Localizable (true)]
+#endif
 		[DefaultValue (""), Bindable (true), WebCategory ("Appearance")]
 		[PersistenceMode (PersistenceMode.InnerDefaultProperty)]
 		[WebSysDescription ("The text that should be shown on this LinkButton.")]
@@ -102,6 +119,64 @@ namespace System.Web.UI.WebControls
 
 			set { ViewState ["Text"] = value; }
 		}
+		
+#if NET_2_0
+		[ThemeableAttribute (false)]
+		[EditorAttribute ("System.Web.UI.Design.UrlEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[DefaultValueAttribute ("")]
+		[UrlProperty]
+		public string PostBackUrl {
+			get {
+				string text = (string)ViewState["PostBackUrl"];
+				if (text!=null) return text;
+				return String.Empty;
+			}
+			set {
+				ViewState["PostBackUrl"] = value;
+			}
+		}
+		
+		[DefaultValueAttribute ("")]
+		[ThemeableAttribute (false)]
+		[WebCategoryAttribute ("Behavior")]
+		public string ValidationGroup {
+			get {
+				string text = (string)ViewState["ValidationGroup"];
+				if (text!=null) return text;
+				return String.Empty;
+			}
+			set {
+				ViewState["ValidationGroup"] = value;
+			}
+		}
+		
+		[DefaultValueAttribute ("")]
+		[ThemeableAttribute (false)]
+		[WebCategoryAttribute ("Behavior")]
+		public string OnClientClick {
+			get {
+				string text = (string)ViewState["OnClientClick"];
+				if (text!=null) return text;
+				return String.Empty;
+			}
+			set {
+				ViewState["OnClientClick"] = value;
+			}
+		}
+		
+		protected virtual PostBackOptions GetPostBackOptions ()
+		{
+			PostBackOptions ops = new PostBackOptions (this);
+			if (PostBackUrl != "")
+				ops.ActionUrl = PostBackUrl;
+			ops.PerformValidation = Page.Validators.Count > 0 && CausesValidation;
+			if (ops.PerformValidation && ValidationGroup != "")
+				ops.ValidationGroup = ValidationGroup;
+			ops.RequiresJavaScriptProtocol = false;
+			return ops;
+		}
+#endif
+
 
 		[WebCategory ("Action")]
 		[WebSysDescription ("Raised when the LinkButton is clicked.")]
@@ -143,6 +218,20 @@ namespace System.Web.UI.WebControls
 			base.OnPreRender(e);
 		}
 
+#if NET_2_0
+		void IPostBackEventHandler.RaisePostBackEvent (string eventArgument)
+		{
+			RaisePostBackEvent (eventArgument);
+		}
+		
+		protected virtual void RaisePostBackEvent (string eventArgument)
+		{
+			if (CausesValidation)
+				Page.Validate (ValidationGroup);
+			OnClick (EventArgs.Empty);
+			OnCommand (new CommandEventArgs (CommandName, CommandArgument));
+		}
+#else
 		void IPostBackEventHandler.RaisePostBackEvent (string eventArgument)
 		{
 			if (CausesValidation)
@@ -150,10 +239,34 @@ namespace System.Web.UI.WebControls
 			OnClick (EventArgs.Empty);
 			OnCommand (new CommandEventArgs (CommandName, CommandArgument));
 		}
+#endif
 
 		protected override void AddAttributesToRender (HtmlTextWriter writer)
 		{
 			base.AddAttributesToRender (writer);
+#if NET_2_0
+			if (Page != null && Enabled) {
+				
+				if (OnClientClick != "")
+					writer.AddAttribute (HtmlTextWriterAttribute.Onclick, OnClientClick);
+
+				string script = "";
+
+				PostBackOptions ops = GetPostBackOptions ();
+				if (ops.RequiresSpecialPostBack || ops.ClientSubmit) {
+					script = Page.ClientScript.GetPostBackEventReference (ops);
+				}
+				else if (CausesValidation && Page.Validators.Count > 0) {
+					script = Utils.GetClientValidatedPostBack (this);
+				}
+				else {
+					script = Page.ClientScript.GetPostBackClientEvent (this,"");
+				}			
+				if (script != "")
+					writer.AddAttribute (HtmlTextWriterAttribute.Href, "javascript:" + script);
+			}
+
+#else
 			if (Enabled && Page != null){
 				if (CausesValidation && Page.Validators.Count > 0){
 					writer.AddAttribute (HtmlTextWriterAttribute.Href,
@@ -162,8 +275,9 @@ namespace System.Web.UI.WebControls
 					return;
 				}
 				writer.AddAttribute (HtmlTextWriterAttribute.Href,
-						     Page.GetPostBackClientHyperlink (this, ""));
+						     Page.ClientScript.GetPostBackClientHyperlink (this, ""));
 			}
+#endif
 		}
 
 		protected override void AddParsedSubObject (object obj)
