@@ -48,6 +48,9 @@ namespace System.Web.UI.WebControls
 		{
 		}
 
+#if NET_2_0
+	    [ThemeableAttribute (false)]
+#endif
 		[DefaultValue (""), WebCategory ("Behavior")]
 		[WebSysDescription ("The name of the group that this control belongs to.")]
 		public virtual string GroupName
@@ -85,7 +88,7 @@ namespace System.Web.UI.WebControls
 
 			if (AutoPostBack){
 				writer.AddAttribute (HtmlTextWriterAttribute.Onclick,
-						     Page.GetPostBackClientEvent (this, ""));
+						     Page.ClientScript.GetPostBackClientEvent (this, ""));
 				writer.AddAttribute ("language", "javascript");
 			}
 
@@ -103,10 +106,14 @@ namespace System.Web.UI.WebControls
 		private string UniqueGroupNamePrivate
 		{
 			get {
-				string retVal = GroupName;
-				int unique = UniqueID.LastIndexOf (':');
-				if (unique >= 0)
-					retVal += UniqueID.Substring (unique + 1);
+				string retVal;
+				string uid = UniqueID;
+				int unique = uid.LastIndexOf (':');
+				if (unique == -1) {
+					retVal = GroupName;
+				} else {
+					retVal = uid.Substring (0, unique + 1) + GroupName;
+				}
 
 				return retVal;
 			}
@@ -126,8 +133,17 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		bool IPostBackDataHandler.LoadPostData (string postDataKey,
-						        NameValueCollection postCollection)
+
+#if NET_2_0
+		bool IPostBackDataHandler.LoadPostData (string postDataKey, NameValueCollection postCollection)
+		{
+			return LoadPostData (postDataKey, postCollection);
+		}
+		
+		protected override bool LoadPostData (string postDataKey, NameValueCollection postCollection)
+#else
+		bool IPostBackDataHandler.LoadPostData (string postDataKey, NameValueCollection postCollection)
+#endif
 		{
 			bool _checked = Checked;
 			if (postCollection [UniqueGroupNamePrivate] == ValueAttributePrivate){
@@ -142,9 +158,21 @@ namespace System.Web.UI.WebControls
 			return true;
 		}
 
+#if NET_2_0
+		void IPostBackDataHandler.RaisePostDataChangedEvent ()
+		{
+			RaisePostDataChangedEvent ();
+		}
+		
+		protected override void RaisePostDataChangedEvent ()
+		{
+			OnCheckedChanged (EventArgs.Empty);
+		}
+#else
 		void IPostBackDataHandler.RaisePostDataChangedEvent ()
 		{
 			OnCheckedChanged (EventArgs.Empty);
 		}
+#endif
 	}
 }
