@@ -68,6 +68,10 @@ namespace System.Web.UI.WebControls
 		int  selectionBits;
 		bool selfStateBag;
 		bool marked;
+		
+#if NET_2_0
+		string regClass;
+#endif
 
 		private FontInfo font;
 
@@ -114,8 +118,11 @@ namespace System.Web.UI.WebControls
 				selectionBits |= MARKED;
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (null), Bindable (true), WebCategory ("Appearance")]
+		[DefaultValue (null), WebCategory ("Appearance")]
 		[TypeConverter (typeof (WebColorConverter))]
 		[WebSysDescription ("The background color for the WebControl.")]
 		public Color BackColor {
@@ -130,8 +137,11 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (null), Bindable (true), WebCategory ("Appearance")]
+		[DefaultValue (null), WebCategory ("Appearance")]
 		[TypeConverter (typeof (WebColorConverter))]
 		[WebSysDescription ("The border color for the WebControl.")]
 		public Color BorderColor {
@@ -146,8 +156,11 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (typeof(BorderStyle), "NotSet"), Bindable (true), WebCategory ("Appearance")]
+		[DefaultValue (typeof(BorderStyle), "NotSet"), WebCategory ("Appearance")]
 		[WebSysDescription ("The style/type of the border used for the WebControl.")]
 		public BorderStyle BorderStyle {
 			get {
@@ -161,8 +174,11 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (null), Bindable (true), WebCategory ("Appearance")]
+		[DefaultValue (null), WebCategory ("Appearance")]
 		[WebSysDescription ("The width of the border used for the WebControl.")]
 		public Unit BorderWidth {
 			get {
@@ -191,8 +207,11 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (null), Bindable (true), WebCategory ("Appearance")]
+		[DefaultValue (null), WebCategory ("Appearance")]
 		[TypeConverter (typeof (WebColorConverter))]
 		[WebSysDescription ("The color that is used to paint the primary display of the WebControl.")]
 		public Color ForeColor {
@@ -207,8 +226,11 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (null), Bindable (true), WebCategory ("Layout")]
+		[DefaultValue (null), WebCategory ("Layout")]
 		[WebSysDescription ("The height of this WebControl.")]
 		public Unit Height {
 			get {
@@ -222,8 +244,11 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if !NET_2_0
+		[Bindable (true)]
+#endif
 		[NotifyParentProperty (true)]
-		[DefaultValue (null), Bindable (true), WebCategory ("Layout")]
+		[DefaultValue (null), WebCategory ("Layout")]
 		[WebSysDescription ("The width of this WebControl.")]
 		public Unit Width {
 			get {
@@ -254,10 +279,10 @@ namespace System.Web.UI.WebControls
 			get { return (selectionBits == 0); }
 		}
 
-		private void AddColor (HtmlTextWriter writer, HtmlTextWriterStyle style, Color color)
+		private void AddColor (CssStyleCollection attributes, HtmlTextWriterStyle style, Color color)
 		{
 			if (!color.IsEmpty)
-				writer.AddStyleAttribute (style, ColorTranslator.ToHtml (color));
+				attributes.Add (style, ColorTranslator.ToHtml (color));
 		}
 
 		private static string StringArrayToString (string [] array, char separator)
@@ -283,52 +308,76 @@ namespace System.Web.UI.WebControls
 
 		public virtual void AddAttributesToRender (HtmlTextWriter writer, WebControl owner)
 		{
-			if (IsSet (BACKCOLOR))
-				AddColor (writer, HtmlTextWriterStyle.BackgroundColor, BackColor);
-
-			if (IsSet(BORDERCOLOR))
-				AddColor (writer, HtmlTextWriterStyle.BorderColor, BorderColor);
-
-			if (IsSet (FORECOLOR))
-				AddColor (writer, HtmlTextWriterStyle.Color, ForeColor);
-
 			if (IsSet (CSSCLASS)) {
 				string cssClass = (string) ViewState ["CssClass"];
 				if (cssClass.Length > 0)
 					writer.AddAttribute (HtmlTextWriterAttribute.Class, cssClass);
 			}
 
+			CssStyleCollection ats = new CssStyleCollection ();
+#if NET_2_0
+			FillStyleAttributes (ats, owner);
+#else
+			FillAttributes (ats);
+#endif
+			foreach (string key in ats.Keys)
+				writer.AddStyleAttribute (key, ats [key]);
+		}
+
+#if NET_2_0
+		public CssStyleCollection GetStyleAttributes (IUrlResolutionService urlResolver)
+		{
+			CssStyleCollection ats = new CssStyleCollection ();
+			FillStyleAttributes (ats, urlResolver);
+			return ats;
+		}
+		protected virtual void FillStyleAttributes (CssStyleCollection attributes, IUrlResolutionService urlResolver)
+		{
+			FillAttributes (attributes);
+		}
+#endif
+
+		void FillAttributes (CssStyleCollection attributes)
+		{
+			if (IsSet (BACKCOLOR))
+				AddColor (attributes, HtmlTextWriterStyle.BackgroundColor, BackColor);
+
+			if (IsSet(BORDERCOLOR))
+				AddColor (attributes, HtmlTextWriterStyle.BorderColor, BorderColor);
+
+			if (IsSet (FORECOLOR))
+				AddColor (attributes, HtmlTextWriterStyle.Color, ForeColor);
+
 			if (!BorderWidth.IsEmpty) {
-				writer.AddStyleAttribute (HtmlTextWriterStyle.BorderWidth,
+				attributes.Add (HtmlTextWriterStyle.BorderWidth,
 						BorderWidth.ToString (CultureInfo.InvariantCulture));
 
 				if (BorderStyle != BorderStyle.NotSet) {
-					writer.AddStyleAttribute (HtmlTextWriterStyle.BorderStyle,
+					attributes.Add (HtmlTextWriterStyle.BorderStyle,
 							Enum.Format (typeof (BorderStyle), BorderStyle, "G"));
 				} else {
 					if (BorderWidth.Value != 0.0)
-						writer.AddStyleAttribute (HtmlTextWriterStyle.BorderStyle,
-									  "solid");
+						attributes.Add (HtmlTextWriterStyle.BorderStyle, "solid");
 				}
 			} else {
 				if (BorderStyle != BorderStyle.NotSet)
-					writer.AddStyleAttribute (HtmlTextWriterStyle.BorderStyle,
+					attributes.Add (HtmlTextWriterStyle.BorderStyle,
 							Enum.Format (typeof (BorderStyle), BorderStyle, "G"));
 			}
 
 			if (Font.Names.Length > 0)
-				writer.AddStyleAttribute (HtmlTextWriterStyle.FontFamily,
+				attributes.Add (HtmlTextWriterStyle.FontFamily,
 							StringArrayToString (Font.Names, ','));
 
 			if (!Font.Size.IsEmpty)
-				writer.AddStyleAttribute (HtmlTextWriterStyle.FontSize,
+				attributes.Add (HtmlTextWriterStyle.FontSize,
 							Font.Size.ToString (CultureInfo.InvariantCulture));
 
 			if (Font.Bold)
-				writer.AddStyleAttribute (HtmlTextWriterStyle.FontWeight, "bold");
+				attributes.Add (HtmlTextWriterStyle.FontWeight, "bold");
 
 			if (Font.Italic)
-				writer.AddStyleAttribute (HtmlTextWriterStyle.FontStyle, "italic");
+				attributes.Add (HtmlTextWriterStyle.FontStyle, "italic");
 
 			string textDecoration = String.Empty;
 			if (Font.Strikeout)
@@ -341,24 +390,24 @@ namespace System.Web.UI.WebControls
 				textDecoration += " overline";
 
 			if (textDecoration.Length > 0)
-				writer.AddStyleAttribute( HtmlTextWriterStyle.TextDecoration, textDecoration);
+				attributes.Add (HtmlTextWriterStyle.TextDecoration, textDecoration);
 
 			Unit u = Unit.Empty;
 			if (IsSet (HEIGHT)) {
 				u = (Unit) ViewState ["Height"];
 				if (!u.IsEmpty)
-					writer.AddStyleAttribute (HtmlTextWriterStyle.Height,
+					attributes.Add (HtmlTextWriterStyle.Height,
 								u.ToString (CultureInfo.InvariantCulture));
 			}
 
 			if (IsSet (WIDTH)) {
 				u = (Unit) ViewState ["Width"];
 				if (!u.IsEmpty)
-					writer.AddStyleAttribute (HtmlTextWriterStyle.Width,
+					attributes.Add (HtmlTextWriterStyle.Width,
 								u.ToString (CultureInfo.InvariantCulture));
 			}
 		}
-
+		
 		public virtual void CopyFrom (Style source)
 		{
 			if (source == null || source.IsEmpty)
@@ -515,15 +564,45 @@ namespace System.Web.UI.WebControls
 			get { return IsTrackingViewState; }
 		}
 
+#if !NET_2_0
 		public override string ToString ()
 		{
 			return String.Empty;
 		}
+#endif
 
 		protected internal void SetBit (int bit)
 		{
 			Set (bit);
 		}
+		
+		internal virtual void CopyTextStylesFrom (Style source)
+		{
+			if (source.IsSet (FORECOLOR)&& (source.ForeColor != Color.Empty))
+				ForeColor = source.ForeColor;
+		}
+		
+#if NET_2_0
+		public void SetDirty ()
+		{
+			if (selectionBits != 0x00)
+				Set (MARKED);
+			if (viewState != null)
+				viewState.SetDirty ();
+		}
+		
+		[DesignerSerializationVisibility (DesignerSerializationVisibility.Hidden)]
+		[BrowsableAttribute (false)]
+		[EditorBrowsableAttribute (EditorBrowsableState.Advanced)]
+		public string RegisteredCssClass {
+			get { return regClass; }
+		}
+		
+		internal void SetRegisteredCssClass (string name)
+		{
+			regClass = name;
+		}
+#endif
 	}
 }
 
