@@ -38,11 +38,15 @@ using System.ComponentModel;
 
 namespace System.Web.UI.WebControls
 {
+#if NET_2_0
+	[Bindable (false)]
+#else
+	[PersistChildren(true)]
+#endif
 	[DefaultProperty("Text")]
 	[ToolboxItem(false)]
 	[ControlBuilder(typeof(TableCellControlBuilder))]
 	[ParseChildren(false)]
-	[PersistChildren(true)]
 	public class TableCell: WebControl
 	{
 		public TableCell () : base (HtmlTextWriterTag.Td)
@@ -55,7 +59,10 @@ namespace System.Web.UI.WebControls
 			PreventAutoID ();
 		}
 
-		[DefaultValue (0), Bindable (true), WebCategory ("Appearance")]
+#if !NET_2_0
+		[Bindable (true)]
+#endif
+		[DefaultValue (0), WebCategory ("Appearance")]
 		[WebSysDescription ("The number of columns spanned by this cell.")]
 		public virtual int ColumnSpan
 		{
@@ -71,7 +78,10 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[DefaultValue (0), Bindable (true), WebCategory ("Layout")]
+#if !NET_2_0
+		[Bindable (true)]
+#endif
+		[DefaultValue (0), WebCategory ("Layout")]
 		[WebSysDescription ("The number of rows spanned by this cell.")]
 		public virtual int RowSpan
 		{
@@ -87,7 +97,13 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[DefaultValue (""), Bindable (true), WebCategory ("Appearance")]
+#if NET_2_0
+	    [LocalizableAttribute (true)]
+    	[PersistenceModeAttribute (PersistenceMode.EncodedInnerDefaultProperty)]
+#else
+		[Bindable (true)]
+#endif
+		[DefaultValue (""), WebCategory ("Appearance")]
 		[WebSysDescription ("The text that is shown in this cell.")]
 		public virtual string Text
 		{
@@ -103,7 +119,10 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[DefaultValue (typeof (HorizontalAlign), "NotSet"), Bindable (true), WebCategory ("Layout")]
+#if !NET_2_0
+		[Bindable (true)]
+#endif
+		[DefaultValue (typeof (HorizontalAlign), "NotSet"), WebCategory ("Layout")]
 		[WebSysDescription ("The horizontal alignment for this cell.")]
 		public virtual HorizontalAlign HorizontalAlign
 		{
@@ -115,7 +134,10 @@ namespace System.Web.UI.WebControls
 			set { ((TableItemStyle) ControlStyle).HorizontalAlign = value; }
 		}
 
-		[DefaultValue (typeof (VerticalAlign), "NotSet"), Bindable (true), WebCategory ("Layout")]
+#if !NET_2_0
+		[Bindable (true)]
+#endif
+		[DefaultValue (typeof (VerticalAlign), "NotSet"), WebCategory ("Layout")]
 		[WebSysDescription ("The horizontal alignment for this cell.")]
 		public virtual VerticalAlign VerticalAlign
 		{
@@ -128,7 +150,10 @@ namespace System.Web.UI.WebControls
 			set { ((TableItemStyle) ControlStyle).VerticalAlign = value; }
 		}
 
-		[DefaultValue (true), Bindable (true), WebCategory ("Layout")]
+#if !NET_2_0
+		[Bindable (true)]
+#endif
+		[DefaultValue (true), WebCategory ("Layout")]
 		[WebSysDescription ("Determines if the text in the cell should be wraped at line-end.")]
 		public virtual bool Wrap
 		{
@@ -180,7 +205,7 @@ namespace System.Web.UI.WebControls
 
 		protected override void RenderContents (HtmlTextWriter writer)
 		{
-			if (HasControls ())
+			if (HasControls () || HasRenderMethodDelegate ())
 				base.RenderContents (writer);
 			else
 				writer.Write (Text);
