@@ -37,6 +37,8 @@ using System.Globalization;
 using System.ComponentModel;
 using System.Web;
 using System.Web.UI;
+using System.ComponentModel.Design.Serialization;
+using System.Reflection;
 
 namespace System.Web.UI.WebControls
 {
@@ -53,6 +55,19 @@ namespace System.Web.UI.WebControls
 			return CanConvertFrom(context, sourceType);
 		}
 
+#if NET_2_0
+		public override bool CanConvertTo (ITypeDescriptorContext context, Type destinationType)
+		{
+			if (destinationType == typeof (string))
+				return true;
+
+			if (destinationType == typeof (InstanceDescriptor))
+				return true;
+
+			return base.CanConvertTo (context, destinationType);
+		}
+#endif
+
 		public override object ConvertFrom(ITypeDescriptorContext context, CultureInfo culture, object value)
 		{
 			if(value == null)
@@ -80,6 +95,21 @@ namespace System.Web.UI.WebControls
 				}
 				return val.ToString(culture);
 			}
+
+#if NET_2_0
+			if (destinationType == typeof (InstanceDescriptor) && value is Unit) {
+				Unit s = (Unit) value;
+				ConstructorInfo ci = typeof(Unit).GetConstructor (new Type[] { typeof(double), typeof(UnitType) });
+				return new InstanceDescriptor (ci, new object[] { s.Value, s.Type });
+			}
+
+			if (destinationType == typeof (InstanceDescriptor) && value is string) {
+				Unit s = Unit.Parse ((string)value, CultureInfo.InvariantCulture);
+				ConstructorInfo ci = typeof(Unit).GetConstructor (new Type[] { typeof(double), typeof(UnitType) });
+				return new InstanceDescriptor (ci, new object[] { s.Value, s.Type });
+			}
+#endif
+			
 			return ConvertTo(context, culture, value, destinationType);
 		}
 	}
