@@ -35,7 +35,8 @@ namespace System.Web.UI{
 public class HtmlTextWriter : System.IO.TextWriter {
 
 static HtmlTextWriter(){
-	HtmlTextWriter._tagKeyLookupTable = new Hashtable(97);
+	HtmlTextWriter._tagKeyLookupTable = new Hashtable(97,CaseInsensitiveHashCodeProvider.DefaultInvariant,
+							  CaseInsensitiveComparer.DefaultInvariant);
 	HtmlTextWriter._tagNameLookupArray = new TagInformation[97];
 	HtmlTextWriter.RegisterTag("", HtmlTextWriterTag.Unknown, TagType.Other);
 	HtmlTextWriter.RegisterTag("a", HtmlTextWriterTag.A, TagType.Inline);
@@ -135,8 +136,13 @@ static HtmlTextWriter(){
 	HtmlTextWriter.RegisterTag("wbr", HtmlTextWriterTag.Wbr, TagType.NonClosing);
 	HtmlTextWriter.RegisterTag("xml", HtmlTextWriterTag.Xml, TagType.Other);
 
-	HtmlTextWriter._attrKeyLookupTable = new Hashtable(40);
+#if NET_2_0
+	HtmlTextWriter._attrNameLookupArray = new AttributeInformation[54];
+#else
 	HtmlTextWriter._attrNameLookupArray = new AttributeInformation[40];
+#endif
+	HtmlTextWriter._attrKeyLookupTable = new Hashtable (HtmlTextWriter._attrNameLookupArray.Length,
+					CaseInsensitiveHashCodeProvider.DefaultInvariant, CaseInsensitiveComparer.DefaultInvariant);
 	HtmlTextWriter.RegisterAttribute("accesskey", HtmlTextWriterAttribute.Accesskey, true);
 	HtmlTextWriter.RegisterAttribute("align", HtmlTextWriterAttribute.Align, false);
 	HtmlTextWriter.RegisterAttribute("alt", HtmlTextWriterAttribute.Alt, true);
@@ -177,9 +183,30 @@ static HtmlTextWriter(){
 	HtmlTextWriter.RegisterAttribute("value", HtmlTextWriterAttribute.Value, true);
 	HtmlTextWriter.RegisterAttribute("width", HtmlTextWriterAttribute.Width, false);
 	HtmlTextWriter.RegisterAttribute("wrap", HtmlTextWriterAttribute.Wrap, false);
+#if NET_2_0
+	HtmlTextWriter.RegisterAttribute("abbr", HtmlTextWriterAttribute.Abbr, false);
+	HtmlTextWriter.RegisterAttribute("autocomplete", HtmlTextWriterAttribute.AutoComplete, false);
+	HtmlTextWriter.RegisterAttribute("axis", HtmlTextWriterAttribute.Axis, false);
+	HtmlTextWriter.RegisterAttribute("content", HtmlTextWriterAttribute.Content, false);
+	HtmlTextWriter.RegisterAttribute("coords", HtmlTextWriterAttribute.Coords, false);
+	HtmlTextWriter.RegisterAttribute("_designerRegion", HtmlTextWriterAttribute.DesignerRegion, false);
+	HtmlTextWriter.RegisterAttribute("dir", HtmlTextWriterAttribute.Dir, false);
+	HtmlTextWriter.RegisterAttribute("headers", HtmlTextWriterAttribute.Headers, false);
+	HtmlTextWriter.RegisterAttribute("longdesc", HtmlTextWriterAttribute.Longdesc, false);
+	HtmlTextWriter.RegisterAttribute("rel", HtmlTextWriterAttribute.Rel, false);
+	HtmlTextWriter.RegisterAttribute("scope", HtmlTextWriterAttribute.Scope, false);
+	HtmlTextWriter.RegisterAttribute("shape", HtmlTextWriterAttribute.Shape, false);
+	HtmlTextWriter.RegisterAttribute("usemap", HtmlTextWriterAttribute.Usemap, false);
+	HtmlTextWriter.RegisterAttribute("vcard_name", HtmlTextWriterAttribute.VCardName, false);
+#endif
 
-	HtmlTextWriter._styleKeyLookupTable = new Hashtable(14);
+#if NET_2_0
+	HtmlTextWriter._styleNameLookupArray = new String[42];
+#else
 	HtmlTextWriter._styleNameLookupArray = new String[14];
+#endif
+	HtmlTextWriter._styleKeyLookupTable = new Hashtable (HtmlTextWriter._styleNameLookupArray.Length,
+				CaseInsensitiveHashCodeProvider.DefaultInvariant, CaseInsensitiveComparer.DefaultInvariant);
 	HtmlTextWriter.RegisterStyle("background-color", HtmlTextWriterStyle.BackgroundColor);
 	HtmlTextWriter.RegisterStyle("background-image", HtmlTextWriterStyle.BackgroundImage);
 	HtmlTextWriter.RegisterStyle("border-collapse", HtmlTextWriterStyle.BorderCollapse);
@@ -194,6 +221,36 @@ static HtmlTextWriter(){
 	HtmlTextWriter.RegisterStyle("height", HtmlTextWriterStyle.Height);
 	HtmlTextWriter.RegisterStyle("text-decoration", HtmlTextWriterStyle.TextDecoration);
 	HtmlTextWriter.RegisterStyle("width", HtmlTextWriterStyle.Width);
+#if NET_2_0
+	HtmlTextWriter.RegisterStyle("list-style-image", HtmlTextWriterStyle.ListStyleImage);
+	HtmlTextWriter.RegisterStyle("list-style-type", HtmlTextWriterStyle.ListStyleType);
+	HtmlTextWriter.RegisterStyle("cursor", HtmlTextWriterStyle.Cursor);
+	HtmlTextWriter.RegisterStyle("direction", HtmlTextWriterStyle.Direction);
+	HtmlTextWriter.RegisterStyle("display", HtmlTextWriterStyle.Display);
+	HtmlTextWriter.RegisterStyle("filter", HtmlTextWriterStyle.Filter);
+	HtmlTextWriter.RegisterStyle("font-variant", HtmlTextWriterStyle.FontVariant);
+	HtmlTextWriter.RegisterStyle("left", HtmlTextWriterStyle.Left);
+	HtmlTextWriter.RegisterStyle("margin", HtmlTextWriterStyle.Margin);
+	HtmlTextWriter.RegisterStyle("margin-bottom", HtmlTextWriterStyle.MarginBottom);
+	HtmlTextWriter.RegisterStyle("margin-left", HtmlTextWriterStyle.MarginLeft);
+	HtmlTextWriter.RegisterStyle("margin-right", HtmlTextWriterStyle.MarginRight);
+	HtmlTextWriter.RegisterStyle("margin-top", HtmlTextWriterStyle.MarginTop);
+	HtmlTextWriter.RegisterStyle("overflow", HtmlTextWriterStyle.Overflow);
+	HtmlTextWriter.RegisterStyle("overflow-x", HtmlTextWriterStyle.OverflowX);
+	HtmlTextWriter.RegisterStyle("overflow-y", HtmlTextWriterStyle.OverflowY);
+	HtmlTextWriter.RegisterStyle("padding", HtmlTextWriterStyle.Padding);
+	HtmlTextWriter.RegisterStyle("padding-bottom", HtmlTextWriterStyle.PaddingBottom);
+	HtmlTextWriter.RegisterStyle("padding-left", HtmlTextWriterStyle.PaddingLeft);
+	HtmlTextWriter.RegisterStyle("padding-right", HtmlTextWriterStyle.PaddingRight);
+	HtmlTextWriter.RegisterStyle("padding-top", HtmlTextWriterStyle.PaddingTop);
+	HtmlTextWriter.RegisterStyle("position", HtmlTextWriterStyle.Position);
+	HtmlTextWriter.RegisterStyle("text-align", HtmlTextWriterStyle.TextAlign);
+	HtmlTextWriter.RegisterStyle("text-overflow", HtmlTextWriterStyle.TextOverflow);
+	HtmlTextWriter.RegisterStyle("top", HtmlTextWriterStyle.Top);
+	HtmlTextWriter.RegisterStyle("visibility", HtmlTextWriterStyle.Visibility);
+	HtmlTextWriter.RegisterStyle("white-space", HtmlTextWriterStyle.WhiteSpace);
+	HtmlTextWriter.RegisterStyle("z-index", HtmlTextWriterStyle.ZIndex);
+#endif
 }
 
 public HtmlTextWriter(TextWriter writer):this(writer, "	"){}
@@ -335,7 +392,7 @@ public override void Flush(){
 
 protected HtmlTextWriterAttribute GetAttributeKey(string attrName){
 	if (attrName != null && attrName.Length > 0) {
-		object attr = HtmlTextWriter._attrKeyLookupTable[attrName.ToLower()];
+		object attr = HtmlTextWriter._attrKeyLookupTable[attrName];
 		if (attr != null)
 			return (HtmlTextWriterAttribute) attr;
 	}
@@ -350,7 +407,7 @@ protected string GetAttributeName(HtmlTextWriterAttribute attrKey){
 
 protected HtmlTextWriterStyle GetStyleKey(string styleName){
 	if (styleName != null && styleName.Length > 0) {
-		object style = HtmlTextWriter._styleKeyLookupTable[styleName.ToLower()];
+		object style = HtmlTextWriter._styleKeyLookupTable[styleName];
 		if (style != null)
 			return (HtmlTextWriterStyle) style;
 	}
@@ -358,6 +415,10 @@ protected HtmlTextWriterStyle GetStyleKey(string styleName){
 }
 
 protected string GetStyleName(HtmlTextWriterStyle styleKey){
+	return StaticGetStyleName (styleKey);
+}
+
+internal static string StaticGetStyleName (HtmlTextWriterStyle styleKey){
 	if ((int) styleKey >= 0 && (int) styleKey < HtmlTextWriter._styleNameLookupArray.Length)
 		return HtmlTextWriter._styleNameLookupArray[(int) styleKey];
 	return System.String.Empty;
@@ -365,7 +426,7 @@ protected string GetStyleName(HtmlTextWriterStyle styleKey){
 
 protected virtual HtmlTextWriterTag GetTagKey(string tagName){
 	if (tagName != null && tagName.Length > 0) {
-		object tag = HtmlTextWriter._tagKeyLookupTable[tagName.ToLower()];
+		object tag = HtmlTextWriter._tagKeyLookupTable[tagName];
 		if (tag != null)
 			return (HtmlTextWriterTag) tag;
 	}
