@@ -42,8 +42,8 @@ namespace System.Web.UI
 
 		static RootBuilder ()
 		{
-			htmlControls = new Hashtable (new CaseInsensitiveHashCodeProvider (),
-						      new CaseInsensitiveComparer ()); 
+			htmlControls = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						      CaseInsensitiveComparer.DefaultInvariant); 
 
 			htmlControls.Add ("A", typeof (HtmlAnchor));
 			htmlControls.Add ("BUTTON", typeof (HtmlButton));
@@ -56,9 +56,12 @@ namespace System.Web.UI
 			htmlControls.Add ("TH", typeof (HtmlTableCell));
 			htmlControls.Add ("TR", typeof (HtmlTableRow));
 			htmlControls.Add ("TEXTAREA", typeof (HtmlTextArea));
+#if NET_2_0
+			htmlControls.Add ("HEAD", typeof (HtmlHead));
+#endif
 
-			htmlInputControls = new Hashtable (new CaseInsensitiveHashCodeProvider (),
-							   new CaseInsensitiveComparer ());
+			htmlInputControls = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						      CaseInsensitiveComparer.DefaultInvariant); 
 
 			htmlInputControls.Add ("BUTTON", typeof (HtmlInputButton));
 			htmlInputControls.Add ("SUBMIT", typeof (HtmlInputButton));
