@@ -164,7 +164,7 @@ namespace System.Web.Util
 				return Reduce (basePath + slash + relPath);
 			}
 
-			if (basePath == null || basePath == "")
+			if (basePath == null || basePath == "" || basePath [0] == '~')
 				basePath = HttpRuntime.AppDomainAppVirtualPath;
 
 			if (basePath.Length <= 1)
@@ -256,39 +256,39 @@ namespace System.Web.Util
 			}
 			return false;
 		}
-
+		static char [] path_sep = {'\\', '/'};
+		
 		public static string Reduce (string path)
 		{
-			path = path.Replace ('\\','/');
-
-			string [] parts = path.Split ('/');
-			ArrayList result = new ArrayList ();
-			
+			string [] parts = path.Split (path_sep);
 			int end = parts.Length;
+			
+			int dest = 0;
+			
 			for (int i = 0; i < end; i++) {
 				string current = parts [i];
 				if (current == "." )
 					continue;
 
 				if (current == "..") {
-					if (result.Count == 0) {
+					if (dest == 0) {
 						if (i == 1) // see bug 52599
 							continue;
 
 						throw new HttpException ("Invalid path.");
 					}
 
-					result.RemoveAt (result.Count - 1);
+					dest --;
 					continue;
 				}
 
-				result.Add (current);
+				parts [dest++] = current;
 			}
 
-			if (result.Count == 0)
+			if (dest == 0)
 				return "/";
 
-			return String.Join ("/", (string []) result.ToArray (typeof (string)));
+			return String.Join ("/", parts, 0, dest);
 		}
 		
 		public static string GetDirectory(string url)
@@ -333,32 +333,61 @@ namespace System.Web.Util
 			return url;
 		}
 
+		// appRoot + SessionID + vpath
 		public static string InsertSessionId (string id, string path)
 		{
 			string dir = GetDirectory (path);
 			if (!dir.EndsWith ("/"))
 				dir += "/";
 
-			return Reduce (dir + "(" + id + ")/" + GetFile (path));
+			string appvpath = HttpRuntime.AppDomainAppVirtualPath;
+			if (!appvpath.EndsWith ("/"))
+				appvpath += "/";
+
+			if (path.StartsWith (appvpath))
+				path = path.Substring (appvpath.Length);
+
+			if (path [0] == '/')
+				path = path.Length > 1 ? path.Substring (1) : "";
+
+			return Reduce (appvpath + "(" + id + ")/" + path);
 		}
 
 		public static string GetSessionId (string path)
 		{
-			int len = path.Length;
-			if ((len < SessionId.IdLength + 2) || (path [len - 1] != ')') ||
-			    (path [len - SessionId.IdLength - 2] != '('))
+			string appvpath = HttpRuntime.AppDomainAppVirtualPath;
+			if (path.Length <= appvpath.Length)
 				return null;
 
-			return path.Substring (len - SessionId.IdLength - 1, SessionId.IdLength);
+			path = path.Substring (appvpath.Length);
+			if (path.Length == 0 || path [0] != '/')
+				path = '/' + path;
+
+			int len = path.Length;
+			if ((len < SessionId.IdLength + 3) || (path [1] != '(') ||
+			    (path [SessionId.IdLength + 2] != ')'))
+				return null;
+
+			return path.Substring (2, SessionId.IdLength);
 		}
 
 		public static string RemoveSessionId (string base_path, string file_path)
 		{
-			int len = base_path.Length;
-			string dir = base_path.Substring (0, len - SessionId.IdLength - 2);
+			// Caller did a GetSessionId first
+			int idx = base_path.IndexOf ("/(");
+			string dir = base_path.Substring (0, idx + 1);
 			if (!dir.EndsWith ("/"))
 				dir += "/";
 
+			idx = base_path.IndexOf (")/");
+			if (idx != -1 && base_path.Length > idx + 2) {
+				string dir2 = base_path.Substring (idx + 2);
+				if (!dir2.EndsWith ("/"))
+					dir2 += "/";
+
+				dir += dir2;
+			}
+
 			return Reduce (dir + GetFile (file_path));
 		}
 		
