@@ -1,4 +1,4 @@
-// 
+//
 // System.Web.HttpApplicationFactory
 //
 // Author:
@@ -6,6 +6,7 @@
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
 // (c) 2002,2003 Ximian, Inc. (http://www.ximian.com)
+// (c) Copyright 2004 Novell, Inc. (http://www.novell.com)
 //
 
 //
@@ -33,8 +34,13 @@ using System.Collections;
 using System.IO;
 using System.Reflection;
 using System.Web.UI;
-using System.Web.Compilation;
 using System.Web.SessionState;
+#if !TARGET_J2EE
+using System.Web.Compilation;
+#else
+using System.Web.Configuration;
+using vmw.common;
+#endif
 
 namespace System.Web {
 	class HttpApplicationFactory {
@@ -44,16 +50,32 @@ namespace System.Web {
 		private bool _appInitialized;
 		private bool _appFiredEnd;
 
-		private Stack			_appFreePublicList;
-		private int				_appFreePublicInstances;
-		static private int	_appMaxFreePublicInstances = 32;
+		private Stack _appFreePublicList;
+		private int _appFreePublicInstances;
+
+		FileSystemWatcher appFileWatcher;
+		FileSystemWatcher binWatcher;
+		
+		static private int _appMaxFreePublicInstances = 32;
 
 		private HttpApplicationState _state;
 
 		static IHttpHandler custApplication;
 
+#if TARGET_J2EE
+		static private HttpApplicationFactory s_Factory {
+					get {
+						HttpApplicationFactory factory = (HttpApplicationFactory)AppDomain.CurrentDomain.GetData("HttpApplicationFactory");
+						if (factory == null) {
+							factory = new HttpApplicationFactory();
+							AppDomain.CurrentDomain.SetData("HttpApplicationFactory", factory);
+						}
+						return factory;
+					}
+		}
+#else
 		static private HttpApplicationFactory s_Factory = new HttpApplicationFactory();
-
+#endif
 		public HttpApplicationFactory() {
 			_appInitialized = false;
 			_appFiredEnd = false;
@@ -71,21 +93,41 @@ namespace System.Web {
 
 			return Path.Combine (physicalAppPath, "global.asax");
 		}
+#if TARGET_J2EE
+		void CompileApp(HttpContext context)
+		{
 
-		private void CompileApp(HttpContext context) {
-			if (File.Exists(_appFilename)) {
-				// Setup filemonitor for all filedepend also. CacheDependency?
+			try
+			{
+				String url = IAppDomainConfig.WAR_ROOT_SYMBOL+"/global.asax";
+				_appType = System.Web.GH.PageMapper.GetObjectType(url);
+			}
+			catch (Exception e)
+			{
+				_appType = typeof (System.Web.HttpApplication);
+			}
+			_state = new HttpApplicationState ();
 
+		}
+#endif
+
+#if !TARGET_J2EE
+		void CompileApp (HttpContext context)
+		{
+			if (File.Exists (_appFilename)) {
 				_appType = ApplicationFileParser.GetCompiledApplicationType (_appFilename, context);
 				if (_appType == null) {
 					string msg = String.Format ("Error compiling application file ({0}).", _appFilename);
 					throw new ApplicationException (msg);
 				}
+
+				appFileWatcher = CreateWatcher (_appFilename, new FileSystemEventHandler (OnAppFileChanged));
 			} else {
 				_appType = typeof (System.Web.HttpApplication);
 				_state = new HttpApplicationState ();
 			}
 		}
+#endif
 
 		static bool IsEventHandler (MethodInfo m)
 		{
@@ -128,7 +170,6 @@ namespace System.Web {
 		{
 			Type appType = app.GetType ();
 			Hashtable appTypeEventHandlers = new Hashtable ();
-			ArrayList evtMethods = new ArrayList ();
 			BindingFlags flags = BindingFlags.Public    |
 					     BindingFlags.NonPublic | 
 					     BindingFlags.Instance |
@@ -143,7 +184,7 @@ namespace System.Web {
 			return appTypeEventHandlers;
 		}
 
-		static bool FireEvents (string method_name, object target, object [] args)
+		static bool FireEvent (string method_name, object target, object [] args)
 		{
 			Hashtable possibleEvents = GetApplicationTypeEvents ((HttpApplication) target);
 			MethodInfo method = possibleEvents [method_name] as MethodInfo;
@@ -151,34 +192,56 @@ namespace System.Web {
 				return false;
 
 			if (method.GetParameters ().Length == 0)
-				method.Invoke (target, null);
-			else
-				method.Invoke (target, args);
+				args = null;
 
+			try {
+				method.Invoke (target, args);
+			} catch {
+				// Ignore any exception here
+			}
 			return true;
 		}
-		
+
 		internal static void FireOnAppStart (HttpApplication app)
 		{
 			object [] args = new object [] {app, EventArgs.Empty};
-			FireEvents ("Application_Start", app, args);
+			FireEvent ("Application_Start", app, args);
 		}
 
 		void FireOnAppEnd ()
 		{
-		//	FireEvents ("Application_End", this, new object [] {this, EventArgs.Empty});
+			if (_appType == null)
+				return; // we didn't even get an application
+
+			HttpApplication app = (HttpApplication) HttpRuntime.CreateInternalObject (_appType);
+			AttachEvents (app);
+			FireEvent ("Application_End", app, new object [] {this, EventArgs.Empty});
+			app.Dispose ();
 		}
 
-		void FireOnSessionStart (HttpSessionState state, object source, EventArgs args)
+		FileSystemWatcher CreateWatcher (string file, FileSystemEventHandler hnd)
 		{
-		//	FireEvents ("Session_Start", state, new object [] {source, EventArgs.Empty});
+			FileSystemWatcher watcher = new FileSystemWatcher ();
+
+			watcher.Path = Path.GetFullPath (Path.GetDirectoryName (file));
+			watcher.Filter = Path.GetFileName (file);
+
+			watcher.Changed += hnd;
+			watcher.Created += hnd;
+			watcher.Deleted += hnd;
+
+			watcher.EnableRaisingEvents = true;
+
+			return watcher;
 		}
-		
-		void FireOnSessionEnd (HttpSessionState state, object source, EventArgs args)
+
+		void OnAppFileChanged (object sender, FileSystemEventArgs args)
 		{
-		//	FireEvents ("Session_End", state, new object [] {source, args});
+			binWatcher.EnableRaisingEvents = false;
+			appFileWatcher.EnableRaisingEvents = false;
+			HttpRuntime.UnloadAppDomain ();
 		}
-	
+
 		private void InitializeFactory (HttpContext context)
 		{
 			_appFilename = GetAppFilename (context);
@@ -191,6 +254,13 @@ namespace System.Web {
 			// Startup
 			app.Startup(context, HttpApplicationFactory.ApplicationState);
 
+			// Shutdown the application if bin directory changes.
+			string binFiles = HttpRuntime.BinDirectory;
+			if (Directory.Exists (binFiles))
+				binFiles = Path.Combine (binFiles, "*.*");
+
+			binWatcher = CreateWatcher (binFiles, new FileSystemEventHandler (OnAppFileChanged));
+
 			// Fire OnAppStart
 			HttpApplicationFactory.FireOnAppStart (app);
 
@@ -299,6 +369,23 @@ namespace System.Web {
 			}
 		}
 
+#if TARGET_J2EE
+		internal HttpApplication GetPublicInstance()
+		{
+			HttpApplication app = null;
+
+			lock (_appFreePublicList)
+			{
+				if (_appFreePublicInstances > 0)
+				{
+					app = (HttpApplication) _appFreePublicList.Pop();
+					_appFreePublicInstances--;
+				}
+			}
+			return app;
+		}
+#endif
+
 		private IHttpHandler GetPublicInstance(HttpContext context) {
 			HttpApplication app = null;
 
@@ -348,6 +435,17 @@ namespace System.Web {
 		}
 		
 		static internal HttpApplicationState ApplicationState {
+#if TARGET_J2EE
+			get {
+				if (null == s_Factory._state) {
+					HttpStaticObjectsCollection app = null;
+					HttpStaticObjectsCollection ses = null;
+					s_Factory._state = new HttpApplicationState (app, ses);
+				}
+
+				return s_Factory._state;
+			}
+#else
 			get {
 				if (null == s_Factory._state) {
 					HttpStaticObjectsCollection app = MakeStaticCollection (GlobalAsaxCompiler.ApplicationObjects);
@@ -357,6 +455,7 @@ namespace System.Web {
 
 				return s_Factory._state;
 			}
+#endif
 		}
 
 		internal static void EndApplication() {
