@@ -43,22 +43,22 @@ namespace System.Web
 		const int HaveBeta = 4;
 		const int HaveBrowser = 5;
 		const int HaveCDF = 6;
-		const int HaveClrVersion = 7;
+		//const int HaveClrVersion = 7;
 		const int HaveCookies = 8;
 		const int HaveCrawler = 9;
-		const int HaveEcmaScriptVersion = 10;
+		//const int HaveEcmaScriptVersion = 10;
 		const int HaveFrames = 11;
 		const int HaveJavaApplets = 12;
 		const int HaveJavaScript = 13;
 		const int HaveMajorVersion = 14;
 		const int HaveMinorVersion = 15;
-		const int HaveMSDomVersion = 16;
+		//const int HaveMSDomVersion = 16;
 		const int HavePlatform = 17;
 		const int HaveTables = 18;
-		const int HaveTagWriter = 19;
+		//const int HaveTagWriter = 19;
 		const int HaveVBScript = 20;
 		const int HaveVersion = 21;
-		const int HaveW3CDomVersion = 22;
+		//const int HaveW3CDomVersion = 22;
 		const int HaveWin16 = 23;
 		const int HaveWin32 = 24;
 
@@ -72,20 +72,21 @@ namespace System.Web
 		Version clrVersion;
 		bool cookies;
 		bool crawler;
-		Version ecmaScriptVersion;
+		//Version ecmaScriptVersion;
 		bool frames;
 		bool javaApplets;
 		bool javaScript;
 		int majorVersion;
 		double minorVersion;
-		Version msDomVersion;
+		//Version msDomVersion;
 		string platform;
 		bool tables;
-		Type tagWriter;
+		//Type tagWriter;
 		bool vbscript;
 		string version;
-		Version w3CDomVersion;
+		//Version w3CDomVersion;
 		bool win16;
+		bool win32;
 		Version [] clrVersions;
 		internal string useragent;
 
@@ -340,7 +341,12 @@ namespace System.Web
 
 		public bool Win32 {
 			get {
-				return !Win16;
+				if (!Get (HaveWin32)) {
+					Set (HaveWin32);
+					win32 = ReadBoolean ("win32", false);
+				}
+
+				return win32;
 			}
 		}
 
