@@ -34,8 +34,10 @@ using System.IO;
 using System.Runtime.Serialization;
 using System.Runtime.InteropServices;
 using System.Text;
-using System.Web.Compilation;
 using System.Web.Util;
+#if !TARGET_J2EE
+using System.Web.Compilation;
+#endif
 
 namespace System.Web
 {
@@ -45,8 +47,6 @@ namespace System.Web
 	public class HttpException : ExternalException
 	{
 		int http_code = 500;
-		int hr;
-		string fileName;
 
 		public HttpException () : base ()
 		{
@@ -79,15 +79,13 @@ namespace System.Web
 		}
 #endif
 
-		public HttpException (int iHttpCode, string sMessage, int iHR) : base (sMessage)
+		public HttpException (int iHttpCode, string sMessage, int iHR) : base (sMessage, iHR)
 		{
 			http_code = iHttpCode;
-			hr = iHR;
 		}
 
-		public HttpException (string sMessage, int iHR) : base (sMessage)
+		public HttpException (string sMessage, int iHR) : base (sMessage, iHR)
 		{
-			hr = iHR;
 		}
 	
 		public HttpException (int iHttpCode,
@@ -98,13 +96,6 @@ namespace System.Web
 			http_code = iHttpCode;
 		}
 
-		internal HttpException (string message, string fileName)
-			: base (message)
-		{
-			this.fileName = fileName;
-			
-		}
-		
 		public string GetHtmlErrorMessage ()
 		{
 			if (!(this.InnerException is HtmlizedException))
@@ -144,11 +135,17 @@ namespace System.Web
 				builder.Append ("<table summary=\"Stack Trace\" width=\"100%\" " +
 						"bgcolor=\"#ffffc\">\r\n<tr><td>");
 				WriteTextAsCode (builder, InnerException.ToString ());
+#if TARGET_J2EE //Required, because toString of Java doesn't print stackTrace
+				WriteTextAsCode (builder, InnerException.StackTrace);
+#endif
 				builder.Append ("</td></tr>\r\n</table>\r\n<p>\r\n");
 			}
 
 			builder.Append ("<hr>\r\n</body>\r\n</html>\r\n");
 			builder.AppendFormat ("<!--\r\n{0}\r\n-->\r\n", HttpUtility.HtmlEncode (this.ToString ()));
+#if TARGET_J2EE //Required, because toString of Java doesn't print stackTrace
+			builder.AppendFormat ("<!--\r\n{0}\r\n-->\r\n", HttpUtility.HtmlEncode (this.StackTrace));
+#endif
 
 			return builder.ToString ();
 		}
@@ -161,7 +158,44 @@ namespace System.Web
 			string res = HttpUtility.HtmlEncode (s);
 			return res.Replace ("\r\n", "<br />");
 		}
+#if TARGET_J2EE
+		string GetHtmlizedErrorMessage ()
+		{
+			StringBuilder builder = new StringBuilder ("<html>\r\n<title>");
+			HtmlizedException exc = (HtmlizedException) this.InnerException;
+			builder.Append (exc.Title);
+			builder.AppendFormat ("</title><body bgcolor=\"white\">" +
+					      "<h1><font color=\"red\">Server Error in '{0}' " +
+					      "Application</font></h1><hr>\r\n",
+					      HttpRuntime.AppDomainAppVirtualPath);
 		
+			builder.AppendFormat ("<h2><font color=\"maroon\"><i>{0}</i></font></h2>\r\n", exc.Title);
+			builder.AppendFormat ("<b>Description: </b>{0}\r\n<p>\r\n", HtmlEncode (exc.Description));
+			builder.AppendFormat ("<b>Error message: </b>{0}\r\n<p>\r\n", HtmlEncode (exc.ErrorMessage));
+
+			if (exc.FileName != null)
+				builder.AppendFormat ("<b>File name: </b> {0}", HtmlEncode (exc.FileName));
+
+			if (exc.FileText != null) {
+				if (exc.SourceFile != exc.FileName)
+					builder.AppendFormat ("<p><b>Source File: </b>{0}", exc.SourceFile);
+
+				builder.Append ("\r\n<p>\r\n");
+
+				builder.Append ("<table summary=\"Source file\" width=\"100%\" " +
+							"bgcolor=\"#ffffc\">\r\n<tr><td>");
+					WriteSource (builder, exc);
+					builder.Append ("</td></tr>\r\n</table>\r\n<p>\r\n");
+			}
+
+			builder.Append ("<hr>\r\n</body>\r\n</html>\r\n");
+			builder.AppendFormat ("<!--\r\n{0}\r\n-->\r\n", HtmlEncode (exc.ToString ()));
+			builder.AppendFormat ("<!--\r\n{0}\r\n-->\r\n", HtmlEncode (exc.StackTrace));
+			return builder.ToString ();
+		}
+#endif
+
+#if !TARGET_J2EE
 		string GetHtmlizedErrorMessage ()
 		{
 			StringBuilder builder = new StringBuilder ("<html>\r\n<title>");
@@ -208,7 +242,7 @@ namespace System.Web
 			builder.AppendFormat ("<!--\r\n{0}\r\n-->\r\n", HtmlEncode (exc.ToString ()));
 			return builder.ToString ();
 		}
-
+#endif
 		static void WriteTextAsCode (StringBuilder builder, string text)
 		{
 			builder.Append ("<code><pre>\r\n");
@@ -216,6 +250,17 @@ namespace System.Web
 			builder.Append ("</pre></code>\r\n");
 		}
 
+#if TARGET_J2EE
+		static void WriteSource (StringBuilder builder, HtmlizedException e)
+		{
+			builder.Append ("<code><pre>");
+			WritePageSource (builder, e);
+			builder.Append ("</pre></code>\r\n");
+		}
+
+#endif
+
+#if !TARGET_J2EE
 		static void WriteSource (StringBuilder builder, HtmlizedException e)
 		{
 			builder.Append ("<code><pre>");
@@ -226,6 +271,7 @@ namespace System.Web
 
 			builder.Append ("</pre></code>\r\n");
 		}
+#endif
 		
 		static void WriteCompilationSource (StringBuilder builder, HtmlizedException e)
 		{
