@@ -33,11 +33,16 @@
 using System;
 using System.Collections;
 using System.Collections.Specialized;
+using System.Globalization;
 using System.IO;
 using System.Text;
 using System.Web.Configuration;
 using System.Web.Util;
 
+#if TARGET_J2EE
+using vmw.common;
+#endif
+
 namespace System.Web {
 	[MonoTODO("Review security in all path access function")]
 	public sealed class HttpRequest {
@@ -84,16 +89,23 @@ namespace System.Web {
 		private HttpBrowserCapabilities _browser;
 
 		private HttpCookieCollection cookies;
-		private bool rewritten;
 		Stream userFilter;
 		HttpRequestStream requestFilter;
 		string clientTarget;
+		string currentExePath;
 #if NET_1_1
 		bool validateCookies;
 		bool validateForm;
 		bool validateQueryString;
+
+		bool checkedCookies;
+		bool checkedForm;
+		bool checkedQueryString;
 #endif
 
+#if TARGET_J2EE
+		private string _sGhFilePath;
+#endif
 		public HttpRequest(string Filename, string Url, string Querystring) {
 			_iContentLength = -1;
 			_iTotalBytes = -1;
@@ -117,11 +129,36 @@ namespace System.Web {
 			_iTotalBytes = -1;
 		}
 
-		static private string MakeServerVariableFromHeader(string header) {
-			return "HTTP_" + header.ToUpper().Replace("-", "_");
+		internal void AddHeaderVariables (ServerVariablesCollection coll)
+		{
+			if (null == _WorkerRequest)
+				return;
+
+			string hname;
+			string hvalue;
+
+			// Add all known headers
+			for (int i = 0; i < HttpWorkerRequest.RequestHeaderMaximum; i++) {
+				hvalue = _WorkerRequest.GetKnownRequestHeader (i);
+				if (null != hvalue && hvalue.Length > 0) {
+					hname = HttpWorkerRequest.GetKnownRequestHeaderName (i);
+					if (null != hname && hname.Length > 0)
+						coll.Add ("HTTP_" + hname.ToUpper ().Replace ('-', '_'), hvalue);
+				}
+			}
+
+			// Get all other headers
+			string [][] unknown = _WorkerRequest.GetUnknownRequestHeaders ();
+			if (null != unknown) {
+				for (int i = 0; i < unknown.Length; i++) {
+					hname = unknown [i][0];
+					hvalue = unknown [i][1];
+					coll.Add ("HTTP_" + hname.ToUpper ().Replace ('-', '_'), hvalue);
+				}
+			}
 		}
 
-		internal string GetAllHeaders(bool Raw) {
+		internal string GetAllHeaders(bool raw) {
 			StringBuilder oData;
 
 			if (null == _WorkerRequest) {
@@ -134,13 +171,18 @@ namespace System.Web {
 			string sHeaderName;
 			int iCount = 0;
 
-			// Add all know headers
-			for (; iCount != 40; iCount++) {
+			// Add all known headers
+			for (; iCount != HttpWorkerRequest.RequestHeaderMaximum; iCount++) {
 				sHeaderValue = _WorkerRequest.GetKnownRequestHeader(iCount);
 				if (null != sHeaderValue && sHeaderValue.Length > 0) {
-					sHeaderName = _WorkerRequest.GetKnownRequestHeader(iCount);
+					sHeaderName = HttpWorkerRequest.GetKnownRequestHeaderName(iCount);
 					if (null != sHeaderName && sHeaderName.Length > 0) {
-						oData.Append(sHeaderName);
+						if (raw) {
+							oData.Append(sHeaderName);
+						} else {
+							oData.Append ("HTTP_");
+							oData.Append (sHeaderName.ToUpper ().Replace ('-', '_'));
+						}
 						oData.Append(": ");
 						oData.Append(sHeaderValue);
 						oData.Append("\r\n");
@@ -152,7 +194,13 @@ namespace System.Web {
 			string [][] arrUnknownHeaders = _WorkerRequest.GetUnknownRequestHeaders();
 			if (null != arrUnknownHeaders) {
 				for (iCount = 0; iCount != arrUnknownHeaders.Length; iCount++) {
-					oData.Append(arrUnknownHeaders[iCount][0]);
+					string hname = arrUnknownHeaders[iCount][0];
+					if (raw) {
+						oData.Append (hname);
+					} else {
+						oData.Append ("HTTP_");
+						oData.Append (hname.ToUpper ().Replace ('-', '_'));
+					}
 					oData.Append(": ");
 					oData.Append(arrUnknownHeaders[iCount][1]);
 					oData.Append("\r\n");
@@ -176,30 +224,31 @@ namespace System.Web {
 
 		private void ParseFormData ()
 		{
-			string contentType = ContentType;
-			if (0 == String.Compare (contentType, "application/x-www-form-urlencoded", true)) {
+			string content_type = ContentType;
+			if (content_type == null)
+				return;
+
+			content_type = content_type.ToLower (CultureInfo.InvariantCulture);
+			if (content_type == "application/x-www-form-urlencoded") {
 				byte [] arrData = GetRawContent ();
 				Encoding enc = ContentEncoding;
 				string data = enc.GetString (arrData);
 				_oFormData = new HttpValueCollection (data, true, enc);
 				return;
 			}
-			if (!ContentType.StartsWith ("multipart/form-data")) {
-				if (contentType.Length > 0)
-					Console.WriteLine ("Content-Type -> {0} not supported", contentType);
-				_oFormData = new HttpValueCollection ();
-				return;
-			}
-			
-			MultipartContentElement [] parts = GetMultipartFormData ();
-			_oFormData = new HttpValueCollection ();
-			if (parts == null) return;
-				
-			foreach (MultipartContentElement p in parts) {
-				if (!p.IsFormItem) continue;
-				_oFormData.Add (p.Name, p.GetString (ContentEncoding));
-			}
 
+			_oFormData = new HttpValueCollection ();
+			if (StrUtils.StartsWith (content_type, "multipart/form-data")) {
+				MultipartContentElement [] parts = GetMultipartFormData ();
+				if (parts == null)
+					return;
+				Encoding content_encoding = ContentEncoding;
+				foreach (MultipartContentElement p in parts) {
+					if (p.IsFormItem) {
+						_oFormData.Add (p.Name, p.GetString (content_encoding));
+					}
+				}
+			}
 		}
 
 		[MonoTODO("void Dispose")]
@@ -226,31 +275,40 @@ namespace System.Web {
 				_arrRawContent = new byte [0];
 
 			int length = ContentLength;
+			HttpRuntimeConfig cfg = (HttpRuntimeConfig) _oContext.GetConfig ("system.web/httpRuntime");
+			int maxRequestLength = cfg.MaxRequestLength * 1024;
+			if (length > maxRequestLength) {
+				throw new HttpException (400, "Maximum request length exceeded.");
+			}
+				
 			if (_WorkerRequest.IsEntireEntityBodyIsPreloaded () || length <= _arrRawContent.Length)
 				return _arrRawContent;
 
-			byte [] arrBuffer = new byte [Math.Min (16384, length)];
-			MemoryStream ms = new MemoryStream (arrBuffer.Length);
-			ms.Write (_arrRawContent, 0, _arrRawContent.Length);
+			byte [] result = new byte [length];
+			int offset = _arrRawContent.Length;
+			Buffer.BlockCopy (_arrRawContent, 0, result, 0, offset);
+
 			int read = 0;
+			byte [] arrBuffer = new byte [Math.Min (16384, length)];
 			int bufLength = arrBuffer.Length;
-			for (int loaded = _arrRawContent.Length; loaded < length; loaded += read) {
-				if (length - loaded < bufLength)
-					bufLength = length - loaded;
+			for (; offset < length; offset += read) {
+				if (length - offset < bufLength)
+					bufLength = length - offset;
 
 				read = _WorkerRequest.ReadEntityBody (arrBuffer, bufLength);
 				if (read == 0 ||read == -1 )
 					break;
 
-				ms.Write (arrBuffer, 0, read);
+				if (length > maxRequestLength || offset + read > maxRequestLength)
+					throw new HttpException (400, "Maximum request length exceeded.");
+
+				Buffer.BlockCopy (arrBuffer, 0, result, offset, read);
 			}
 
-			byte [] msBuffer = ms.GetBuffer ();
-			if (msBuffer.Length == length)
-				_arrRawContent = msBuffer;
-			else
-				_arrRawContent = ms.ToArray ();
+			if (offset < length)
+				throw new HttpException (400, "Data length is shorter than Content-Length.");
 
+			_arrRawContent = result;
 			if (userFilter != null) {
 				requestFilter.Set (_arrRawContent, 0, _arrRawContent.Length);
 				int userLength = Convert.ToInt32 (userFilter.Length - userFilter.Position);
@@ -491,18 +549,22 @@ namespace System.Web {
 					cookies = new HttpCookieCollection (null, false);
 					if (_WorkerRequest != null)
 						GetCookies ();
-#if NET_1_1
-					if (validateCookies)
-						ValidateCookieCollection (cookies);
-#endif
 				}
-
+#if NET_1_1
+				if (validateCookies && !checkedCookies) {
+					ValidateCookieCollection (cookies);
+					checkedCookies = true;
+				}
+#endif
 				return cookies;
 			}
 		}
 
 		public string CurrentExecutionFilePath {
 			get {
+				if (currentExePath != null)
+					return currentExePath;
+
 				return FilePath;
 			}
 		}
@@ -522,6 +584,25 @@ namespace System.Web {
 			}
 		}
 
+#if TARGET_J2EE
+		internal string GhFilePath {
+			get {
+				if (null == _sGhFilePath) {
+					_sGhFilePath = FilePath;
+					if (_sGhFilePath == null)
+						return null;
+
+					if (_sGhFilePath.StartsWith(IAppDomainConfig.WAR_ROOT_SYMBOL))
+						_sGhFilePath = _sGhFilePath.Substring(IAppDomainConfig.WAR_ROOT_SYMBOL.Length);
+					if (_sGhFilePath.StartsWith(HttpRuntime.AppDomainAppVirtualPath))
+						_sGhFilePath = _sGhFilePath.Substring(HttpRuntime.AppDomainAppVirtualPath.Length);
+				}
+
+				return _sGhFilePath;
+			}
+		}
+#endif
+
 		HttpFileCollection files;
 		public HttpFileCollection Files {
 			get {
@@ -537,7 +618,8 @@ namespace System.Web {
 		
 		void FillPostedFiles ()
 		{
-			if (!ContentType.StartsWith ("multipart/form-data")) return;
+			if (!StrUtils.StartsWith (ContentType, "multipart/form-data"))
+				return;
 			
 			MultipartContentElement [] parts = GetMultipartFormData ();
 			if (parts == null) return;
@@ -582,11 +664,13 @@ namespace System.Web {
 			get {
 				if (_oFormData == null) {
 					ParseFormData ();
-#if NET_1_1
-					if (validateForm)
-						ValidateNameValueCollection ("Form", _oFormData);
-#endif
 				}
+#if NET_1_1
+				if (validateForm && !checkedForm) {
+					ValidateNameValueCollection ("Form", _oFormData);
+					checkedForm = true;
+				}
+#endif
 
 				return _oFormData;
 			}
@@ -711,9 +795,11 @@ namespace System.Web {
 					_oParams.Merge(QueryString);
 					_oParams.Merge(Form);
 					_oParams.Merge(ServerVariables);
-					string [] cookies = Cookies.AllKeys;
-					foreach (string k in cookies)
-						_oParams.Add (k, Cookies [k].ToString ());
+					int count = Cookies.Count;
+					for (int i = 0; i< count; i++) {
+						HttpCookie cookie = Cookies [i];
+						_oParams.Add (cookie.Name, cookie.Value);
+					}
 					_oParams.MakeReadOnly();
 				}
 
@@ -765,14 +851,8 @@ namespace System.Web {
 
 		public string PhysicalPath {
 			get {
-				if (_sPathTranslated == null && _WorkerRequest != null) {
-					if (rewritten)
-						_sPathTranslated = _WorkerRequest.GetFilePathTranslated ();
-
-					string verifyPath = _WorkerRequest.MapPath (FilePath);
-					if (null == _sPathTranslated)
-						_sPathTranslated = verifyPath;
-				}
+				if (_sPathTranslated == null && _WorkerRequest != null)
+					_sPathTranslated = _WorkerRequest.MapPath (CurrentExecutionFilePath);
 
 				return _sPathTranslated;
 			}
@@ -788,12 +868,13 @@ namespace System.Web {
 						_oQueryString = new HttpValueCollection(QueryStringRaw, true,
 											Encoding.ASCII);
 					}
-#if NET_1_1
-					if (validateQueryString)
-						ValidateNameValueCollection ("QueryString", _oQueryString);
-#endif
 				}
-
+#if NET_1_1
+				if (validateQueryString && !checkedQueryString) {
+					ValidateNameValueCollection ("QueryString", _oQueryString);
+					checkedQueryString = true;
+				}
+#endif
 				return _oQueryString;
 			}
 		}
@@ -1004,25 +1085,20 @@ namespace System.Web {
                         }
                 }
                 
-		public byte [] BinaryRead(int count) {
-			int iSize = TotalBytes;
-			if (iSize == 0) {
-				throw new ArgumentException();
+		public byte [] BinaryRead (int count)
+		{
+			if (count < 0 || count > TotalBytes)
+				throw new ArgumentOutOfRangeException ("count");
+
+			byte [] data = new byte [count];
+			int nread = InputStream.Read (data, 0, count);
+			if (nread != count) {
+				byte [] tmp = new byte [nread];
+				Buffer.BlockCopy (data, 0, tmp, 0, nread);
+				data = tmp;
 			}
 
-			byte [] arrData = new byte[iSize];
-			
-			int iRetSize = InputStream.Read(arrData, 0, iSize);
-			if (iRetSize != iSize) {
-				byte [] tmpData = new byte[iRetSize];
-				if (iRetSize > 0) {
-					Array.Copy(arrData, 0, tmpData, 0, iRetSize);
-				}
-
-				arrData = tmpData;
-			}
-
-			return arrData;
+			return data;
 		}
 
 		public int [] MapImageCoordinates(string ImageFieldName) {
@@ -1065,6 +1141,15 @@ namespace System.Web {
 			if (_WorkerRequest == null)
 				throw new HttpException ("No HttpWorkerRequest!!!");
 
+#if TARGET_J2EE
+			if (baseVirtualDir.Equals(BaseVirtualDir))
+			{
+				string val =  System.Web.GH.PageMapper.GetFromMapPathCache(virtualPath);
+				if (val != null)
+					return val;
+			}
+#endif
+
 			if (virtualPath == null || virtualPath.Length == 0)
 				virtualPath = ".";
 			else
@@ -1072,7 +1157,10 @@ namespace System.Web {
 
 			if (virtualPath.IndexOf (':') != -1)
 				throw new ArgumentException ("Invalid path -> " + virtualPath);
-
+#if TARGET_J2EE
+			if (virtualPath.StartsWith(IAppDomainConfig.WAR_ROOT_SYMBOL))
+				return 	virtualPath;
+#endif
 			if (System.IO.Path.DirectorySeparatorChar != '/')
 				virtualPath = virtualPath.Replace (System.IO.Path.DirectorySeparatorChar, '/');
 
@@ -1087,7 +1175,7 @@ namespace System.Web {
 			}
 
 			if (!allowCrossAppMapping) {
-				if (!virtualPath.ToLower ().StartsWith (RootVirtualDir.ToLower ()))
+				if (!StrUtils.StartsWith (virtualPath, RootVirtualDir, true))
 					throw new HttpException ("Mapping across applications not allowed.");
 
 				if (RootVirtualDir.Length > 1 && virtualPath.Length > 1 && virtualPath [0] != '/')
@@ -1139,11 +1227,14 @@ namespace System.Web {
 		}
 #endif
 		
-		internal void SetFilePath (string filePath)
+		internal void SetCurrentExePath (string filePath)
 		{
+			currentExePath = filePath;
+			_sPath = filePath;
 			_sFilePath = filePath;
 			_sRequestRootVirtualDir = null;
 			baseVirtualDir = null;
+			_sPathTranslated = null;
 		}
 
 		internal void SetPathInfo (string pathInfo)
@@ -1205,7 +1296,7 @@ namespace System.Web {
 		{
 			string v = "\"" + value + "\"";
 			if (v.Length > 20)
-				v = v.Substring (16) + "...\"";
+				v = v.Substring (0, 16) + "...\"";
 
 			string msg = String.Format ("A potentially dangerous Request.{0} value was " +
 						    "detected from the client ({1}={2}).", name, key, v);
@@ -1213,17 +1304,15 @@ namespace System.Web {
 			throw new HttpRequestValidationException (msg);
 		}
 
-		static char [] dangerousChars = "<>".ToCharArray ();
 		static bool CheckString (string val)
 		{
 			if (val == null)
 				return false;
 
 			//TODO: More checks
-			if (val.IndexOfAny (dangerousChars) != -1 && val.Length > 1) {
-				foreach (char c in val)
-					if (Array.IndexOf (dangerousChars, c) != -1)
-						return true;
+			foreach (char c in val) {
+				if (c == '<' || c == '>' || c == '\xff1c' || c == '\xff1e')
+					return true;
 			}
 
 			return false;
