@@ -7,6 +7,27 @@
 //   Tim Coleman (tim@timcoleman.com)
 //   Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //
+
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
 using System;
 using System.Collections;
 using System.Globalization;
@@ -19,8 +40,6 @@ namespace System.Web {
 
 		#region Fields
 	
-		const string _hex = "0123456789ABCDEF";
-		const string _chars = "<>;:.?=&@*+%/\\";
 		static Hashtable entities;
 		static object lock_ = new object ();
 	
@@ -316,8 +335,10 @@ namespace System.Web {
 			if (null == s) 
 				return null;
 	
+			if (s.IndexOf ('&') == -1 && s.IndexOf ('"') == -1)
+				return s;
+
 			StringBuilder output = new StringBuilder ();
-	
 			foreach (char c in s) 
 				switch (c) {
 				case '&' : 
@@ -349,6 +370,9 @@ namespace System.Web {
 			if (null == s) 
 				return null;
 
+			if (s.IndexOf ('%') == -1 && s.IndexOf ('+') == -1)
+				return s;
+
 			if (e == null)
 				e = Encoding.UTF8;
 	
@@ -418,7 +442,7 @@ namespace System.Web {
 			int value = 0;
 			int end = length + offset;
 			for (int i = offset; i < end; i++)
-				value = (value << 4) + GetInt (bytes [offset]);
+				value = (value << 4) + GetInt (bytes [i]);
 
 			return (char) value;
 		}
@@ -448,10 +472,10 @@ namespace System.Web {
 							output.Append (GetChars (acc, e));
 							acc.SetLength (0);
 						}
-						output.Append (GetChar (bytes, offset + 2, 4));
+						output.Append (GetChar (bytes, i + 2, 4));
 						i += 5;
 					} else {
-						acc.WriteByte ((byte) GetChar (bytes, offset + 1, 2));
+						acc.WriteByte ((byte) GetChar (bytes, i + 1, 2));
 						i += 2;
 					}
 					continue;
@@ -520,7 +544,7 @@ namespace System.Web {
 				if (c == '+')
 					c = ' ';
 				else if (c == '%' && i < end - 2) {
-					c = GetChar (bytes, i, 2);
+					c = GetChar (bytes, i + 1, 2);
 					i += 2;
 				}
 				result.WriteByte ((byte) c);
@@ -696,43 +720,102 @@ namespace System.Web {
 			if (s == null)
 				throw new ArgumentNullException ("s");
 
-			bool insideEntity = false; // used to indicate that we are in a potential entity
-			string entity = String.Empty;
+			if (s.IndexOf ('&') == -1)
+				return s;
+
+			StringBuilder entity = new StringBuilder ();
 			StringBuilder output = new StringBuilder ();
+			int len = s.Length;
+			// 0 -> nothing,
+			// 1 -> right after '&'
+			// 2 -> between '&' and ';' but no '#'
+			// 3 -> '#' found after '&' and getting numbers
+			int state = 0;
+			int number = 0;
+			bool have_trailing_digits = false;
 	
-			foreach (char c in s) {
-				switch (c) {
-				case '&' :
-					output.Append (entity);
-					entity = "&";
-					insideEntity = true;
-					break;
-				case ';' :
-					if (!insideEntity) {
+			for (int i = 0; i < len; i++) {
+				char c = s [i];
+				if (state == 0) {
+					if (c == '&') {
+						entity.Append (c);
+						state = 1;
+					} else {
 						output.Append (c);
-						break;
+					}
+					continue;
+				}
+
+				if (c == '&') {
+					state = 1;
+					if (have_trailing_digits) {
+						entity.Append (number.ToString (CultureInfo.InvariantCulture));
+						have_trailing_digits = false;
 					}
 
-					entity += c;
-					int length = entity.Length;
-					if (length >= 2 && entity[1] == '#' && entity[2] != ';')
-						entity = ((char) Int32.Parse (entity.Substring (2, entity.Length - 3))).ToString();
-					else if (length > 1 && Entities.ContainsKey (entity.Substring (1, entity.Length - 2)))
-						entity = Entities [entity.Substring (1, entity.Length - 2)].ToString ();
-					
-					output.Append (entity);
-					entity = String.Empty;
-					insideEntity = false;
-					break;
-				default :
-					if (insideEntity)
-						entity += c;
-					else
+					output.Append (entity.ToString ());
+					entity.Length = 0;
+					entity.Append ('&');
+					continue;
+				}
+
+				if (state == 1) {
+					if (c == ';') {
+						state = 0;
+						output.Append (entity.ToString ());
 						output.Append (c);
-					break;
+						entity.Length = 0;
+					} else {
+						number = 0;
+						if (c != '#') {
+							state = 2;
+						} else {
+							state = 3;
+						}
+						entity.Append (c);
+					}
+				} else if (state == 2) {
+					entity.Append (c);
+					if (c == ';') {
+						string key = entity.ToString ();
+						if (key.Length > 1 && Entities.ContainsKey (key.Substring (1, key.Length - 2)))
+							key = Entities [key.Substring (1, key.Length - 2)].ToString ();
+
+						output.Append (key);
+						state = 0;
+						entity.Length = 0;
+					}
+				} else if (state == 3) {
+					if (c == ';') {
+						if (number > 65535) {
+							output.Append ("&#");
+							output.Append (number.ToString (CultureInfo.InvariantCulture));
+							output.Append (";");
+						} else {
+							output.Append ((char) number);
+						}
+						state = 0;
+						entity.Length = 0;
+						have_trailing_digits = false;
+					} else if (Char.IsDigit (c)) {
+						number = number * 10 + ((int) c - '0');
+						have_trailing_digits = true;
+					} else {
+						state = 2;
+						if (have_trailing_digits) {
+							entity.Append (number.ToString (CultureInfo.InvariantCulture));
+							have_trailing_digits = false;
+						}
+						entity.Append (c);
+					}
 				}
 			}
-			output.Append (entity);
+
+			if (entity.Length > 0) {
+				output.Append (entity.ToString ());
+			} else if (have_trailing_digits) {
+				output.Append (number.ToString (CultureInfo.InvariantCulture));
+			}
 			return output.ToString ();
 		}
 	
@@ -774,13 +857,16 @@ namespace System.Web {
 					output.Append ("&quot;");
 					break;
 				default:
-					if ((int) c > 128) {
+					// MS starts encoding with &# from 160 and stops at 255.
+					// We don't do that. One reason is the 65308/65310 unicode
+					// characters that look like '<' and '>'.
+					if (c > 159) {
 						output.Append ("&#");
-						output.Append (((int) c).ToString ());
+						output.Append (((int) c).ToString (CultureInfo.InvariantCulture));
 						output.Append (";");
-					}
-					else
+					} else {
 						output.Append (c);
+					}
 					break;
 				}
 			return output.ToString ();
@@ -798,7 +884,7 @@ namespace System.Web {
 		}
 
 #if NET_1_1
-		public string UrlPathEncode (string s)
+		public static string UrlPathEncode (string s)
 		{
 			if (s == null)
 				return null;
@@ -818,3 +904,4 @@ namespace System.Web {
 		#endregion // Methods
 	}
 }
+
