@@ -39,8 +39,8 @@ namespace System.Web
 
 		static MimeTypes ()
 		{
-			mimeTypes = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
-						   CaseInsensitiveComparer.Default);
+			mimeTypes = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						   CaseInsensitiveComparer.DefaultInvariant);
 
 			mimeTypes.Add ("3dm", "x-world/x-3dmf");
                 	mimeTypes.Add ("3dmf", "x-world/x-3dmf");
@@ -159,8 +159,8 @@ namespace System.Web
                 	mimeTypes.Add ("html", "text/html");
                 	mimeTypes.Add ("htmls", "text/html");
                 	mimeTypes.Add ("htt", "text/webviewhtml");
-                	mimeTypes.Add ("htx ", "text/html");
-                	mimeTypes.Add ("ice ", "x-conference/x-cooltalk");
+                	mimeTypes.Add ("htx", "text/html");
+                	mimeTypes.Add ("ice", "x-conference/x-cooltalk");
                 	mimeTypes.Add ("ico", "image/x-icon");
                 	mimeTypes.Add ("idc", "text/plain");
                 	mimeTypes.Add ("ief", "image/ief");
@@ -169,42 +169,42 @@ namespace System.Web
                 	mimeTypes.Add ("igs", "application/iges");
                 	mimeTypes.Add ("ima", "application/x-ima");
                 	mimeTypes.Add ("imap", "application/x-httpd-imap");
-                	mimeTypes.Add ("inf ", "application/inf");
+                	mimeTypes.Add ("inf", "application/inf");
                 	mimeTypes.Add ("ins", "application/x-internett-signup");
-                	mimeTypes.Add ("ip ", "application/x-ip2");
+                	mimeTypes.Add ("ip", "application/x-ip2");
                 	mimeTypes.Add ("isu", "video/x-isvideo");
                 	mimeTypes.Add ("it", "audio/it");
                 	mimeTypes.Add ("iv", "application/x-inventor");
                 	mimeTypes.Add ("ivr", "i-world/i-vrml");
                 	mimeTypes.Add ("ivy", "application/x-livescreen");
-                	mimeTypes.Add ("jam ", "audio/x-jam");
+                	mimeTypes.Add ("jam", "audio/x-jam");
                 	mimeTypes.Add ("jav", "text/plain");
                 	mimeTypes.Add ("java", "text/plain");
-                	mimeTypes.Add ("jcm ", "application/x-java-commerce");
+                	mimeTypes.Add ("jcm", "application/x-java-commerce");
                 	mimeTypes.Add ("jfif", "image/jpeg");
                 	mimeTypes.Add ("jfif-tbnl", "image/jpeg");
                 	mimeTypes.Add ("jpe", "image/jpeg");
                 	mimeTypes.Add ("jpeg", "image/jpeg");
                 	mimeTypes.Add ("jps", "image/x-jps");
-                	mimeTypes.Add ("js ", "application/x-javascript");
+                	mimeTypes.Add ("js", "application/x-javascript");
                 	mimeTypes.Add ("jut", "image/jutvision");
                 	mimeTypes.Add ("kar", "audio/midi");
                 	mimeTypes.Add ("ksh", "text/x-script.ksh");
-                	mimeTypes.Add ("la ", "audio/nspaudio");
+                	mimeTypes.Add ("la", "audio/nspaudio");
                 	mimeTypes.Add ("lam", "audio/x-liveaudio");
-                	mimeTypes.Add ("latex ", "application/x-latex");
+                	mimeTypes.Add ("latex", "application/x-latex");
                 	mimeTypes.Add ("list", "text/plain");
                 	mimeTypes.Add ("lma", "audio/nspaudio");
-                	mimeTypes.Add ("log ", "text/plain");
-                	mimeTypes.Add ("lsp ", "application/x-lisp");
-                	mimeTypes.Add ("lst ", "text/plain");
+                	mimeTypes.Add ("log", "text/plain");
+                	mimeTypes.Add ("lsp", "application/x-lisp");
+                	mimeTypes.Add ("lst", "text/plain");
                 	mimeTypes.Add ("lsx", "text/x-la-asf");
                 	mimeTypes.Add ("ltx", "application/x-latex");
                 	mimeTypes.Add ("m", "text/plain");
                 	mimeTypes.Add ("m1v", "video/mpeg");
                 	mimeTypes.Add ("m2a", "audio/mpeg");
                 	mimeTypes.Add ("m2v", "video/mpeg");
-                	mimeTypes.Add ("m3u ", "audio/x-mpequrl");
+                	mimeTypes.Add ("m3u", "audio/x-mpequrl");
                 	mimeTypes.Add ("man", "application/x-troff-man");
                 	mimeTypes.Add ("map", "application/x-navimap");
                 	mimeTypes.Add ("mar", "text/plain");
@@ -213,15 +213,15 @@ namespace System.Web
                 	mimeTypes.Add ("mcd", "application/mcad");
                 	mimeTypes.Add ("mcf", "image/vasa");
                 	mimeTypes.Add ("mcp", "application/netmc");
-                	mimeTypes.Add ("me ", "application/x-troff-me");
+                	mimeTypes.Add ("me", "application/x-troff-me");
                 	mimeTypes.Add ("mht", "message/rfc822");
                 	mimeTypes.Add ("mhtml", "message/rfc822");
                 	mimeTypes.Add ("mid", "audio/midi");
                 	mimeTypes.Add ("midi", "audio/midi");
                 	mimeTypes.Add ("mif", "application/x-mif");
-                	mimeTypes.Add ("mime ", "message/rfc822");
+                	mimeTypes.Add ("mime", "message/rfc822");
                 	mimeTypes.Add ("mjf", "audio/x-vnd.AudioExplosion.MjuiceMediaFile");
-                	mimeTypes.Add ("mjpg ", "video/x-motion-jpeg");
+                	mimeTypes.Add ("mjpg", "video/x-motion-jpeg");
                 	mimeTypes.Add ("mm", "application/base64");
                 	mimeTypes.Add ("mme", "application/base64");
                 	mimeTypes.Add ("mod", "audio/mod");
@@ -266,9 +266,9 @@ namespace System.Web
                 	mimeTypes.Add ("p7m", "application/pkcs7-mime");
                 	mimeTypes.Add ("p7r", "application/x-pkcs7-certreqresp");
                 	mimeTypes.Add ("p7s", "application/pkcs7-signature");
-                	mimeTypes.Add ("part ", "application/pro_eng");
+                	mimeTypes.Add ("part", "application/pro_eng");
                 	mimeTypes.Add ("pas", "text/pascal");
-                	mimeTypes.Add ("pbm ", "image/x-portable-bitmap");
+                	mimeTypes.Add ("pbm", "image/x-portable-bitmap");
                 	mimeTypes.Add ("pcl", "application/x-pcl");
                 	mimeTypes.Add ("pct", "image/x-pict");
                 	mimeTypes.Add ("pcx", "image/x-pcx");
@@ -283,7 +283,7 @@ namespace System.Web
                 	mimeTypes.Add ("pl", "text/plain");
                 	mimeTypes.Add ("plx", "application/x-PiXCLscript");
                 	mimeTypes.Add ("pm", "image/x-xpixmap");
-                	mimeTypes.Add ("pm4 ", "application/x-pagemaker");
+                	mimeTypes.Add ("pm4", "application/x-pagemaker");
                 	mimeTypes.Add ("pm5", "application/x-pagemaker");
                 	mimeTypes.Add ("png", "image/png");
                 	mimeTypes.Add ("pnm", "application/x-portable-anymap");
@@ -298,12 +298,12 @@ namespace System.Web
                 	mimeTypes.Add ("prt", "application/pro_eng");
                 	mimeTypes.Add ("ps", "application/postscript");
                 	mimeTypes.Add ("pvu", "paleovu/x-pv");
-                	mimeTypes.Add ("pwz ", "application/vnd.ms-powerpoint");
-                	mimeTypes.Add ("py ", "text/x-script.phyton");
-                	mimeTypes.Add ("pyc ", "applicaiton/x-bytecode.python");
-                	mimeTypes.Add ("qcp ", "audio/vnd.qcelp");
-                	mimeTypes.Add ("qd3 ", "x-world/x-3dmf");
-                	mimeTypes.Add ("qd3d ", "x-world/x-3dmf");
+                	mimeTypes.Add ("pwz", "application/vnd.ms-powerpoint");
+                	mimeTypes.Add ("py", "text/x-script.phyton");
+                	mimeTypes.Add ("pyc", "applicaiton/x-bytecode.python");
+                	mimeTypes.Add ("qcp", "audio/vnd.qcelp");
+                	mimeTypes.Add ("qd3", "x-world/x-3dmf");
+                	mimeTypes.Add ("qd3d", "x-world/x-3dmf");
                 	mimeTypes.Add ("qif", "image/x-quicktime");
                 	mimeTypes.Add ("qt", "video/quicktime");
                 	mimeTypes.Add ("qtc", "video/x-qtc");
@@ -313,17 +313,17 @@ namespace System.Web
                 	mimeTypes.Add ("ram", "audio/x-pn-realaudio");
                 	mimeTypes.Add ("ras", "application/x-cmu-raster");
                 	mimeTypes.Add ("rast", "image/cmu-raster");
-                	mimeTypes.Add ("rexx ", "text/x-script.rexx");
+                	mimeTypes.Add ("rexx", "text/x-script.rexx");
                 	mimeTypes.Add ("rf", "image/vnd.rn-realflash");
-                	mimeTypes.Add ("rgb ", "image/x-rgb");
+                	mimeTypes.Add ("rgb", "image/x-rgb");
                 	mimeTypes.Add ("rm", "application/vnd.rn-realmedia");
                 	mimeTypes.Add ("rmi", "audio/mid");
-                	mimeTypes.Add ("rmm ", "audio/x-pn-realaudio");
+                	mimeTypes.Add ("rmm", "audio/x-pn-realaudio");
                 	mimeTypes.Add ("rmp", "audio/x-pn-realaudio");
                 	mimeTypes.Add ("rng", "application/ringing-tones");
-                	mimeTypes.Add ("rnx ", "application/vnd.rn-realplayer");
+                	mimeTypes.Add ("rnx", "application/vnd.rn-realplayer");
                 	mimeTypes.Add ("roff", "application/x-troff");
-                	mimeTypes.Add ("rp ", "image/vnd.rn-realpix");
+                	mimeTypes.Add ("rp", "image/vnd.rn-realpix");
                 	mimeTypes.Add ("rpm", "audio/x-pn-realaudio-plugin");
                 	mimeTypes.Add ("rss", "text/xml");
                 	mimeTypes.Add ("rt", "text/richtext");
@@ -331,37 +331,37 @@ namespace System.Web
                 	mimeTypes.Add ("rtx", "text/richtext");
                 	mimeTypes.Add ("rv", "video/vnd.rn-realvideo");
                 	mimeTypes.Add ("s", "text/x-asm");
-                	mimeTypes.Add ("s3m ", "audio/s3m");
-                	mimeTypes.Add ("sbk ", "application/x-tbook");
+                	mimeTypes.Add ("s3m", "audio/s3m");
+                	mimeTypes.Add ("sbk", "application/x-tbook");
                 	mimeTypes.Add ("scm", "application/x-lotusscreencam");
                 	mimeTypes.Add ("sdml", "text/plain");
-                	mimeTypes.Add ("sdp ", "application/sdp");
+                	mimeTypes.Add ("sdp", "application/sdp");
                 	mimeTypes.Add ("sdr", "application/sounder");
                 	mimeTypes.Add ("sea", "application/sea");
                 	mimeTypes.Add ("set", "application/set");
-                	mimeTypes.Add ("sgm ", "text/sgml");
+                	mimeTypes.Add ("sgm", "text/sgml");
                 	mimeTypes.Add ("sgml", "text/sgml");
                 	mimeTypes.Add ("sh", "text/x-script.sh");
                 	mimeTypes.Add ("shar", "application/x-bsh");
-                	mimeTypes.Add ("shtml ", "text/html");
+                	mimeTypes.Add ("shtml", "text/html");
                 	mimeTypes.Add ("sid", "audio/x-psid");
                 	mimeTypes.Add ("sit", "application/x-sit");
                 	mimeTypes.Add ("skd", "application/x-koan");
-                	mimeTypes.Add ("skm ", "application/x-koan");
-                	mimeTypes.Add ("skp ", "application/x-koan");
-                	mimeTypes.Add ("skt ", "application/x-koan");
-                	mimeTypes.Add ("sl ", "application/x-seelogo");
-                	mimeTypes.Add ("smi ", "application/smil");
-                	mimeTypes.Add ("smil ", "application/smil");
+                	mimeTypes.Add ("skm", "application/x-koan");
+                	mimeTypes.Add ("skp", "application/x-koan");
+                	mimeTypes.Add ("skt", "application/x-koan");
+                	mimeTypes.Add ("sl", "application/x-seelogo");
+                	mimeTypes.Add ("smi", "application/smil");
+                	mimeTypes.Add ("smil", "application/smil");
                 	mimeTypes.Add ("snd", "audio/basic");
                 	mimeTypes.Add ("sol", "application/solids");
-                	mimeTypes.Add ("spc ", "application/x-pkcs7-certificates");
+                	mimeTypes.Add ("spc", "application/x-pkcs7-certificates");
                 	mimeTypes.Add ("spl", "application/futuresplash");
                 	mimeTypes.Add ("spr", "application/x-sprite");
-                	mimeTypes.Add ("sprite ", "application/x-sprite");
+                	mimeTypes.Add ("sprite", "application/x-sprite");
                 	mimeTypes.Add ("src", "application/x-wais-source");
                 	mimeTypes.Add ("ssi", "text/x-server-parsed-html");
-                	mimeTypes.Add ("ssm ", "application/streamingmedia");
+                	mimeTypes.Add ("ssm", "application/streamingmedia");
                 	mimeTypes.Add ("sst", "application/vnd.ms-pki.certstore");
                 	mimeTypes.Add ("step", "application/step");
                 	mimeTypes.Add ("stl", "application/sla");
@@ -403,10 +403,10 @@ namespace System.Web
                 	mimeTypes.Add ("vcs", "text/x-vCalendar");
                 	mimeTypes.Add ("vda", "application/vda");
                 	mimeTypes.Add ("vdo", "video/vdo");
-                	mimeTypes.Add ("vew ", "application/groupwise");
+                	mimeTypes.Add ("vew", "application/groupwise");
                 	mimeTypes.Add ("viv", "video/vivo");
                 	mimeTypes.Add ("vivo", "video/vivo");
-                	mimeTypes.Add ("vmd ", "application/vocaltec-media-desc");
+                	mimeTypes.Add ("vmd", "application/vocaltec-media-desc");
                 	mimeTypes.Add ("vmf", "application/vocaltec-media-file");
                 	mimeTypes.Add ("voc", "audio/voc");
                 	mimeTypes.Add ("vos", "video/vosaic");
@@ -418,7 +418,7 @@ namespace System.Web
                 	mimeTypes.Add ("vrt", "x-world/x-vrt");
                 	mimeTypes.Add ("vsd", "application/x-visio");
                 	mimeTypes.Add ("vst", "application/x-visio");
-                	mimeTypes.Add ("vsw ", "application/x-visio");
+                	mimeTypes.Add ("vsw", "application/x-visio");
                 	mimeTypes.Add ("w60", "application/wordperfect6.0");
                 	mimeTypes.Add ("w61", "application/wordperfect6.1");
                 	mimeTypes.Add ("w6w", "application/msword");
@@ -430,13 +430,13 @@ namespace System.Web
                 	mimeTypes.Add ("wk1", "application/x-123");
                 	mimeTypes.Add ("wmf", "windows/metafile");
                 	mimeTypes.Add ("wml", "text/vnd.wap.wml");
-                	mimeTypes.Add ("wmlc ", "application/vnd.wap.wmlc");
+                	mimeTypes.Add ("wmlc", "application/vnd.wap.wmlc");
                 	mimeTypes.Add ("wmls", "text/vnd.wap.wmlscript");
-                	mimeTypes.Add ("wmlsc ", "application/vnd.wap.wmlscriptc");
-                	mimeTypes.Add ("word ", "application/msword");
+                	mimeTypes.Add ("wmlsc", "application/vnd.wap.wmlscriptc");
+                	mimeTypes.Add ("word", "application/msword");
                 	mimeTypes.Add ("wp", "application/wordperfect");
                 	mimeTypes.Add ("wp5", "application/wordperfect");
-                	mimeTypes.Add ("wp6 ", "application/wordperfect");
+                	mimeTypes.Add ("wp6", "application/wordperfect");
                 	mimeTypes.Add ("wpd", "application/wordperfect");
                 	mimeTypes.Add ("wq1", "application/x-lotus");
                 	mimeTypes.Add ("wri", "application/mswrite");
@@ -444,7 +444,7 @@ namespace System.Web
                 	mimeTypes.Add ("wrz", "model/vrml");
                 	mimeTypes.Add ("wsc", "text/scriplet");
                 	mimeTypes.Add ("wsrc", "application/x-wais-source");
-                	mimeTypes.Add ("wtk ", "application/x-wintalk");
+                	mimeTypes.Add ("wtk", "application/x-wintalk");
                 	mimeTypes.Add ("xbm", "image/x-xbitmap");
                 	mimeTypes.Add ("xdr", "video/x-amt-demorun");
                 	mimeTypes.Add ("xgz", "xgl/drawing");
@@ -453,7 +453,7 @@ namespace System.Web
                 	mimeTypes.Add ("xla", "application/excel");
                 	mimeTypes.Add ("xlb", "application/excel");
                 	mimeTypes.Add ("xlc", "application/excel");
-                	mimeTypes.Add ("xld ", "application/excel");
+                	mimeTypes.Add ("xld", "application/excel");
                 	mimeTypes.Add ("xlk", "application/excel");
                 	mimeTypes.Add ("xll", "application/excel");
                 	mimeTypes.Add ("xlm", "application/excel");
