@@ -158,9 +158,7 @@ namespace MonoTests.System.IO
                 	}                	
 		}
 
-                [Test]
-		[ExpectedException (typeof (DirectoryNotFoundException))]
-		public void CtorDirectoryNotFoundException ()
+		private void CtorDirectoryNotFoundException (FileMode mode)
 		{
 			string path = TempFolder + DSC + "thisDirectoryShouldNotExists";
 			if (Directory.Exists (path))
@@ -168,7 +166,7 @@ namespace MonoTests.System.IO
 
 			FileStream stream = null;				
                 	try {
-                		stream = new FileStream (path + DSC + "eitherthisfile.test", FileMode.CreateNew);
+                		stream = new FileStream (path + DSC + "eitherthisfile.test", mode);
                 	} finally {
 
 				if (stream != null)
@@ -179,6 +177,48 @@ namespace MonoTests.System.IO
                 	}                		
 		}
 
+		[Test]
+		[ExpectedException (typeof (DirectoryNotFoundException))]
+		public void CtorDirectoryNotFoundException_CreateNew ()
+		{
+			CtorDirectoryNotFoundException (FileMode.CreateNew);
+		}
+
+		[Test]
+		[ExpectedException (typeof (DirectoryNotFoundException))]
+		public void CtorDirectoryNotFoundException_Create ()
+		{
+			CtorDirectoryNotFoundException (FileMode.Create);
+		}
+
+		[Test]
+		[ExpectedException (typeof (DirectoryNotFoundException))]
+		public void CtorDirectoryNotFoundException_Open ()
+		{
+			CtorDirectoryNotFoundException (FileMode.Open);
+		}
+
+		[Test]
+		[ExpectedException (typeof (DirectoryNotFoundException))]
+		public void CtorDirectoryNotFoundException_OpenOrCreate ()
+		{
+			CtorDirectoryNotFoundException (FileMode.OpenOrCreate);
+		}
+
+		[Test]
+		[ExpectedException (typeof (DirectoryNotFoundException))]
+		public void CtorDirectoryNotFoundException_Truncate ()
+		{
+			CtorDirectoryNotFoundException (FileMode.Truncate);
+		}
+
+		[Test]
+		[ExpectedException (typeof (DirectoryNotFoundException))]
+		public void CtorDirectoryNotFoundException_Append ()
+		{
+			CtorDirectoryNotFoundException (FileMode.Append);
+		}
+
 		[Test]
 		[ExpectedException (typeof (ArgumentOutOfRangeException))]
 		public void CtorArgumentOutOfRangeException3 ()
@@ -330,21 +370,22 @@ namespace MonoTests.System.IO
 		{
 			FileStream fs = null;
 			FileStream fs2 = null;
+			string fn = Path.Combine (TempFolder, "temp");
 			try {
-				if (!File.Exists ("temp")) {
-					TextWriter tw = File.CreateText ("temp");
+				if (!File.Exists (fn)) {
+					TextWriter tw = File.CreateText (fn);
 					tw.Write ("FOO");
 					tw.Close ();
 				}
-				fs = new FileStream ("temp", FileMode.Open, FileAccess.Read);
-				fs2 = new FileStream ("temp", FileMode.Open, FileAccess.Read);
+				fs = new FileStream (fn, FileMode.Open, FileAccess.Read);
+				fs2 = new FileStream (fn, FileMode.Open, FileAccess.Read);
 			} finally {
 				if (fs != null)
 					fs.Close ();
 				if (fs2 != null)
 					fs2.Close ();
-				if (File.Exists ("temp"))
-					File.Delete ("temp");
+				if (File.Exists (fn))
+					File.Delete (fn);
 			}
 		}
 
@@ -352,20 +393,21 @@ namespace MonoTests.System.IO
 		[ExpectedException (typeof (IOException))]
 		public void CtorAccess1Read2Write ()
 		{
+			string fn = Path.Combine (TempFolder, "temp");
 			FileStream fs = null;
 			try {
-				if (!File.Exists ("temp")) {
-					using (TextWriter tw = File.CreateText ("temp")) {
+				if (!File.Exists (fn)) {
+					using (TextWriter tw = File.CreateText (fn)) {
 						tw.Write ("FOO");
 					}
 				}
-				fs = new FileStream ("temp", FileMode.Open, FileAccess.Read);
-				fs = new FileStream ("temp", FileMode.Create, FileAccess.Write);
+				fs = new FileStream (fn, FileMode.Open, FileAccess.Read);
+				fs = new FileStream (fn, FileMode.Create, FileAccess.Write);
 			} finally {
 				if (fs != null)
 					fs.Close ();
-				if (File.Exists ("temp"))
-					File.Delete ("temp");
+				if (File.Exists (fn))
+					File.Delete (fn);
 			}
 		}
 
@@ -373,20 +415,34 @@ namespace MonoTests.System.IO
 		[ExpectedException (typeof (IOException))]
 		public void CtorAccess1Write2Write ()
 		{
+			string fn = Path.Combine (TempFolder, "temp");
 			FileStream fs = null;
 			try {
-				if (File.Exists ("temp"))
-					File.Delete ("temp");
-				fs = new FileStream ("temp", FileMode.Create, FileAccess.Write);
-				fs = new FileStream ("temp", FileMode.Create, FileAccess.Write);
+				if (File.Exists (fn))
+					File.Delete (fn);
+				fs = new FileStream (fn, FileMode.Create, FileAccess.Write);
+				fs = new FileStream (fn, FileMode.Create, FileAccess.Write);
 			} finally {
 				if (fs != null)
 					fs.Close ();
-				if (File.Exists ("temp"))
-					File.Delete ("temp");
+				if (File.Exists (fn))
+					File.Delete (fn);
 			}
 		}
 
+		[Test]
+		[ExpectedException (typeof (UnauthorizedAccessException))]
+		public void CtorReadDirectoryAsFile ()
+		{
+			FileStream stream = null;
+			try {
+				stream = new FileStream (TempFolder, FileMode.Open, FileAccess.Read);
+                	} finally {
+				if (stream != null)
+					stream.Close ();
+                	}
+		}
+
 		[Test]
 		public void Write ()
 		{
@@ -517,7 +573,41 @@ namespace MonoTests.System.IO
                 	stream.Close ();
 			DeleteFile (path);
                 }
-                
+
+		[Category("NotWorking")]
+		// Bug: 71371 -> duplicate and WONTFIX.
+		public void TestLock_FailsOnMono ()
+		{
+			string path = TempFolder + Path.DirectorySeparatorChar + "TestLock";
+                	DeleteFile (path);
+
+                	FileStream stream = new FileStream (path, FileMode.CreateNew, FileAccess.ReadWrite);
+                	                	
+	               	stream.Write (new Byte [] {0,1,2,3,4,5,6,7,8,9,10}, 0, 10);                              	
+                	stream.Close ();
+
+                	stream = new FileStream (path, FileMode.Open, FileAccess.ReadWrite);
+                	
+                	stream.Lock (0, 5);
+                	
+                	FileStream stream2 = new FileStream (path , FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
+                	
+                	byte [] bytes = new byte [5];
+                	try {                		
+                		stream2.Read (bytes, 0, 5);
+                		Fail ();
+                	} catch (Exception e) {
+                		
+                		// locked
+                		AssertEquals ("test#01", typeof (IOException), e.GetType ());
+                	}
+
+					stream.Close ();
+                	stream2.Close ();
+                	
+                	DeleteFile (path);                		
+		}
+		    
                 public void TestLock()
                 {
 			string path = TempFolder + Path.DirectorySeparatorChar + "TestLock";
@@ -541,7 +631,9 @@ namespace MonoTests.System.IO
                 	} catch (Exception e) {
                 		
                 		// locked
-                		AssertEquals ("test#01", typeof (IOException), e.GetType ());
+                		// AssertEquals ("test#01", typeof (IOException), e.GetType ());
+				//
+				// Moved into the previous test case.
                 	}
                		                
                 	stream2.Seek (5, SeekOrigin.Begin);
@@ -945,7 +1037,7 @@ namespace MonoTests.System.IO
 		[ExpectedException (typeof (ArgumentException))]
 		public void Constructor_InvalidFileHandle () 
 		{
-			new FileStream ((IntPtr)(-1), FileAccess.Read);
+			new FileStream ((IntPtr)(-1L), FileAccess.Read);
 		}
 
 		[Test]
@@ -1044,11 +1136,16 @@ namespace MonoTests.System.IO
 		[ExpectedException (typeof (NotSupportedException))]
 		public void SetLengthWithClosedBaseStream ()
 		{
-			FileStream fs = new FileStream ("temp", FileMode.Create);
-			BufferedStream bs = new BufferedStream (fs);
-			fs.Close ();
-			
-			bs.SetLength (1000);
+			string fn = Path.Combine (TempFolder, "temp");
+			try {
+				FileStream fs = new FileStream (fn, FileMode.Create);
+				BufferedStream bs = new BufferedStream (fs);
+				fs.Close ();
+				
+				bs.SetLength (1000);
+			} finally {
+				File.Delete (fn);
+			}
 		}
         }
 }
