@@ -16,6 +16,7 @@ using System.Reflection.Emit;
 using System.Runtime.CompilerServices;
 using System.Security;
 using System.Security.Permissions;
+using System.Runtime.InteropServices;
 
 using NUnit.Framework;
 
@@ -58,6 +59,7 @@ public class MethodBuilderTest : Assertion
 		return "class" + (typeIndexer ++);
 	}
 
+	[Test]
 	public void TestAttributes () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), MethodAttributes.Public, typeof (void), new Type [0]);
@@ -66,6 +68,7 @@ public class MethodBuilderTest : Assertion
 					  MethodAttributes.Public, mb.Attributes);
 	}
 
+	[Test]
 	public void TestCallingConvention () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type[0]);
@@ -87,6 +90,7 @@ public class MethodBuilderTest : Assertion
 					  mb4.CallingConvention);
 	}
 
+	[Test]
 	public void TestDeclaringType () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type[0]);
@@ -95,6 +99,7 @@ public class MethodBuilderTest : Assertion
 					  genClass, mb.DeclaringType);
 	}
 
+	[Test]
 	public void TestInitLocals () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type[0]);
@@ -103,18 +108,28 @@ public class MethodBuilderTest : Assertion
 		mb.InitLocals = false;
 		Assert ("InitLocals is settable", !mb.InitLocals);
 	}
-
-	public void TestMethodHandle () {
+	
+	[Test]
+	[ExpectedException (typeof(NotSupportedException))]
+	public void TestMethodHandleIncomplete () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type [0]);
 
-		try {
-			RuntimeMethodHandle handle = mb.MethodHandle;
-			Fail ();
-		} catch (NotSupportedException) {
-		}
+		RuntimeMethodHandle handle = mb.MethodHandle;
 	}
 
+	[Test]
+	[ExpectedException (typeof(NotSupportedException))]
+	public void TestMethodHandleComplete () {
+		MethodBuilder mb = genClass.DefineMethod (
+			genMethodName (), 0, typeof (void), new Type [0]);
+		mb.CreateMethodBody (new byte[2], 0);
+		genClass.CreateType ();
+
+		RuntimeMethodHandle handle = mb.MethodHandle;
+	}
+
+	[Test]
 	public void TestName () {
 		string name = genMethodName ();
 		MethodBuilder mb = genClass.DefineMethod (
@@ -123,6 +138,7 @@ public class MethodBuilderTest : Assertion
 		AssertEquals ("Name works", name, mb.Name);
 	}
 
+	[Test]
 	public void TestReflectedType () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type [0]);
@@ -131,6 +147,7 @@ public class MethodBuilderTest : Assertion
 					  genClass, mb.ReflectedType);
 	}
 
+	[Test]
 	public void TestReturnType () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (Console), new Type [0]);
@@ -145,6 +162,7 @@ public class MethodBuilderTest : Assertion
 		Assert ("void ReturnType works", (mb2.ReturnType == null) || (mb2.ReturnType == typeof (void)));
 	}
 
+	[Test]
 	public void TestReturnTypeCustomAttributes () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (Console), new Type [0]);
@@ -162,6 +180,7 @@ public class MethodBuilderTest : Assertion
 	}
 	*/
 
+	[Test]
 	public void TestCreateMethodBody () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type [0]);
@@ -217,6 +236,7 @@ public class MethodBuilderTest : Assertion
 			new Type[2] {
 			typeof(int), typeof(int)
 		});
+		mb.CreateMethodBody (new byte[2], 0);
 		tb.CreateType ();
 		mb.DefineParameter (-5, ParameterAttributes.None, "param1");
 	}
@@ -231,10 +251,12 @@ public class MethodBuilderTest : Assertion
 			new Type[2] {
 			typeof(int), typeof(int)
 		});
+		mb.CreateMethodBody (new byte[2], 0);
 		tb.CreateType ();
 		mb.DefineParameter (1, ParameterAttributes.None, "param1");
 	}
 
+	[Test]
 	public void TestDefineParameter () {
 		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
 		MethodBuilder mb = tb.DefineMethod (
@@ -288,6 +310,7 @@ public class MethodBuilderTest : Assertion
 			methodName.GetHashCode (), mb.GetHashCode ());
 	}
 
+	[Test]
 	public void TestGetBaseDefinition () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type [0]);
@@ -296,6 +319,7 @@ public class MethodBuilderTest : Assertion
 					  mb.GetBaseDefinition (), mb);
 	}
 
+	[Test]
 	public void TestGetILGenerator () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type [0]);
@@ -324,6 +348,7 @@ public class MethodBuilderTest : Assertion
 		}
 	}
 
+	[Test]
 	public void TestMethodImplementationFlags () {
 		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
 		MethodBuilder mb = tb.DefineMethod (
@@ -351,6 +376,7 @@ public class MethodBuilderTest : Assertion
 		}
 	}
 
+	[Test]
 	public void TestGetModule () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), new Type [0]);
@@ -359,6 +385,7 @@ public class MethodBuilderTest : Assertion
 					  mb.GetModule ());
 	}
 
+	[Test]
 	public void TestGetParameters () {
 		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
 		MethodBuilder mb = tb.DefineMethod (
@@ -380,6 +407,7 @@ public class MethodBuilderTest : Assertion
 		*/
 	}
 
+	[Test]
 	public void TestGetToken () {
 		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
 		MethodBuilder mb = tb.DefineMethod (
@@ -388,6 +416,7 @@ public class MethodBuilderTest : Assertion
 		mb.GetToken ();
 	}
 
+	[Test]
 	public void TestInvoke () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), 
@@ -406,23 +435,21 @@ public class MethodBuilderTest : Assertion
 		}
 	}
 
+	[Test]
+	[ExpectedException (typeof (NotSupportedException))]
 	public void TestIsDefined () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), 
 			new Type [1] {typeof(int)});
-
-		try {
-			mb.IsDefined (null, true);
-			Fail ();
-		} catch (NotSupportedException) {
-		}
+		mb.IsDefined (null, true);
 	}
 
+	[Test]
 	public void TestGetCustomAttributes () {
 		MethodBuilder mb = genClass.DefineMethod (
 			genMethodName (), 0, typeof (void), 
 			new Type [1] {typeof(int)});
-
+		
 		try {
 			mb.GetCustomAttributes (true);
 			Fail ();
@@ -436,6 +463,7 @@ public class MethodBuilderTest : Assertion
 		}
 	}
 
+	[Test]
 	public void TestSetCustomAttribute () {
 		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
 		string name = genMethodName ();
@@ -553,18 +581,25 @@ public class MethodBuilderTest : Assertion
 
 		MethodBuilder mb = genClass.DefineMethod (
 			mname, MethodAttributes.Public, typeof (void), 
-			new Type [] { typeof (string) });
-		ParameterBuilder pb = mb.DefineParameter (1, 0, "foo");
+			new Type [] { typeof (int), typeof (string) });
+		ParameterBuilder pb = mb.DefineParameter (1, ParameterAttributes.In, "foo");
+		pb.SetConstant (52);
 		pb.SetCustomAttribute (new CustomAttributeBuilder (typeof (ParamAttribute).GetConstructors () [0], new object [] { }));
+		ParameterBuilder pb2 = mb.DefineParameter (2, 0, "bar");
+		pb2.SetConstant ("foo");
 		mb.GetILGenerator ().Emit (OpCodes.Ret);
 
 		Type t = genClass.CreateType ();
 		MethodInfo m = t.GetMethod (mname);
-		ParameterInfo pi = m.GetParameters ()[0];
+		ParameterInfo[] pi = m.GetParameters ();
 
-		AssertEquals ("foo", pi.Name);
+		AssertEquals ("foo", pi [0].Name);
+		AssertEquals (true, pi [0].IsIn);
+		AssertEquals (52, pi [0].DefaultValue);
+		object[] cattrs = pi [0].GetCustomAttributes (true);
 
-		object[] cattrs = pi.GetCustomAttributes (true);
+		AssertEquals ("foo", pi [1].DefaultValue);
+		
 
 		/* This test does not run under MS.NET: */
 		/*
@@ -572,5 +607,102 @@ public class MethodBuilderTest : Assertion
 		  AssertEquals (typeof (ParamAttribute), cattrs [0].GetType ());
 		*/
 	}
+
+#if NET_2_0
+	[Test]
+	public void SetCustomAttribute_DllImport1 () {
+		string mname = genMethodName ();
+
+		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+		MethodBuilder mb = tb.DefineMethod (
+			mname, MethodAttributes.Public, typeof (void), 
+			new Type [] { typeof (int), typeof (string) });
+
+		// Create an attribute with default values
+		mb.SetCustomAttribute (new CustomAttributeBuilder(typeof(DllImportAttribute).GetConstructor(new Type[] { typeof(string) }), new object[] { "kernel32" }));
+
+		Type t = tb.CreateType ();
+
+		DllImportAttribute attr = (DllImportAttribute)((t.GetMethod (mname).GetCustomAttributes (typeof (DllImportAttribute), true)) [0]);
+
+		AssertEquals (CallingConvention.Winapi, attr.CallingConvention);
+		AssertEquals (mname, attr.EntryPoint);
+		AssertEquals ("kernel32", attr.Value);
+		AssertEquals (false, attr.ExactSpelling);
+		AssertEquals (true, attr.PreserveSig);
+		AssertEquals (false, attr.SetLastError);
+		AssertEquals (false, attr.BestFitMapping);
+		AssertEquals (false, attr.ThrowOnUnmappableChar);
+	}
+
+	[Test]
+	public void SetCustomAttribute_DllImport2 () {
+		string mname = genMethodName ();
+
+		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+		MethodBuilder mb = tb.DefineMethod (
+			mname, MethodAttributes.Public, typeof (void), 
+			new Type [] { typeof (int), typeof (string) });
+
+		CustomAttributeBuilder cb = new CustomAttributeBuilder (typeof (DllImportAttribute).GetConstructor (new Type [] {typeof (String)}), new object [] { "foo" }, new FieldInfo [] {typeof (DllImportAttribute).GetField ("EntryPoint"), typeof (DllImportAttribute).GetField ("CallingConvention"), typeof (DllImportAttribute).GetField ("CharSet"), typeof (DllImportAttribute).GetField ("ExactSpelling"), typeof (DllImportAttribute).GetField ("PreserveSig")}, new object [] { "bar", CallingConvention.StdCall, CharSet.Unicode, true, false });
+		mb.SetCustomAttribute (cb);
+
+		Type t = tb.CreateType ();
+
+		DllImportAttribute attr = (DllImportAttribute)((t.GetMethod (mname).GetCustomAttributes (typeof (DllImportAttribute), true)) [0]);
+
+		AssertEquals (CallingConvention.StdCall, attr.CallingConvention);
+		AssertEquals (CharSet.Unicode, attr.CharSet);
+		AssertEquals ("bar", attr.EntryPoint);
+		AssertEquals ("foo", attr.Value);
+		AssertEquals (true, attr.ExactSpelling);
+		AssertEquals (false, attr.PreserveSig);
+		AssertEquals (false, attr.SetLastError);
+		AssertEquals (false, attr.BestFitMapping);
+		AssertEquals (false, attr.ThrowOnUnmappableChar);
+	}
+
+	[Test]
+	public void SetCustomAttribute_DllImport3 () {
+		string mname = genMethodName ();
+
+		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+		MethodBuilder mb = tb.DefineMethod (
+			mname, MethodAttributes.Public, typeof (void), 
+			new Type [] { typeof (int), typeof (string) });
+
+		// Test attributes with three values (on/off/missing)
+		CustomAttributeBuilder cb = new CustomAttributeBuilder (typeof (DllImportAttribute).GetConstructor (new Type [] {typeof (String)}), new object [] { "foo" }, new FieldInfo [] { typeof (DllImportAttribute).GetField ("BestFitMapping"), typeof (DllImportAttribute).GetField ("ThrowOnUnmappableChar")}, new object [] { false, false });
+		mb.SetCustomAttribute (cb);
+
+		Type t = tb.CreateType ();
+
+		DllImportAttribute attr = (DllImportAttribute)((t.GetMethod (mname).GetCustomAttributes (typeof (DllImportAttribute), true)) [0]);
+
+		AssertEquals (false, attr.BestFitMapping);
+		AssertEquals (false, attr.ThrowOnUnmappableChar);
+	}
+
+	[Test]
+	public void SetCustomAttribute_DllImport4 () {
+		string mname = genMethodName ();
+
+		TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+		MethodBuilder mb = tb.DefineMethod (
+			mname, MethodAttributes.Public, typeof (void), 
+			new Type [] { typeof (int), typeof (string) });
+
+		CustomAttributeBuilder cb = new CustomAttributeBuilder (typeof (DllImportAttribute).GetConstructor (new Type [] {typeof (String)}), new object [] { "foo" }, new FieldInfo [] { typeof (DllImportAttribute).GetField ("SetLastError"), typeof (DllImportAttribute).GetField ("BestFitMapping"), typeof (DllImportAttribute).GetField ("ThrowOnUnmappableChar")}, new object [] { true, true, true });
+		mb.SetCustomAttribute (cb);
+
+		Type t = tb.CreateType ();
+
+		DllImportAttribute attr = (DllImportAttribute)((t.GetMethod (mname).GetCustomAttributes (typeof (DllImportAttribute), true)) [0]);
+
+		AssertEquals (true, attr.SetLastError);
+		AssertEquals (true, attr.BestFitMapping);
+		AssertEquals (true, attr.ThrowOnUnmappableChar);
+	}
+#endif
 }
 }
